﻿local E = AleaUI
local Skins = E:Module("Skins")
local AB = E:Module('ActionBars')
local L = E.L

local _G = _G
local type, error, tostring, tonumber, assert, select = type, error, tostring, tonumber, assert, select
local setmetatable, wipe, unpack, pairs, next = setmetatable, wipe, unpack, pairs, next
local str_match, format, tinsert, tremove = string.match, format, tinsert, tremove
local gsub = string.gsub
local split = string.split
local ceil = math.ceil;
local ceil = math.ceil;
local lower = string.lower;

local CooldownFrame_SetTimer = CooldownFrame_Set or CooldownFrame_SetTimer

local lib = {}

local DD_points = {
	["TOPLEFT"] = "TOPLEFT",
	["TOPRIGHT"] = "TOPRIGHT",
	["BOTTOMLEFT"] = "BOTTOMLEFT",
	["BOTTOMRIGHT"] = "BOTTOMRIGHT",
}

local options = {
	['font'] = E.media.default_font_name,
	['fontSize'] = 10,
	['fontOutline'] = 'OUTLINE',

	["macrotext"] = true,
	["hotkeytext"] = true,
	['showGrid'] = true,

	['noRangeColor'] = { r = 0.8, g = 0.1, b = 0.1 },
	['noPowerColor'] = { r = 0.5, g = 0.5, b = 1 },

	['keyDown'] = false,
	['movementModifier'] = 'SHIFT',

	['microbar'] = {
		['enabled'] = false,
		['mouseover'] = false,
		['buttonsPerRow'] = 12,
		['alpha'] = 1,
	},
	['bar1'] = {
		['enabled'] = true,
		['buttons'] = 12,
		['mouseover'] = false,
		['buttonsPerRow'] = 12,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {
			["DRUID"] = "[bonusbar:1,nostealth] 7; [bonusbar:1,stealth] 7; [bonusbar:2] 8; [bonusbar:3] 9; [bonusbar:4] 10;",
			["PRIEST"] = "[bonusbar:1] 7;",
			["ROGUE"] = "[stance:1] 7;  [stance:2] 7; [stance:3] 7;", -- set to "[stance:1] 7; [stance:3] 10;" if you want a shadow dance bar
			["MONK"] = "[bonusbar:1] 7; [bonusbar:2] 8; [bonusbar:3] 9;",
			["WARRIOR"] = "[bonusbar:1] 7; [bonusbar:2] 8;"
		},
		['visibility'] = "[petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['bar2'] = {
		['enabled'] = true,
		['mouseover'] = false,
		['buttons'] = 12,
		['buttonsPerRow'] = 12,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {},
		['visibility'] = "[vehicleui] hide; [overridebar] hide; [petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['bar3'] = {
		['enabled'] = true,
		['mouseover'] = false,
		['buttons'] = 12,
		['buttonsPerRow'] = 12,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {},
		['visibility'] = "[vehicleui] hide; [overridebar] hide; [petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['bar4'] = {
		['enabled'] = false,
		['mouseover'] = false,
		['buttons'] = 12,
		['buttonsPerRow'] = 1,
		['point'] = 'TOPRIGHT',
		['backdrop'] = true,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {},
		['visibility'] = "[vehicleui] hide; [overridebar] hide; [petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['bar5'] = {
		['enabled'] = false,
		['mouseover'] = false,
		['buttons'] = 6,
		['buttonsPerRow'] = 6,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {},
		['visibility'] = "[vehicleui] hide; [overridebar] hide; [petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['bar6'] = {
		['enabled'] = false,
		['mouseover'] = false,
		['buttons'] = 12,
		['buttonsPerRow'] = 12,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['paging'] = {},
		['visibility'] = "[vehicleui] hide; [overridebar] hide; [petbattle] hide; show",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['barPet'] = {
		['enabled'] = true,
		['mouseover'] = false,
		['buttons'] = NUM_PET_ACTION_SLOTS,
		['buttonsPerRow'] = 1,
		['point'] = 'TOPRIGHT',
		['backdrop'] = true,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		['visibility'] = "[petbattle] hide;[pet,novehicleui,nooverridebar,nopossessbar] show;hide",
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['stanceBar'] = {
		['enabled'] = true,
		['style'] = 'darkenInactive',
		['mouseover'] = false,
		['buttonsPerRow'] = NUM_STANCE_SLOTS,
		['buttons'] = NUM_STANCE_SLOTS,
		['point'] = 'TOPLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 28,
		["buttonspacing"] = 2,
		['alpha'] = 1,
		["border"] = {
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
		["artwork"] = {
			['enable'] = false,
			['width'] = 1,
			['height'] = 1,
			["background_texture"] = AleaUI.media.default_bar_texture_name3,
			["size"] = 1,
			["inset"] = 0,
			["color"] = {
				0,  
				0,  
				0,  
				1,  
			},
			["background_inset"] = 0,
			["background_color"] = {
				0,  
				0,  
				0,  
				0.6,  
			},
			["texture"] = AleaUI.media.default_bar_texture_name3,
		},
	},
	['extraActionButton'] = {
		['alpha'] = 1,
	},
};

E.default_settings.actionbars = options

lib.eventFrame = CreateFrame("Frame")
lib.eventFrame:UnregisterAllEvents()

AleaUIButtonsCore = lib

lib.buttonRegistry = lib.buttonRegistry or {}
lib.activeButtons = lib.activeButtons or {}
lib.actionButtons = lib.actionButtons or {}
lib.nonActionButtons = lib.nonActionButtons or {}

lib.ChargeCooldowns = lib.ChargeCooldowns or {}
lib.NumChargeCooldowns = lib.NumChargeCooldowns or 0

lib.ACTION_HIGHLIGHT_MARKS = lib.ACTION_HIGHLIGHT_MARKS or setmetatable({}, { __index = ACTION_HIGHLIGHT_MARKS })

--lib.callbacks = lib.callbacks or CBH:New(lib)

local Generic = CreateFrame("CheckButton")
local Generic_MT = {__index = Generic}

local Action = setmetatable({}, {__index = Generic})
local Action_MT = {__index = Action}

local PetAction = setmetatable({}, {__index = Generic})
local PetAction_MT = {__index = PetAction}

local Spell = setmetatable({}, {__index = Generic})
local Spell_MT = {__index = Spell}

local Item = setmetatable({}, {__index = Generic})
local Item_MT = {__index = Item}

local Macro = setmetatable({}, {__index = Generic})
local Macro_MT = {__index = Macro}

local Custom = setmetatable({}, {__index = Generic})
local Custom_MT = {__index = Custom}

local type_meta_map = {
	empty  = Generic_MT,
	action = Action_MT,
	--pet    = PetAction_MT,
	spell  = Spell_MT,
	item   = Item_MT,
	macro  = Macro_MT,
	custom = Custom_MT
}

local ButtonRegistry, ActiveButtons, ActionButtons, NonActionButtons = lib.buttonRegistry, lib.activeButtons, lib.actionButtons, lib.nonActionButtons

local Update, UpdateButtonState, UpdateUsable, UpdateCount, UpdateCooldown, UpdateTooltip, UpdateNewAction
local StartFlash, StopFlash, UpdateFlash, UpdateHotkeys, UpdateRangeTimer, UpdateOverlayGlow
local UpdateFlyout, ShowGrid, HideGrid, UpdateGrid, SetupSecureSnippets, WrapOnClick
local ShowOverlayGlow, HideOverlayGlow
local EndChargeCooldown

local InitializeEventHandler, OnEvent, ForAllButtons, OnUpdate


local SPELL_POWER_HOLY_POWER = SPELL_POWER_HOLY_POWER;
local HAND_OF_LIGHT = GetSpellInfo(90174);
local DIVINE_CRUSADER = GetSpellInfo(144595)
local PLAYERCLASS = select(2, UnitClass('player'))
local HOLY_POWER_SPELLS = {
	[85256] = GetSpellInfo(85256), --Templar's Verdict
	[53385] = GetSpellInfo(53385), --Divine Storm
	[53600] = GetSpellInfo(53600), --Shield of the Righteous
	[157048] = GetSpellInfo(157048), -- Final Verdict
	[152262] = GetSpellInfo(152262), --Seraphim
};

local DefaultConfig = {
	outOfRangeColoring = "button",
	tooltip = "enabled",
	showGrid = false,
	colors = {
		range = { 0.8, 0.1, 0.1 },
		mana = { 0.5, 0.5, 1.0 },
		hp = { 0.5, 0.5, 1.0 }
	},
	hideElements = {
		macro = false,
		hotkey = false,
		equipped = false,
	},
	keyBoundTarget = false,
	clickOnDown = false,
	flyoutDirection = "UP",
}

--- Create a new action button.
-- @param id Internal id of the button (not used by LibActionButton-1.0, only for tracking inside the calling addon)
-- @param name Name of the button frame to be created (not used by LibActionButton-1.0 aside from naming the frame)
-- @param header Header that drives these action buttons (if any)
function lib:CreateButton(id, name, header, config)
	if type(name) ~= "string" then
		error("Usage: CreateButton(id, name. header): Buttons must have a valid name!", 2)
	end
	if not header then
		error("Usage: CreateButton(id, name, header): Buttons without a secure header are not yet supported!", 2)
	end

	local button = setmetatable(CreateFrame("CheckButton", name, header, "SecureActionButtonTemplate, ActionButtonTemplate"), Generic_MT)
	button:RegisterForDrag("LeftButton", "RightButton")
	button:RegisterForClicks("AnyUp")

	lib.NumChargeCooldowns = lib.NumChargeCooldowns + 1
	
	local cooldown = CreateFrame("Cooldown", "AleaUIChargeCooldown"..lib.NumChargeCooldowns, button, "CooldownFrameTemplate");
	cooldown:SetScript("OnCooldownDone", EndChargeCooldown)
	cooldown:SetHideCountdownNumbers(true)
	cooldown:SetDrawEdge(true)
	cooldown:SetDrawSwipe(false)
	cooldown:SetBlingTexture('')
	
	cooldown:SetParent(button)
	cooldown:SetAllPoints(button)
	cooldown:SetFrameStrata("TOOLTIP")
	cooldown:Show()
	button.chargeCooldown = cooldown
	cooldown.parent = button
		
		
		
	-- Frame Scripts
	button:SetScript("OnEnter", Generic.OnEnter)
	button:SetScript("OnLeave", Generic.OnLeave)
	button:SetScript("PreClick", Generic.PreClick)
	button:SetScript("PostClick", Generic.PostClick)

	button.id = id
	button.header = header
	-- Mapping of state -> action
	button.state_types = {}
	button.state_actions = {}

	-- Store the LAB Version that created this button for debugging
	button.__LAB_Version = MINOR_VERSION

	-- just in case we're not run by a header, default to state 0
	button:SetAttribute("state", 0)

	SetupSecureSnippets(button)
	WrapOnClick(button)

	-- adjust hotkey style for better readability
	button.HotKey:SetFont(button.HotKey:GetFont(), 13, "OUTLINE")
	button.HotKey:SetVertexColor(0.75, 0.75, 0.75)

	-- adjust count/stack size
	button.Count:SetFont(button.Count:GetFont(), 16, "OUTLINE")

	if _G[name.."Cooldown"] then
		_G[name.."Cooldown"]:SetBlingTexture('') --<BlingTexture file="Interface\Cooldown\star4">
	end
	
	-- Store the button in the registry, needed for event and OnUpdate handling
	if not next(ButtonRegistry) then
		InitializeEventHandler()
	end
	ButtonRegistry[button] = true

	button:UpdateConfig(config)

	-- run an initial update
	button:UpdateAction()
	UpdateHotkeys(button)

	-- somewhat of a hack for the Flyout buttons to not error.
	button.action = 0

--	lib.callbacks:Fire("OnButtonCreated", button)

	return button
end

function SetupSecureSnippets(button)
	button:SetAttribute("_custom", Custom.RunCustom)
	-- secure UpdateState(self, state)
	-- update the type and action of the button based on the state
	button:SetAttribute("UpdateState", [[
		local state = ...
		self:SetAttribute("state", state)
		local type, action = (self:GetAttribute(format("labtype-%s", state)) or "empty"), self:GetAttribute(format("labaction-%s", state))

		self:SetAttribute("type", type)
		if type ~= "empty" and type ~= "custom" then
			local action_field = (type == "pet") and "action" or type
			self:SetAttribute(action_field, action)
			self:SetAttribute("action_field", action_field)
		end
		local onStateChanged = self:GetAttribute("OnStateChanged")
		if onStateChanged then
			self:Run(onStateChanged, state, type, action)
		end
	]])

	-- this function is invoked by the header when the state changes
	button:SetAttribute("_childupdate-state", [[
		self:RunAttribute("UpdateState", message)
		self:CallMethod("UpdateAction")
	]])

	-- secure PickupButton(self, kind, value, ...)
	-- utility function to place a object on the cursor
	button:SetAttribute("PickupButton", [[
		local kind, value = ...
		if kind == "empty" then
			return "clear"
		elseif kind == "action" or kind == "pet" then
			local actionType = (kind == "pet") and "petaction" or kind
			return actionType, value
		elseif kind == "spell" or kind == "item" or kind == "macro" then
			return "clear", kind, value
		else
			print("LibActionButton-1.0: Unknown type: " .. tostring(kind))
			return false
		end
	]])

	button:SetAttribute("OnDragStart", [[
		if (self:GetAttribute("buttonlock") and not IsModifiedClick("PICKUPACTION")) or self:GetAttribute("LABdisableDragNDrop") then return false end
		local state = self:GetAttribute("state")
		local type = self:GetAttribute("type")
		-- if the button is empty, we can't drag anything off it
		if type == "empty" or type == "custom" then
			return false
		end
		-- Get the value for the action attribute
		local action_field = self:GetAttribute("action_field")
		local action = self:GetAttribute(action_field)

		-- non-action fields need to change their type to empty
		if type ~= "action" and type ~= "pet" then
			self:SetAttribute(format("labtype-%s", state), "empty")
			self:SetAttribute(format("labaction-%s", state), nil)
			-- update internal state
			self:RunAttribute("UpdateState", state)
			-- send a notification to the insecure code
			self:CallMethod("ButtonContentsChanged", state, "empty", nil)
		end
		-- return the button contents for pickup
		return self:RunAttribute("PickupButton", type, action)
	]])

	button:SetAttribute("OnReceiveDrag", [[
		if self:GetAttribute("LABdisableDragNDrop") then return false end
		local kind, value, subtype, extra = ...
		if not kind or not value then return false end
		local state = self:GetAttribute("state")
		local buttonType, buttonAction = self:GetAttribute("type"), nil
		if buttonType == "custom" then return false end
		-- action buttons can do their magic themself
		-- for all other buttons, we'll need to update the content now
		if buttonType ~= "action" and buttonType ~= "pet" then
			-- with "spell" types, the 4th value contains the actual spell id
			if kind == "spell" then
				if extra then
					value = extra
				else
					print("no spell id?", ...)
				end
			elseif kind == "item" and value then
				value = format("item:%d", value)
			end

			-- Get the action that was on the button before
			if buttonType ~= "empty" then
				buttonAction = self:GetAttribute(self:GetAttribute("action_field"))
			end

			-- TODO: validate what kind of action is being fed in here
			-- We can only use a handful of the possible things on the cursor
			-- return false for all those we can't put on buttons

			self:SetAttribute(format("labtype-%s", state), kind)
			self:SetAttribute(format("labaction-%s", state), value)
			-- update internal state
			self:RunAttribute("UpdateState", state)
			-- send a notification to the insecure code
			self:CallMethod("ButtonContentsChanged", state, kind, value)
		else
			-- get the action for (pet-)action buttons
			buttonAction = self:GetAttribute("action")
		end
		return self:RunAttribute("PickupButton", buttonType, buttonAction)
	]])

	button:SetScript("OnDragStart", nil)
	-- Wrapped OnDragStart(self, button, kind, value, ...)
	button.header:WrapScript(button, "OnDragStart", [[
		return self:RunAttribute("OnDragStart")
	]])
	-- Wrap twice, because the post-script is not run when the pre-script causes a pickup (doh)
	-- we also need some phony message, or it won't work =/
	button.header:WrapScript(button, "OnDragStart", [[
		return "message", "update"
	]], [[
		self:RunAttribute("UpdateState", self:GetAttribute("state"))
	]])

	button:SetScript("OnReceiveDrag", nil)
	-- Wrapped OnReceiveDrag(self, button, kind, value, ...)
	button.header:WrapScript(button, "OnReceiveDrag", [[
		return self:RunAttribute("OnReceiveDrag", kind, value, ...)
	]])
	-- Wrap twice, because the post-script is not run when the pre-script causes a pickup (doh)
	-- we also need some phony message, or it won't work =/
	button.header:WrapScript(button, "OnReceiveDrag", [[
		return "message", "update"
	]], [[
		self:RunAttribute("UpdateState", self:GetAttribute("state"))
	]])
end

function WrapOnClick(button)
	-- Wrap OnClick, to catch changes to actions that are applied with a click on the button.
	button.header:WrapScript(button, "OnClick", [[
		if self:GetAttribute("type") == "action" then
			local type, action = GetActionInfo(self:GetAttribute("action"))
			return nil, format("%s|%s", tostring(type), tostring(action))
		end
	]], [[
		local type, action = GetActionInfo(self:GetAttribute("action"))
		if message ~= format("%s|%s", tostring(type), tostring(action)) then
			self:RunAttribute("UpdateState", self:GetAttribute("state"))
		end
	]])
end

-----------------------------------------------------------
--- utility

function lib:GetAllButtons()
	local buttons = {}
	for button in next, ButtonRegistry do
		buttons[button] = true
	end
	return buttons
end

function Generic:ClearSetPoint(...)
	self:ClearAllPoints()
	self:SetPoint(...)
end

function Generic:NewHeader(header)
	self.header = header
	self:SetParent(header)
	SetupSecureSnippets(self)
	WrapOnClick(self)
end


-----------------------------------------------------------
--- state management

function Generic:ClearStates()
	for state in pairs(self.state_types) do
		self:SetAttribute(format("labtype-%s", state), nil)
		self:SetAttribute(format("labaction-%s", state), nil)
	end
	wipe(self.state_types)
	wipe(self.state_actions)
end

function Generic:SetState(state, kind, action)
	if not state then state = self:GetAttribute("state") end
	state = tostring(state)
	-- we allow a nil kind for setting a empty state
	if not kind then kind = "empty" end
	if not type_meta_map[kind] then
		error("SetStateAction: unknown action type: " .. tostring(kind), 2)
	end
	if kind ~= "empty" and action == nil then
		error("SetStateAction: an action is required for non-empty states", 2)
	end
	if kind ~= "custom" and action ~= nil and type(action) ~= "number" and type(action) ~= "string" or (kind == "custom" and type(action) ~= "table") then
		error("SetStateAction: invalid action data type, only strings and numbers allowed", 2)
	end

	if kind == "item" then
		if tonumber(action) then
			action = format("item:%s", action)
		else
			local itemString = str_match(action, "^|c%x+|H(item[%d:]+)|h%[")
			if itemString then
				action = itemString
			end
		end
	end

	self.state_types[state] = kind
	self.state_actions[state] = action
	self:UpdateState(state)
end

function Generic:UpdateState(state)
	if not state then state = self:GetAttribute("state") end
	state = tostring(state)
	self:SetAttribute(format("labtype-%s", state), self.state_types[state])
	self:SetAttribute(format("labaction-%s", state), self.state_actions[state])
	if state ~= tostring(self:GetAttribute("state")) then return end
	if self.header then
		self.header:SetFrameRef("updateButton", self)
		self.header:Execute([[
			local frame = self:GetFrameRef("updateButton")
			control:RunFor(frame, frame:GetAttribute("UpdateState"), frame:GetAttribute("state"))
		]])
	else
	-- TODO
	end
	self:UpdateAction()
end

function Generic:GetAction(state)
	if not state then state = self:GetAttribute("state") end
	state = tostring(state)
	return self.state_types[state] or "empty", self.state_actions[state]
end

function Generic:UpdateAllStates()
	for state in pairs(self.state_types) do
		self:UpdateState(state)
	end
end

function Generic:ButtonContentsChanged(state, kind, value)
	state = tostring(state)
	self.state_types[state] = kind or "empty"
	self.state_actions[state] = value
--	lib.callbacks:Fire("OnButtonContentsChanged", self, state, self.state_types[state], self.state_actions[state])
	self:UpdateAction(self)
end

function Generic:DisableDragNDrop(flag)
	if InCombatLockdown() then
		error("LibActionButton-1.0: You can only toggle DragNDrop out of combat!", 2)
	end
	if flag then
		self:SetAttribute("LABdisableDragNDrop", true)
	else
		self:SetAttribute("LABdisableDragNDrop", nil)
	end
end

function Generic:AddToButtonFacade(group)
	if type(group) ~= "table" or type(group.AddButton) ~= "function" then
		error("LibActionButton-1.0:AddToButtonFacade: You need to supply a proper group to use!", 2)
	end
	group:AddButton(self)
	self.LBFSkinned = true
end

function Generic:AddToMasque(group)
	if type(group) ~= "table" or type(group.AddButton) ~= "function" then
		error("LibActionButton-1.0:AddToMasque: You need to supply a proper group to use!", 2)
	end
	group:AddButton(self)
	self.MasqueSkinned = true
end

function Generic:UpdateAlpha()
	UpdateCooldown(self)
end

-----------------------------------------------------------
--- frame scripts

-- copied (and adjusted) from SecureHandlers.lua
local function PickupAny(kind, target, detail, ...)
	if kind == "clear" then
		ClearCursor()
		kind, target, detail = target, detail, ...
	end

	if kind == 'action' then
		PickupAction(target)
	elseif kind == 'item' then
		PickupItem(target)
	elseif kind == 'macro' then
		PickupMacro(target)
	elseif kind == 'petaction' then
		PickupPetAction(target)
	elseif kind == 'spell' then
		PickupSpell(target)
	elseif kind == 'companion' then
		PickupCompanion(target, detail)
	elseif kind == 'equipmentset' then
		PickupEquipmentSet(target)
	end
end

function Generic:OnUpdate(elapsed)
	if not GetCVarBool('lockActionBars') then return; end

	self.lastupdate = (self.lastupdate or 0) + elapsed;
	if (self.lastupdate < .2) then return end
	self.lastupdate = 0

	local isDragKeyDown
	if GetModifiedClick("PICKUPACTION") == 'ALT' then
		isDragKeyDown = IsAltKeyDown()
	elseif GetModifiedClick("PICKUPACTION") == 'CTRL' then
		isDragKeyDown = IsControlKeyDown()
	elseif GetModifiedClick("PICKUPACTION") == 'SHIFT' then
		isDragKeyDown = IsShiftKeyDown()
	end

	if isDragKeyDown and (self.clickState == 'AnyDown' or self.clickState == nil) then
		self.clickState = 'AnyUp'
		self:RegisterForClicks(self.clickState)
	elseif self.clickState == 'AnyUp' and not isDragKeyDown then
		self.clickState = 'AnyDown'
		self:RegisterForClicks(self.clickState)
	end
end

function Generic:OnEnter()
	if self.config.tooltip ~= "disabled" and (self.config.tooltip ~= "nocombat" or not InCombatLockdown()) then
		UpdateTooltip(self)
	end

	if self._state_type == "action" and self.NewActionTexture then
		lib.ACTION_HIGHLIGHT_MARKS[self._state_action] = false
		UpdateNewAction(self)
	end

	if self.config.clickOnDown then
		self:SetScript('OnUpdate', Generic.OnUpdate)
	end
end

function Generic:OnLeave()
	GameTooltip:Hide()
	self:SetScript('OnUpdate', nil)
end

-- Insecure drag handler to allow clicking on the button with an action on the cursor
-- to place it on the button. Like action buttons work.
function Generic:PreClick()
	if self._state_type == "action" or self._state_type == "pet"
	   or InCombatLockdown() or self:GetAttribute("LABdisableDragNDrop")
	then
		return
	end
	-- check if there is actually something on the cursor
	local kind, value, subtype = GetCursorInfo()
	if not (kind and value) then return end
	self._old_type = self._state_type
	if self._state_type and self._state_type ~= "empty" then
		self._old_type = self._state_type
		self:SetAttribute("type", "empty")
		--self:SetState(nil, "empty", nil)
	end
	self._receiving_drag = true
end

local function formatHelper(input)
	if type(input) == "string" then
		return format("%q", input)
	else
		return tostring(input)
	end
end

function Generic:PostClick()
	UpdateButtonState(self)
	if self._receiving_drag and not InCombatLockdown() then
		if self._old_type then
			self:SetAttribute("type", self._old_type)
			self._old_type = nil
		end
		local oldType, oldAction = self._state_type, self._state_action
		local kind, data, subtype, extra = GetCursorInfo()
		self.header:SetFrameRef("updateButton", self)
		self.header:Execute(format([[
			local frame = self:GetFrameRef("updateButton")
			control:RunFor(frame, frame:GetAttribute("OnReceiveDrag"), %s, %s, %s, %s)
			control:RunFor(frame, frame:GetAttribute("UpdateState"), %s)
		]], formatHelper(kind), formatHelper(data), formatHelper(subtype), formatHelper(extra), formatHelper(self:GetAttribute("state"))))
		PickupAny("clear", oldType, oldAction)
	end
	self._receiving_drag = nil
end

-----------------------------------------------------------
--- configuration

local function merge(target, source, default)
	for k,v in pairs(default) do
		if type(v) ~= "table" then
			if source and source[k] ~= nil then
				target[k] = source[k]
			else
				target[k] = v
			end
		else
			if type(target[k]) ~= "table" then target[k] = {} else wipe(target[k]) end
			merge(target[k], type(source) == "table" and source[k], v)
		end
	end
	return target
end

function Generic:UpdateConfig(config)
	if config and type(config) ~= "table" then
		error("LibActionButton-1.0: UpdateConfig requires a valid configuration!", 2)
	end
	local oldconfig = self.config
	if not self.config then self.config = {} end
	-- merge the two configs
	merge(self.config, config, DefaultConfig)

	if self.config.outOfRangeColoring == "button" or (oldconfig and oldconfig.outOfRangeColoring == "button") then
		UpdateUsable(self)
	end
	if self.config.outOfRangeColoring == "hotkey" then
		self.outOfRange = nil
	elseif oldconfig and oldconfig.outOfRangeColoring == "hotkey" then
		self.HotKey:SetVertexColor(0.75, 0.75, 0.75)
	end

	if self.config.hideElements.macro then
		self.Name:Hide()
	else
		self.Name:Show()
	end

	self:SetAttribute("flyoutDirection", self.config.flyoutDirection)

	UpdateHotkeys(self)
	UpdateGrid(self)
	Update(self)
	self:RegisterForClicks(self.config.clickOnDown and "AnyDown" or "AnyUp")
end

-----------------------------------------------------------
--- event handler

function ForAllButtons(method, onlyWithAction)
	assert(type(method) == "function")
	for button in next, (onlyWithAction and ActiveButtons or ButtonRegistry) do
		method(button)
	end
end

function InitializeEventHandler()
	lib.eventFrame:SetScript("OnEvent", OnEvent)
	lib.eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	lib.eventFrame:RegisterEvent("ACTIONBAR_SHOWGRID")
	lib.eventFrame:RegisterEvent("ACTIONBAR_HIDEGRID")
	--lib.eventFrame:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
	--lib.eventFrame:RegisterEvent("UPDATE_BONUS_ACTIONBAR")
	lib.eventFrame:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
	lib.eventFrame:RegisterEvent("UPDATE_BINDINGS")
	lib.eventFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	lib.eventFrame:RegisterEvent("UPDATE_VEHICLE_ACTIONBAR")

	lib.eventFrame:RegisterEvent("ACTIONBAR_UPDATE_STATE")
	lib.eventFrame:RegisterEvent("ACTIONBAR_UPDATE_USABLE")
	lib.eventFrame:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	lib.eventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	lib.eventFrame:RegisterEvent("TRADE_SKILL_SHOW")
	lib.eventFrame:RegisterEvent("TRADE_SKILL_CLOSE")
	lib.eventFrame:RegisterEvent("ARCHAEOLOGY_CLOSED")
	lib.eventFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
	lib.eventFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
	lib.eventFrame:RegisterEvent("START_AUTOREPEAT_SPELL")
	lib.eventFrame:RegisterEvent("STOP_AUTOREPEAT_SPELL")
	lib.eventFrame:RegisterEvent("UNIT_ENTERED_VEHICLE")
	lib.eventFrame:RegisterEvent("UNIT_EXITED_VEHICLE")
	lib.eventFrame:RegisterEvent("COMPANION_UPDATE")
	lib.eventFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")
	lib.eventFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
	lib.eventFrame:RegisterEvent("PET_STABLE_UPDATE")
	lib.eventFrame:RegisterEvent("PET_STABLE_SHOW")
	lib.eventFrame:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_SHOW")
	lib.eventFrame:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_HIDE")
	lib.eventFrame:RegisterEvent("SPELL_UPDATE_CHARGES")
	lib.eventFrame:RegisterEvent("UPDATE_SUMMONPETS_ACTION")

	-- With those two, do we still need the ACTIONBAR equivalents of them?
	lib.eventFrame:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	lib.eventFrame:RegisterEvent("SPELL_UPDATE_USABLE")
	lib.eventFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")

	lib.eventFrame:RegisterEvent("LOSS_OF_CONTROL_ADDED")
	lib.eventFrame:RegisterEvent("LOSS_OF_CONTROL_UPDATE")

	lib.eventFrame:Show()
	lib.eventFrame:SetScript("OnUpdate", OnUpdate)
end

function OnEvent(frame, event, arg1, ...)
	if (event == "UNIT_INVENTORY_CHANGED" and arg1 == "player") or event == "LEARNED_SPELL_IN_TAB" then
		local tooltipOwner = GameTooltip:GetOwner()
		if ButtonRegistry[tooltipOwner] then
			tooltipOwner:SetTooltip()
		end
	elseif event == "ACTIONBAR_SLOT_CHANGED" then
		for button in next, ButtonRegistry do
			if button._state_type == "action" and (arg1 == 0 or arg1 == tonumber(button._state_action)) then
				Update(button)
			end
		end
	elseif event == "PLAYER_ENTERING_WORLD" or event == "UPDATE_SHAPESHIFT_FORM" or event == "UPDATE_VEHICLE_ACTIONBAR" then
		ForAllButtons(Update)
	elseif event == "ACTIONBAR_PAGE_CHANGED" or event == "UPDATE_BONUS_ACTIONBAR" then
		-- TODO: Are these even needed?
	elseif event == "ACTIONBAR_SHOWGRID" then
		ShowGrid()
	elseif event == "ACTIONBAR_HIDEGRID" then
		HideGrid()
	elseif event == "UPDATE_BINDINGS" then
		ForAllButtons(UpdateHotkeys)
	elseif event == "PLAYER_TARGET_CHANGED" then
		UpdateRangeTimer()
	elseif (event == "ACTIONBAR_UPDATE_STATE") or
		((event == "UNIT_ENTERED_VEHICLE" or event == "UNIT_EXITED_VEHICLE") and (arg1 == "player")) or
		((event == "COMPANION_UPDATE") and (arg1 == "MOUNT")) then
		ForAllButtons(UpdateButtonState, true)
	elseif event == "ACTIONBAR_UPDATE_USABLE" then
		for button in next, ActionButtons do
			UpdateUsable(button)
		end
	elseif event == "SPELL_UPDATE_USABLE" then
		for button in next, NonActionButtons do
			UpdateUsable(button)
		end
	elseif event == "ACTIONBAR_UPDATE_COOLDOWN" then
		for button in next, ActionButtons do
			UpdateCooldown(button)
			if GameTooltip:GetOwner() == button then
				UpdateTooltip(button)
			end
		end
	elseif event == "SPELL_UPDATE_COOLDOWN" then
		for button in next, NonActionButtons do
			UpdateCooldown(button)
			if GameTooltip:GetOwner() == button then
				UpdateTooltip(button)
			end
		end
	elseif event == "LOSS_OF_CONTROL_ADDED" then
		for button in next, ActiveButtons do
			UpdateCooldown(button)
			if GameTooltip:GetOwner() == button then
				UpdateTooltip(button)
			end
		end
	elseif event == "LOSS_OF_CONTROL_UPDATE" then
		for button in next, ActiveButtons do
			UpdateCooldown(button)
		end
	elseif event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE"  or event == "ARCHAEOLOGY_CLOSED" then
		ForAllButtons(UpdateButtonState, true)
	elseif event == "PLAYER_ENTER_COMBAT" then
		for button in next, ActiveButtons do
			if button:IsAttack() then
				StartFlash(button)
			end
		end
	elseif event == "PLAYER_LEAVE_COMBAT" then
		for button in next, ActiveButtons do
			if button:IsAttack() then
				StopFlash(button)
			end
		end
	elseif event == "START_AUTOREPEAT_SPELL" then
		for button in next, ActiveButtons do
			if button:IsAutoRepeat() then
				StartFlash(button)
			end
		end
	elseif event == "STOP_AUTOREPEAT_SPELL" then
		for button in next, ActiveButtons do
			if button.flashing == 1 and not button:IsAttack() then
				StopFlash(button)
			end
		end
	elseif event == "PET_STABLE_UPDATE" or event == "PET_STABLE_SHOW" then
		ForAllButtons(Update)
	elseif event == "SPELL_ACTIVATION_OVERLAY_GLOW_SHOW" then
		for button in next, ActiveButtons do
			local spellId = button:GetSpellId()
			if spellId and spellId == arg1 then
				ShowOverlayGlow(button)
			else
				if button._state_type == "action" then
					local actionType, id = GetActionInfo(button._state_action)
					if actionType == "flyout" and FlyoutHasSpell(id, arg1) then
						ShowOverlayGlow(button)
					end
				end
			end
		end
	elseif event == "SPELL_ACTIVATION_OVERLAY_GLOW_HIDE" then
		for button in next, ActiveButtons do
			local spellId = button:GetSpellId()
			if spellId and spellId == arg1 then
				HideOverlayGlow(button)
			else
				if button._state_type == "action" then
					local actionType, id = GetActionInfo(button._state_action)
					if actionType == "flyout" and FlyoutHasSpell(id, arg1) then
						HideOverlayGlow(button)
					end
				end
			end
		end
	elseif event == "PLAYER_EQUIPMENT_CHANGED" then
		for button in next, ActiveButtons do
			if button._state_type == "item" then
				Update(button)
			end
		end
	elseif event == "SPELL_UPDATE_CHARGES" then
		ForAllButtons(UpdateCount, true)
	elseif event == "UPDATE_SUMMONPETS_ACTION" then
		for button in next, ActiveButtons do
			if button._state_type == "action" then
				local actionType, id = GetActionInfo(button._state_action)
				if actionType == "summonpet" then
					local texture = GetActionTexture(button._state_action)
					if texture then
						button.icon:SetTexture(texture)
					end
				end
			end
		end
	end
end

local flashTime = 0
local rangeTimer = -1
function OnUpdate(_, elapsed)
	flashTime = flashTime - elapsed
	rangeTimer = rangeTimer - elapsed
	-- Run the loop only when there is something to update
	if rangeTimer <= 0 or flashTime <= 0 then
		for button in next, ActiveButtons do
			-- Flashing
			if button.flashing == 1 and flashTime <= 0 then
				if button.Flash:IsShown() then
					button.Flash:Hide()
				else
					button.Flash:Show()
				end
			end

			-- Range
			if rangeTimer <= 0 then
				local inRange = button:IsInRange()
				local oldRange = button.outOfRange
				button.outOfRange = (inRange == false)
				if oldRange ~= button.outOfRange then
					if button.config.outOfRangeColoring == "button" then
						UpdateUsable(button)
					elseif button.config.outOfRangeColoring == "hotkey" then
						local hotkey = button.HotKey
						if hotkey:GetText() == RANGE_INDICATOR then
							if inRange == false then
								hotkey:Show()
							else
								hotkey:Hide()
							end
						end
						if inRange == false then
							hotkey:SetVertexColor(unpack(button.config.colors.range))
						else
							hotkey:SetVertexColor(0.75, 0.75, 0.75)
						end
					end
				end
			end
		end

		-- Update values
		if flashTime <= 0 then
			flashTime = flashTime + ATTACK_BUTTON_FLASH_TIME
		end
		if rangeTimer <= 0 then
			rangeTimer = TOOLTIP_UPDATE_TIME
		end
	end
end

local gridCounter = 0
function ShowGrid()
	gridCounter = gridCounter + 1
	if gridCounter >= 1 then
		for button in next, ButtonRegistry do
			if button:IsShown() then
				button:SetAlpha(1.0)
			end
		end
	end
end

function HideGrid()
	if gridCounter > 0 then
		gridCounter = gridCounter - 1
	end
	if gridCounter == 0 then
		for button in next, ButtonRegistry do
			if button:IsShown() and not button:HasAction() and not button.config.showGrid then
				button:SetAlpha(0.0)
			end
		end
	end
end

function UpdateGrid(self)
	if self.config.showGrid then
		self:SetAlpha(1.0)
	elseif gridCounter == 0 and self:IsShown() and not self:HasAction() then
		self:SetAlpha(0.0)
	end
end

-----------------------------------------------------------
--- integration

function Generic:GetBindingAction()
	return self.config.keyBoundTarget or "CLICK "..self:GetName()..":LeftButton"
end

function Generic:GetHotkey()
	local name = "CLICK "..self:GetName()..":LeftButton"
	local key = GetBindingKey(self.config.keyBoundTarget or name)
	if not key and self.config.keyBoundTarget then
		key = GetBindingKey(name)
	end
	if key then
		return key
	end
end

local function getKeys(binding, keys)
	keys = keys or ""
	for i = 1, select("#", GetBindingKey(binding)) do
		local hotKey = select(i, GetBindingKey(binding))
		if keys ~= "" then
			keys = keys .. ", "
		end
		keys = keys .. GetBindingText(hotKey, "KEY_")
	end
	return keys
end

function Generic:GetBindings()
	local keys, binding

	if self.config.keyBoundTarget then
		keys = getKeys(self.config.keyBoundTarget)
	end

	keys = getKeys("CLICK "..self:GetName()..":LeftButton")

	return keys
end

function Generic:SetKey(key)
	if self.config.keyBoundTarget then
		SetBinding(key, self.config.keyBoundTarget)
	else
		SetBindingClick(key, self:GetName(), "LeftButton")
	end
--	lib.callbacks:Fire("OnKeybindingChanged", self, key)
end

local function clearBindings(binding)
	while GetBindingKey(binding) do
		SetBinding(GetBindingKey(binding), nil)
	end
end

function Generic:ClearBindings()
	if self.config.keyBoundTarget then
		clearBindings(self.config.keyBoundTarget)
	end
	clearBindings("CLICK "..self:GetName()..":LeftButton")
--	lib.callbacks:Fire("OnKeybindingChanged", self, nil)
end

-----------------------------------------------------------
--- button management

function Generic:UpdateAction(force)
	local type, action = self:GetAction()
	if force or type ~= self._state_type or action ~= self._state_action then
		-- type changed, update the metatable
		if force or self._state_type ~= type then
			local meta = type_meta_map[type] or type_meta_map.empty
			setmetatable(self, meta)
			self._state_type = type
		end
		self._state_action = action
		Update(self)
	end
end

function Update(self)
	if self:HasAction() then
		ActiveButtons[self] = true
		if self._state_type == "action" then
			ActionButtons[self] = true
			NonActionButtons[self] = nil
		else
			ActionButtons[self] = nil
			NonActionButtons[self] = true
		end
		self:SetAlpha(1.0)
		UpdateButtonState(self)
		UpdateUsable(self)
		UpdateCooldown(self)
		UpdateFlash(self)
	else
		ActiveButtons[self] = nil
		ActionButtons[self] = nil
		NonActionButtons[self] = nil
		if gridCounter == 0 and not self.config.showGrid then
			self:SetAlpha(0.0)
		end
		self.cooldown:Hide()
		self:SetChecked(false)

		if self.chargeCooldown then
			EndChargeCooldown(self.chargeCooldown)
		end
	end

	-- Add a green border if button is an equipped item
	if self:IsEquipped() and not self.config.hideElements.equipped then
		self.Border:SetVertexColor(0, 1.0, 0, 0.35)
		self.Border:Show()
	else
		self.Border:Hide()
	end

	-- Update Action Text
	if not self:IsConsumableOrStackable() then
		self.Name:SetText(self:GetActionText())
	else
		self.Name:SetText("")
	end

	-- Update icon and hotkey
	local texture = self:GetTexture()

	-- Draenor zone button handling
	self.draenorZoneDisabled = false
	self.icon:SetDesaturated(false)
	if self._state_type == "action" then
		local action_type, id = GetActionInfo(self._state_action)
		if ((action_type == "spell" or action_type == "companion") and DraenorZoneAbilityFrame and DraenorZoneAbilityFrame.baseName and not HasDraenorZoneAbility()) then
			local name = GetSpellInfo(DraenorZoneAbilityFrame.baseName)
			local abilityName = GetSpellInfo(id)
			if name == abilityName then
				texture = GetLastDraenorSpellTexture()
				self.draenorZoneDisabled = true
				self.icon:SetDesaturated(true)
			end
		end
	end

	if texture then
		self.icon:SetTexture(texture)
		self.icon:Show()
		self.rangeTimer = - 1
		self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
		if not self.LBFSkinned and not self.MasqueSkinned then
			self.NormalTexture:SetTexCoord(0, 0, 0, 0)
		end
	else
		self.icon:Hide()
		self.cooldown:Hide()
		self.rangeTimer = nil
		self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot")
		if self.HotKey:GetText() == RANGE_INDICATOR then
			self.HotKey:Hide()
		else
			self.HotKey:SetVertexColor(0.75, 0.75, 0.75)
		end
		if not self.LBFSkinned and not self.MasqueSkinned then
			self.NormalTexture:SetTexCoord(-0.15, 1.15, -0.15, 1.17)
		end
	end

	self:UpdateLocal()

	UpdateCount(self)

	UpdateFlyout(self)

	UpdateOverlayGlow(self)

	UpdateNewAction(self)

	if GameTooltip:GetOwner() == self then
		UpdateTooltip(self)
	end

	-- this could've been a spec change, need to call OnStateChanged for action buttons, if present
	if not InCombatLockdown() and self._state_type == "action" then
		local onStateChanged = self:GetAttribute("OnStateChanged")
		if onStateChanged then
			self.header:SetFrameRef("updateButton", self)
			self.header:Execute(([[
				local frame = self:GetFrameRef("updateButton")
				control:RunFor(frame, frame:GetAttribute("OnStateChanged"), %s, %s, %s)
			]]):format(formatHelper(self:GetAttribute("state")), formatHelper(self._state_type), formatHelper(self._state_action)))
		end
	end
--	lib.callbacks:Fire("OnButtonUpdate", self)
end

function Generic:UpdateLocal()
-- dummy function the other button types can override for special updating
end

function UpdateButtonState(self)
	if self:IsCurrentlyActive() or self:IsAutoRepeat() then
		self:SetChecked(true)
	else
		self:SetChecked(false)
	end
--	lib.callbacks:Fire("OnButtonState", self)
end

local function IsHolyPowerAbility(actionId)
	if not actionId or type(actionId) ~= 'number' then return false; end
	local actionType, id = GetActionInfo(actionId);
	if actionType == 'macro' then
		local macroSpell = GetMacroSpell(id);
		if macroSpell then
			for spellId, spellName in pairs(HOLY_POWER_SPELLS) do
				if macroSpell == spellName then
					return true;
				end
			end
		end
	else
		return HOLY_POWER_SPELLS[id];
	end
	return false;
end
function UpdateUsable(self)
	-- TODO: make the colors configurable
	-- TODO: allow disabling of the whole recoloring
	if self.config.outOfRangeColoring == "button" and self.outOfRange then
		self.icon:SetVertexColor(unpack(self.config.colors.range))
	else
		local isUsable, notEnoughMana = self:IsUsable()
		local action = self._state_action
		if PLAYERCLASS == 'PALADIN' and IsHolyPowerAbility(action) and not(UnitPower('player', SPELL_POWER_HOLY_POWER) >= 3) then
			self.icon:SetVertexColor(unpack(self.config.colors.hp))
		elseif isUsable then
			self.icon:SetVertexColor(1.0, 1.0, 1.0)
			--self.NormalTexture:SetVertexColor(1.0, 1.0, 1.0)
		elseif notEnoughMana then
			self.icon:SetVertexColor(unpack(self.config.colors.mana))
			--self.NormalTexture:SetVertexColor(0.5, 0.5, 1.0)
		else
			self.icon:SetVertexColor(0.4, 0.4, 0.4)
			--self.NormalTexture:SetVertexColor(1.0, 1.0, 1.0)
		end
	end
--	lib.callbacks:Fire("OnButtonUsable", self)
end

function UpdateCount(self)
	if not self:HasAction() then
		self.Count:SetText("")
		return
	end
	if self:IsConsumableOrStackable() then
		local count = self:GetCount()
		if count > (self.maxDisplayCount or 9999) then
			self.Count:SetText("*")
		else
			self.Count:SetText(count)
		end
	else
		local charges, maxCharges, chargeStart, chargeDuration = self:GetCharges()
		if charges and maxCharges and maxCharges > 1 then
			self.Count:SetText(charges)
		else
			self.Count:SetText("")
		end
	end
end

function EndChargeCooldown(self)
	self:Hide()
--	self:SetParent(UIParent)
--	self.parent.chargeCooldown = nil
--	self.parent = nil
--	tinsert(lib.ChargeCooldowns, self)
end

local function StartChargeCooldown(parent, chargeStart, chargeDuration)
	--[==[
	if not parent.chargeCooldown then
		local cooldown = tremove(lib.ChargeCooldowns)
		if not cooldown then
			lib.NumChargeCooldowns = lib.NumChargeCooldowns + 1
			cooldown = CreateFrame("Cooldown", "AleaUIChargeCooldown"..lib.NumChargeCooldowns, parent, "CooldownFrameTemplate");
			cooldown:SetScript("OnCooldownDone", EndChargeCooldown)
			cooldown:SetHideCountdownNumbers(true)
			cooldown:SetDrawEdge(true)
			cooldown:SetDrawSwipe(false)
			cooldown:SetBlingTexture('')
		end
		cooldown:SetParent(parent)
		cooldown:SetAllPoints(parent)
		cooldown:SetFrameStrata("TOOLTIP")
		cooldown:Show()
		parent.chargeCooldown = cooldown
		cooldown.parent = parent
	end
	]==]
	parent.chargeCooldown:SetDrawBling(parent.chargeCooldown:GetEffectiveAlpha() > 0.5)
	parent.chargeCooldown:SetCooldown(chargeStart, chargeDuration)
	if not chargeStart or chargeStart == 0 then
		EndChargeCooldown(parent.chargeCooldown)
	end
end

local function OnCooldownDone(self)
	self:SetScript("OnCooldownDone", nil)
	UpdateCooldown(self:GetParent())
end

function UpdateCooldown(self)
	local locStart, locDuration = self:GetLossOfControlCooldown()
	local start, duration, enable = self:GetCooldown()
	local charges, maxCharges, chargeStart, chargeDuration = self:GetCharges()

	self.cooldown:SetDrawBling(self.cooldown:GetEffectiveAlpha() > 0.5)

	if (locStart + locDuration) > (start + duration) then
		if self.cooldown.currentCooldownType ~= COOLDOWN_TYPE_LOSS_OF_CONTROL then
			self.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge-LoC")
			self.cooldown:SetSwipeColor(0.17, 0, 0)
			self.cooldown:SetHideCountdownNumbers(true)
			self.cooldown.currentCooldownType = COOLDOWN_TYPE_LOSS_OF_CONTROL
		end
		CooldownFrame_SetTimer(self.cooldown, locStart, locDuration, 1, true)
	else
		if self.cooldown.currentCooldownType ~= COOLDOWN_TYPE_NORMAL then
			self.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge")
			self.cooldown:SetSwipeColor(0, 0, 0)
			self.cooldown:SetHideCountdownNumbers(false)
			self.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL
		end
		if locStart > 0 then
			self.cooldown:SetScript("OnCooldownDone", OnCooldownDone)
		end

		if charges and maxCharges and maxCharges > 0 and charges > 0 and charges < maxCharges then
			StartChargeCooldown(self, chargeStart, chargeDuration)
		elseif self.chargeCooldown then
			EndChargeCooldown(self.chargeCooldown)
		end

		CooldownFrame_SetTimer(self.cooldown, start, duration, enable)
	end
end

function StartFlash(self)
	self.flashing = 1
	flashTime = 0
	UpdateButtonState(self)
end

function StopFlash(self)
	self.flashing = 0
	self.Flash:Hide()
	UpdateButtonState(self)
end

function UpdateFlash(self)
	if (self:IsAttack() and self:IsCurrentlyActive()) or self:IsAutoRepeat() then
		StartFlash(self)
	else
		StopFlash(self)
	end
end

function UpdateTooltip(self)
	if (GetCVar("UberTooltips") == "1") then
		GameTooltip_SetDefaultAnchor(GameTooltip, self);
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	end
	if self:SetTooltip() then
		self.UpdateTooltip = UpdateTooltip
	else
		self.UpdateTooltip = nil
	end
end

function UpdateHotkeys(self)
	local key = self:GetHotkey()
	if not key or key == "" or self.config.hideElements.hotkey then
		self.HotKey:SetText(RANGE_INDICATOR)
		self.HotKey:SetPoint("TOPLEFT", self, "TOPLEFT", 1, - 2)
		self.HotKey:Hide()
	else
		self.HotKey:SetText(key)
		self.HotKey:SetPoint("TOPLEFT", self, "TOPLEFT", - 2, - 2)
		self.HotKey:Show()
	end

	if self.postKeybind then
		self.postKeybind(nil, self)
	end
end

function ShowOverlayGlow(self)
	Skins.ShowOverlayGlow(self)
end

function HideOverlayGlow(self)
	Skins.HideOverlayGlow(self)
end

function UpdateOverlayGlow(self)
	local spellId = self:GetSpellId()
	if spellId and IsSpellOverlayed(spellId) then
		ShowOverlayGlow(self)
	else
		HideOverlayGlow(self)
	end
end

hooksecurefunc("MarkNewActionHighlight", function(action, flag)
	lib.ACTION_HIGHLIGHT_MARKS[action] = flag
	for button in next, ButtonRegistry do
		if button._state_type == "action" and action == tonumber(button._state_action) then
			UpdateNewAction(button)
		end
	end
end)

function UpdateNewAction(self)
	-- special handling for "New Action" markers
	if self.NewActionTexture then
		if self._state_type == "action" and lib.ACTION_HIGHLIGHT_MARKS[self._state_action] then
			self.NewActionTexture:Show()
		else
			self.NewActionTexture:Hide()
		end
	end
end

-- Hook UpdateFlyout so we can use the blizzy templates
hooksecurefunc("ActionButton_UpdateFlyout", function(self, ...)
	if ButtonRegistry[self] then
		UpdateFlyout(self)
	end
end)

function UpdateFlyout(self)
	-- disabled FlyoutBorder/BorderShadow, those are not handled by LBF and look terrible
	self.FlyoutBorder:Hide()
	self.FlyoutBorderShadow:Hide()
	if self._state_type == "action" then
		-- based on ActionButton_UpdateFlyout in ActionButton.lua
		local actionType = GetActionInfo(self._state_action)
		if actionType == "flyout" then
			-- Update border and determine arrow position
			local arrowDistance
			if (SpellFlyout and SpellFlyout:IsShown() and SpellFlyout:GetParent() == self) or GetMouseFocus() == self then
				arrowDistance = 5
			else
				arrowDistance = 2
			end

			-- Update arrow
			self.FlyoutArrow:Show()
			self.FlyoutArrow:ClearAllPoints()
			local direction = self:GetAttribute("flyoutDirection");
			if direction == "LEFT" then
				self.FlyoutArrow:SetPoint("LEFT", self, "LEFT", -arrowDistance, 0)
				SetClampedTextureRotation(self.FlyoutArrow, 270)
			elseif direction == "RIGHT" then
				self.FlyoutArrow:SetPoint("RIGHT", self, "RIGHT", arrowDistance, 0)
				SetClampedTextureRotation(self.FlyoutArrow, 90)
			elseif direction == "DOWN" then
				self.FlyoutArrow:SetPoint("BOTTOM", self, "BOTTOM", 0, -arrowDistance)
				SetClampedTextureRotation(self.FlyoutArrow, 180)
			else
				self.FlyoutArrow:SetPoint("TOP", self, "TOP", 0, arrowDistance)
				SetClampedTextureRotation(self.FlyoutArrow, 0)
			end

			if self.FlyoutUpdateFunc then
				self.FlyoutUpdateFunc(nil, self)
			end

			-- return here, otherwise flyout is hidden
			return
		end
	end
	self.FlyoutArrow:Hide()
end

function UpdateRangeTimer()
	rangeTimer = -1
end

-----------------------------------------------------------
--- WoW API mapping
--- Generic Button
Generic.HasAction               = function(self) return nil end
Generic.GetActionText           = function(self) return "" end
Generic.GetTexture              = function(self) return nil end
Generic.GetCharges              = function(self) return nil end
Generic.GetCount                = function(self) return 0 end
Generic.GetCooldown             = function(self) return 0, 0, 0 end
Generic.IsAttack                = function(self) return nil end
Generic.IsEquipped              = function(self) return nil end
Generic.IsCurrentlyActive       = function(self) return nil end
Generic.IsAutoRepeat            = function(self) return nil end
Generic.IsUsable                = function(self) return nil end
Generic.IsConsumableOrStackable = function(self) return nil end
Generic.IsUnitInRange           = function(self, unit) return nil end
Generic.IsInRange               = function(self)
	local unit = self:GetAttribute("unit")
	if unit == "player" then
		unit = nil
	end
	local val = self:IsUnitInRange(unit)
	-- map 1/0 to true false, since the return values are inconsistent between actions and spells
	if val == 1 then val = true elseif val == 0 then val = false end
	return val
end
Generic.SetTooltip              = function(self) return nil end
Generic.GetSpellId              = function(self) return nil end
Generic.GetLossOfControlCooldown = function(self) return 0, 0 end

-----------------------------------------------------------
--- Action Button
Action.HasAction               = function(self) return HasAction(self._state_action) end
Action.GetActionText           = function(self) return GetActionText(self._state_action) end
Action.GetTexture              = function(self) return GetActionTexture(self._state_action) end
Action.GetCharges              = function(self) return GetActionCharges(self._state_action) end
Action.GetCount                = function(self) return GetActionCount(self._state_action) end
Action.GetCooldown             = function(self) return GetActionCooldown(self._state_action) end
Action.IsAttack                = function(self) return IsAttackAction(self._state_action) end
Action.IsEquipped              = function(self) return IsEquippedAction(self._state_action) end
Action.IsCurrentlyActive       = function(self) return IsCurrentAction(self._state_action) end
Action.IsAutoRepeat            = function(self) return IsAutoRepeatAction(self._state_action) end
Action.IsUsable                = function(self) return IsUsableAction(self._state_action) end
Action.IsConsumableOrStackable = function(self) return IsConsumableAction(self._state_action) or IsStackableAction(self._state_action) or (not IsItemAction(self._state_action) and GetActionCount(self._state_action) > 0) end
Action.IsUnitInRange           = function(self, unit) return IsActionInRange(self._state_action, unit) end
Action.SetTooltip              = function(self) return GameTooltip:SetAction(self._state_action) end
Action.GetSpellId              = function(self)
	local actionType, id, subType = GetActionInfo(self._state_action)
	if actionType == "spell" then
		return id
	elseif actionType == "macro" then
		local _, _, spellId = GetMacroSpell(id)
		return spellId
	end
end
Action.GetLossOfControlCooldown = function(self) return GetActionLossOfControlCooldown(self._state_action) end

-----------------------------------------------------------
--- Spell Button
Spell.HasAction               = function(self) return true end
Spell.GetActionText           = function(self) return "" end
Spell.GetTexture              = function(self) return GetSpellTexture(self._state_action) end
Spell.GetCharges              = function(self) return GetSpellCharges(self._state_action) end
Spell.GetCount                = function(self) return GetSpellCount(self._state_action) end
Spell.GetCooldown             = function(self) return GetSpellCooldown(self._state_action) end
Spell.IsAttack                = function(self) return IsAttackSpell(FindSpellBookSlotBySpellID(self._state_action), "spell") end -- needs spell book id as of 4.0.1.13066
Spell.IsEquipped              = function(self) return nil end
Spell.IsCurrentlyActive       = function(self) return IsCurrentSpell(self._state_action) end
Spell.IsAutoRepeat            = function(self) return IsAutoRepeatSpell(FindSpellBookSlotBySpellID(self._state_action), "spell") end -- needs spell book id as of 4.0.1.13066
Spell.IsUsable                = function(self) return IsUsableSpell(self._state_action) end
Spell.IsConsumableOrStackable = function(self) return IsConsumableSpell(self._state_action) end
Spell.IsUnitInRange           = function(self, unit) return IsSpellInRange(FindSpellBookSlotBySpellID(self._state_action), "spell", unit) end -- needs spell book id as of 4.0.1.13066
Spell.SetTooltip              = function(self) return GameTooltip:SetSpellByID(self._state_action) end
Spell.GetSpellId              = function(self) return self._state_action end

-----------------------------------------------------------
--- Item Button
local function getItemId(input)
	return input:match("^item:(%d+)")
end

Item.HasAction               = function(self) return true end
Item.GetActionText           = function(self) return "" end
Item.GetTexture              = function(self) return GetItemIcon(self._state_action) end
Item.GetCharges              = function(self) return nil end
Item.GetCount                = function(self) return GetItemCount(self._state_action, nil, true) end
Item.GetCooldown             = function(self) return GetItemCooldown(getItemId(self._state_action)) end
Item.IsAttack                = function(self) return nil end
Item.IsEquipped              = function(self) return IsEquippedItem(self._state_action) end
Item.IsCurrentlyActive       = function(self) return IsCurrentItem(self._state_action) end
Item.IsAutoRepeat            = function(self) return nil end
Item.IsUsable                = function(self) return IsUsableItem(self._state_action) end
Item.IsConsumableOrStackable = function(self) return IsConsumableItem(self._state_action) end
Item.IsUnitInRange           = function(self, unit) return IsItemInRange(self._state_action, unit) end
Item.SetTooltip              = function(self) return GameTooltip:SetHyperlink(self._state_action) end
Item.GetSpellId              = function(self) return nil end

-----------------------------------------------------------
--- Macro Button
-- TODO: map results of GetMacroSpell/GetMacroItem to proper results
Macro.HasAction               = function(self) return true end
Macro.GetActionText           = function(self) return (GetMacroInfo(self._state_action)) end
Macro.GetTexture              = function(self) return (select(2, GetMacroInfo(self._state_action))) end
Macro.GetCharges              = function(self) return nil end
Macro.GetCount                = function(self) return 0 end
Macro.GetCooldown             = function(self) return 0, 0, 0 end
Macro.IsAttack                = function(self) return nil end
Macro.IsEquipped              = function(self) return nil end
Macro.IsCurrentlyActive       = function(self) return nil end
Macro.IsAutoRepeat            = function(self) return nil end
Macro.IsUsable                = function(self) return nil end
Macro.IsConsumableOrStackable = function(self) return nil end
Macro.IsUnitInRange           = function(self, unit) return nil end
Macro.SetTooltip              = function(self) return nil end
Macro.GetSpellId              = function(self) return nil end

-----------------------------------------------------------
--- Custom Button
Custom.HasAction               = function(self) return true end
Custom.GetActionText           = function(self) return "" end
Custom.GetTexture              = function(self) return self._state_action.texture end
Custom.GetCharges              = function(self) return nil end
Custom.GetCount                = function(self) return 0 end
Custom.GetCooldown             = function(self) return 0, 0, 0 end
Custom.IsAttack                = function(self) return nil end
Custom.IsEquipped              = function(self) return nil end
Custom.IsCurrentlyActive       = function(self) return nil end
Custom.IsAutoRepeat            = function(self) return nil end
Custom.IsUsable                = function(self) return true end
Custom.IsConsumableOrStackable = function(self) return nil end
Custom.IsUnitInRange           = function(self, unit) return nil end
Custom.SetTooltip              = function(self) return GameTooltip:SetText(self._state_action.tooltip) end
Custom.GetSpellId              = function(self) return nil end
Custom.RunCustom               = function(self, unit, button) return self._state_action.func(self, unit, button) end

local LAB = AleaUIButtonsCore

AB["handledBars"] = {} --List of all bars
AB["handledbuttons"] = {} --List of all buttons that have been modified.
AB["barDefaults"] = {
	["bar1"] = {
		['page'] = 1,
		['bindButtons'] = "ACTIONBUTTON",
		['conditions'] = format("[vehicleui] %d; [possessbar] %d; [overridebar] %d; [shapeshift] 13; [form,noform] 0; [bar:2] 2; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6;", GetVehicleBarIndex(), GetVehicleBarIndex(), GetOverrideBarIndex()),
		['position'] = "BOTTOM,AleaUIParent,BOTTOM,327,5",
	},
	["bar2"] = {
		['page'] = 5,
		['bindButtons'] = "MULTIACTIONBAR2BUTTON",
		['conditions'] = "",
		['position'] = "BOTTOM,AleaUIParent,BOTTOM,0,32",
	},
	["bar3"] = {
		['page'] = 6,
		['bindButtons'] = "MULTIACTIONBAR1BUTTON",
		['conditions'] = "",
		['position'] = "BOTTOM,AleaUIParent,BOTTOM,0,59",
	},
	["bar4"] = {
		['page'] = 4,
		['bindButtons'] = "MULTIACTIONBAR4BUTTON",
		['conditions'] = "",
		['position'] = "BOTTOM,AleaUIParent,BOTTOM,406,234",
	},
	["bar5"] = {
		['page'] = 3,
		['bindButtons'] = "MULTIACTIONBAR3BUTTON",
		['conditions'] = "",
		['position'] = "BOTTOM,AleaUIParent,BOTTOM,0,85",
	},
	["bar6"] = {
		['page'] = 2,
		['bindButtons'] = "ALEAUIBAR6BUTTON",
		['conditions'] = "",
		['position'] = "BOTTOM,AleaUI_Bar2,TOP,0,2",
	},
}

AB.customExitButton = {
	func = function(button)
		if UnitExists('vehicle') then
			VehicleExit()
		else
			PetDismiss()
		end
	end,
	texture = "Interface\\Icons\\Spell_Shadow_SacrificialShield",
	tooltip = LEAVE_VEHICLE,
}



function AB:PositionAndSizeBar(barName)
	local spacing = E.db.actionbars[barName].buttonspacing --E:Scale();
	local buttonsPerRow = E.db.actionbars[barName].buttonsPerRow;
	local numButtons = E.db.actionbars[barName].buttons;
	local size = E.db.actionbars[barName].buttonsize--  E:Scale();
	local point = E.db.actionbars[barName].point;
	local numColumns = ceil(numButtons / buttonsPerRow);
	local widthMult = E.db.actionbars[barName].widthMult;
	local heightMult = E.db.actionbars[barName].heightMult;
	local bar = self["handledBars"][barName]

	bar.db = E.db.actionbars[barName]
	bar.db.position = nil; --Depreciated

	if numButtons < buttonsPerRow then
		buttonsPerRow = numButtons;
	end

	if numColumns < 1 then
		numColumns = 1;
	end

	bar:SetWidth(spacing + ((size * (buttonsPerRow * widthMult)) + ((spacing * (buttonsPerRow - 1)) * widthMult) + (spacing * widthMult)));
	bar:SetHeight(spacing + ((size * (numColumns * heightMult)) + ((spacing * (numColumns - 1)) * heightMult) + (spacing * heightMult)));
	
	if not bar.artWork then
		bar.artWork = CreateFrame("Frame", nil, bar)
		bar.artWork:SetFrameLevel(bar:GetFrameLevel())
		bar.artWork:SetBackdrop({
		  edgeFile = [[Interface\Buttons\WHITE8x8]],
		  edgeSize = 1, 
		})
		bar.artWork:SetBackdropBorderColor(0,0,0,1)
		bar.artWork:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, -0)
		bar.artWork:SetSize(100,100)

		bar.artWork.back = bar:CreateTexture()
		bar.artWork.back:SetDrawLayer('BACKGROUND', -2)
		bar.artWork.back:SetColorTexture(0, 0, 0, 0)
		bar.artWork.back:SetPoint("BOTTOMLEFT", bar.artWork, "BOTTOMLEFT", 0, -0)

		bar.artWork:SetScript('OnEvent', AB.VehicleBarArtWorkVisability)
	end
	
	local opts = E.db.actionbars[barName]
	
	if opts.artwork.enable then
		bar.artWork:SetBackdrop({
		  edgeFile = E:GetBorder(opts.artwork.texture),
		  edgeSize = opts.artwork.size, 
		})
		bar.artWork:SetBackdropBorderColor(opts.artwork.color[1],opts.artwork.color[2],opts.artwork.color[3],opts.artwork.color[4])
		bar.artWork:SetSize(opts.artwork.width, opts.artwork.height)
		bar.artWork:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", opts.artwork.inset, opts.artwork.inset)
		
		bar.artWork.back:SetTexture(E:GetTexture(opts.artwork.background_texture))
		bar.artWork.back:SetVertexColor(opts.artwork.background_color[1],opts.artwork.background_color[2],opts.artwork.background_color[3],opts.artwork.background_color[4])
	--	bar.artWork.back:SetPoint("TOPLEFT", bar.artWork, "TOPLEFT", -opts.artwork.background_inset, opts.artwork.background_inset)
	--	bar.artWork.back:SetPoint("BOTTOMRIGHT", bar.artWork, "BOTTOMRIGHT", opts.artwork.background_inset, -opts.artwork.background_inset)
		bar.artWork.back:SetSize(opts.artwork.width+opts.artwork.background_inset+opts.artwork.background_inset, opts.artwork.height+opts.artwork.background_inset+opts.artwork.background_inset)
		bar.artWork.back:SetPoint("BOTTOMLEFT", bar.artWork, "BOTTOMLEFT", -opts.artwork.background_inset, -opts.artwork.background_inset)
		
		bar.artWork:RegisterEvent('VEHICLE_UPDATE')
		bar.artWork:RegisterEvent('UNIT_ENTERED_VEHICLE')
		bar.artWork:RegisterEvent('UNIT_ENTERING_VEHICLE')
		bar.artWork:RegisterEvent('UNIT_EXITED_VEHICLE')
		bar.artWork:RegisterEvent('UNIT_EXITING_VEHICLE')
	
		bar.artWork:Show()
		bar.artWork.back:Show()
		
		AB.VehicleBarArtWorkVisability(bar.artWork)
	else
		bar.artWork:UnregisterEvent('VEHICLE_UPDATE')
		bar.artWork:UnregisterEvent('UNIT_ENTERED_VEHICLE')
		bar.artWork:UnregisterEvent('UNIT_ENTERING_VEHICLE')
		bar.artWork:UnregisterEvent('UNIT_EXITED_VEHICLE')
		bar.artWork:UnregisterEvent('UNIT_EXITING_VEHICLE')
		
		bar.artWork:Hide()
		bar.artWork.back:Hide()
	end
	
	bar.mouseover = E.db.actionbars[barName].mouseover

	if E.db.actionbars[barName].backdrop == true then
	--	bar.backdrop:Show();
	else
	--	bar.backdrop:Hide();
	end

	local horizontalGrowth, verticalGrowth;
	if point == "TOPLEFT" or point == "TOPRIGHT" then
		verticalGrowth = "DOWN";
	else
		verticalGrowth = "UP";
	end

	if point == "BOTTOMLEFT" or point == "TOPLEFT" then
		horizontalGrowth = "RIGHT";
	else
		horizontalGrowth = "LEFT";
	end

	local button, lastButton, lastColumnButton ;
	for i=1, NUM_ACTIONBAR_BUTTONS do
		button = bar.buttons[i];
		lastButton = bar.buttons[i-1];
		lastColumnButton = bar.buttons[i-buttonsPerRow];
		button:SetParent(bar);
		button:ClearAllPoints();
		button:SetSize(size, size)
		button:SetAttribute("showgrid", 1);
		--[[
		if E.db.actionbars[barName].mouseover == true then
			bar:SetAlpha(0);
			if not self.hooks[bar] then
				self:HookScript(bar, 'OnEnter', 'Bar_OnEnter');
				self:HookScript(bar, 'OnLeave', 'Bar_OnLeave');
			end

			if not self.hooks[button] then
				self:HookScript(button, 'OnEnter', 'Button_OnEnter');
				self:HookScript(button, 'OnLeave', 'Button_OnLeave');
			end
		else
			bar:SetAlpha(E.db.actionbars[barName].alpha);
			if self.hooks[bar] then
				self:Unhook(bar, 'OnEnter');
				self:Unhook(bar, 'OnLeave');
			end

			if self.hooks[button] then
				self:Unhook(button, 'OnEnter');
				self:Unhook(button, 'OnLeave');
			end
		end
		]]
		if i == 1 then
			local x, y;
			if point == "BOTTOMLEFT" then
				x, y = spacing, spacing;
			elseif point == "TOPRIGHT" then
				x, y = -spacing, -spacing;
			elseif point == "TOPLEFT" then
				x, y = spacing, -spacing;
			else
				x, y = -spacing, spacing;
			end

			button:SetPoint(point, bar, point, x, y);
		elseif (i - 1) % buttonsPerRow == 0 then
			local x = 0;
			local y = -spacing;
			local buttonPoint, anchorPoint = "TOP", "BOTTOM";
			if verticalGrowth == 'UP' then
				y = spacing;
				buttonPoint = "BOTTOM";
				anchorPoint = "TOP";
			end
			button:SetPoint(buttonPoint, lastColumnButton, anchorPoint, x, y);
		else
			local x = spacing;
			local y = 0;
			local buttonPoint, anchorPoint = "LEFT", "RIGHT";
			if horizontalGrowth == 'LEFT' then
				x = -spacing;
				buttonPoint = "RIGHT";
				anchorPoint = "LEFT";
			end

			button:SetPoint(buttonPoint, lastButton, anchorPoint, x, y);
		end
		
		local minVihicleExit = 7
		
		for i=7, 11 do
			bar.buttons[i]:SetState(12, "empty")
		end
			
		if numButtons < 12 then
			
			local exitButton = max(numButtons, minVihicleExit)
			
			bar.buttons[exitButton]:SetState(12, "custom", AB.customExitButton)	
		end
		
		if i > numButtons then
			button:Hide()
		else
			button:Show()
		end

		
		self:StyleButton(button, nil, nil, true);
		button:SetCheckedTexture("")
		
		local opts = E.db.actionbars[barName]
		
		button.artBorder:SetBackdrop({
		  edgeFile = E:GetBorder(opts.border.texture),
		  edgeSize = opts.border.size, 
		})
		button.artBorder:SetBackdropBorderColor(opts.border.color[1],opts.border.color[2],opts.border.color[3],opts.border.color[4])
		button.artBorder:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.inset, -opts.border.inset)
		button.artBorder:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.inset, opts.border.inset)

		button.artBorder.back:SetTexture(E:GetTexture(opts.border.background_texture))
		button.artBorder.back:SetVertexColor(opts.border.background_color[1],opts.border.background_color[2],opts.border.background_color[3],opts.border.background_color[4])
		button.artBorder.back:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.background_inset, -opts.border.background_inset)
		button.artBorder.back:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.background_inset, opts.border.background_inset)
		
		
	--	print('Update Bar', barName, opts, E:GetBorder(opts.border.texture))
	end

	if E.db.actionbars[barName].enabled or not bar.initialized then
		if not E.db.actionbars[barName].mouseover then
			bar:SetAlpha(E.db.actionbars[barName].alpha);
		end

		local page = self:GetPage(barName, self['barDefaults'][barName].page, self['barDefaults'][barName].conditions)
		if AB['barDefaults']['bar'..bar.id].conditions:find("[form,noform]") then
			bar:SetAttribute("hasTempBar", true)

			local newCondition = page
			newCondition = gsub(AB['barDefaults']['bar'..bar.id].conditions, " %[form,noform%] 0; ", "")
			bar:SetAttribute("newCondition", newCondition)
		else
			bar:SetAttribute("hasTempBar", false)
		end

		bar:Show()
		RegisterStateDriver(bar, "visibility", E.db.actionbars[barName].visibility); -- this is ghetto
		RegisterStateDriver(bar, "page", page);

		if not bar.initialized then
			bar.initialized = true;
			AB:PositionAndSizeBar(barName)
			return
		end
	else
		bar:Hide()
		UnregisterStateDriver(bar, "visibility");
	end


--	E:SetMoverSnapOffset('AleaAB_'..bar.id, bar.db.buttonspacing / 2)

end

function AB:CreateBar(id)
	local bar = CreateFrame('Frame', 'AleaUI_Bar'..id, E.UIParent, 'SecureHandlerStateTemplate');
	local point, anchor, attachTo, x, y = split(',', self['barDefaults']['bar'..id].position)
	bar:SetPoint(point, anchor, attachTo, x, y)
	bar.id = id
--	bar:CreateBackdrop('Default');
	bar:SetFrameStrata("LOW")
--	bar.backdrop:SetAllPoints();
	bar.buttons = {}
	bar.bindButtons = self['barDefaults']['bar'..id].bindButtons

	for i=1, 12 do
		bar.buttons[i] = LAB:CreateButton(i, format(bar:GetName().."Button%d", i), bar, nil)
	
		bar.buttons[i]:SetState(0, "action", i)
		for k = 1, 14 do
			bar.buttons[i]:SetState(k, "action", (k - 1) * 12 + i)
		end

		if i == 12 then
			bar.buttons[i]:SetState(12, "custom", AB.customExitButton)
		end
	end
	self:UpdateButtonConfig(bar, bar.bindButtons)

	if AB['barDefaults']['bar'..id].conditions:find("[form]") then
		bar:SetAttribute("hasTempBar", true)
	else
		bar:SetAttribute("hasTempBar", false)
	end

	bar:SetAttribute("_onstate-page", [[
		if HasTempShapeshiftActionBar() and self:GetAttribute("hasTempBar") then
			newstate = GetTempShapeshiftBarIndex() or newstate
		end

		if newstate ~= 0 then
			self:SetAttribute("state", newstate)
			control:ChildUpdate("state", newstate)
		else
			local newCondition = self:GetAttribute("newCondition")
			if newCondition then
				newstate = SecureCmdOptionParse(newCondition)
				self:SetAttribute("state", newstate)
				control:ChildUpdate("state", newstate)
			end
		end
	]]);


	self["handledBars"]['bar'..id] = bar;
	self:PositionAndSizeBar('bar'..id);
--	E:CreateMover(bar, 'ElvAB_'..id, "Bar "..id, nil, nil, nil,'ALL,ACTIONBARS')

	
	E:Mover(bar, 'AleaAB_'..bar.id)
	
	E.numActonBars = ( E.numActonBars or 0 ) + 1
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id] = {
		name = L['ActionBar']..bar.id,
		order = E.numActonBars,
		expand = true,
		type = "group",
		args = {}
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.unlock = {
		name = L["Unlock"],
		order = 2,
		type = "execute",
		set = function(self, value)
			E:UnlockMover('AleaAB_'..bar.id) 
		end,
		get = function(self)end
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.enabled = {
		name = L["Enable"],
		order = 1.1,
		type = "toggle",
		set = function(self, value)
			E.db.actionbars['bar'..id].enabled = not E.db.actionbars['bar'..id].enabled
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self)
			return E.db.actionbars['bar'..id].enabled
		end
	}
	
	--[[
	
		['enabled'] = true,
		['buttons'] = 12,
		['mouseover'] = false,
		['buttonsPerRow'] = 12,
		['point'] = 'BOTTOMLEFT',
		['backdrop'] = false,
		['heightMult'] = 1,
		['widthMult'] = 1,
		["buttonsize"] = 32,
		["buttonspacing"] = 2,
		['alpha'] = 1,
	]]
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.buttons = {
		name = L["Buttons"],
		order = 3,
		type = "slider",
		min = 1, max = 12, step = 1,
		set = function(self, value)
			E.db.actionbars['bar'..id].buttons = value
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self) 
			return E.db.actionbars['bar'..id].buttons
		end
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.perrow = {
		name = L["Per row"],
		order = 3.1,
		type = "slider",
		min = 1, max = 12, step = 1,
		set = function(self, value)
			E.db.actionbars['bar'..id].buttonsPerRow = value
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self) 
			return E.db.actionbars['bar'..id].buttonsPerRow
		end
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.size = {
		name = L["Size"],
		order = 4,
		type = "slider",
		min = 1, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['bar'..id].buttonsize = value
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self) 
			return E.db.actionbars['bar'..id].buttonsize
		end
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.spacing = {
		name = L["Spacing"],
		order = 6,
		type = "slider",
		min = 0, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['bar'..id].buttonspacing = value
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self) 
			return E.db.actionbars['bar'..id].buttonspacing
		end
	}
	
		
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.point = {
		name = L["Point"],
		type = "dropdown",
		order = 7,
		values = DD_points,
		set = function(self, value)
			E.db.actionbars['bar'..id].point = value
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(self)
			return E.db.actionbars['bar'..id].point
		end,	
	}
	
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts = {
		name = L["Borders"],
		order = 10,
		embend = true,
		type = "group",
		args = {}
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BorderTexture = {
		order = 1,
		type = 'border',
		name = L["Border texture"],
		values = E:GetBorderList(),
		set = function(info,value) 
			E.db.actionbars['bar'..id].border.texture = value;
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) return E.db.actionbars['bar'..id].border.texture end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BorderColor = {
		order = 2,
		name = L["Border color"],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['bar'..id].border.color={ r, g, b, a}; 
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) 
			return E.db.actionbars['bar'..id].border.color[1],
					E.db.actionbars['bar'..id].border.color[2],
					E.db.actionbars['bar'..id].border.color[3],
					E.db.actionbars['bar'..id].border.color[4] 
		end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BorderSize = {
		name = L['Border width'],
		type = "slider",
		order	= 3,
		min		= 1,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].border.size = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].border.size
		end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BorderInset = {
		name = L['Border inset'],
		type = "slider",
		order	= 4,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].border.inset = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].border.inset
		end,
	}


	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BackgroundTexture = {
		order = 5,
		type = 'statusbar',
		name = L["Background texture"],
		values = E.GetTextureList,
		set = function(info,value) 
			E.db.actionbars['bar'..id].border.background_texture = value;
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) return E.db.actionbars['bar'..id].border.background_texture end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.BackgroundColor = {
		order = 6,
		name = L['Background color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['bar'..id].border.background_color={ r, g, b, a}
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) 
			return E.db.actionbars['bar'..id].border.background_color[1],
					E.db.actionbars['bar'..id].border.background_color[2],
					E.db.actionbars['bar'..id].border.background_color[3],
					E.db.actionbars['bar'..id].border.background_color[4] 
		end,
	}


	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.BorderOpts.args.backgroundInset = {
		name = L['Background offset'],
		type = "slider",
		order	= 7,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].border.background_inset = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].border.background_inset
		end,
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts = {
		name = L['Artwork'],
		order = 11,
		embend = true,
		type = "group",
		args = {}
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.Enable = {
		name = L['Enable'],
		type = 'toggle',
		order = 0.1,
		width = 'full',
		set = function(info)
			E.db.actionbars['bar'..id].artwork.enable = not E.db.actionbars['bar'..id].artwork.enable
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info)
			return E.db.actionbars['bar'..id].artwork.enable
		end,	
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.Width = {
		name = L['Width'],
		type = "slider",
		order	= 0.2,
		min		= 1,
		max		= 500,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].artwork.width = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].artwork.width
		end,
	}
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.Height = {
		name = L['Height'],
		type = "slider",
		order	= 0.3,
		min		= 1,
		max		= 500,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].artwork.height = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].artwork.height
		end,
	}
	
	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BorderTexture = {
		order = 1,
		type = 'border',
		name = L["Border texture"],
		values = E:GetBorderList(),
		set = function(info,value) 
			E.db.actionbars['bar'..id].artwork.texture = value;
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) return E.db.actionbars['bar'..id].artwork.texture end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BorderColor = {
		order = 2,
		name = L['Border color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['bar'..id].artwork.color={ r, g, b, a}; 
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) 
			return E.db.actionbars['bar'..id].artwork.color[1],
					E.db.actionbars['bar'..id].artwork.color[2],
					E.db.actionbars['bar'..id].artwork.color[3],
					E.db.actionbars['bar'..id].artwork.color[4] 
		end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BorderSize = {
		name = L['Border size'],
		type = "slider",
		order	= 3,
		min		= 1,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].artwork.size = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].artwork.size
		end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BorderInset = {
		name = L['Border inset'],
		type = "slider",
		order	= 4,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].artwork.inset = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].artwork.inset
		end,
	}


	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BackgroundTexture = {
		order = 5,
		type = 'statusbar',
		name = L['Background texture'],
		values = E.GetTextureList,
		set = function(info,value) 
			E.db.actionbars['bar'..id].artwork.background_texture = value;
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) return E.db.actionbars['bar'..id].artwork.background_texture end,
	}

	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.BackgroundColor = {
		order = 6,
		name = L['Background color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['bar'..id].artwork.background_color={ r, g, b, a}
			AB:PositionAndSizeBar('bar'..id);
		end,
		get = function(info) 
			return E.db.actionbars['bar'..id].artwork.background_color[1],
					E.db.actionbars['bar'..id].artwork.background_color[2],
					E.db.actionbars['bar'..id].artwork.background_color[3],
					E.db.actionbars['bar'..id].artwork.background_color[4] 
		end,
	}


	E.GUI.args.actionbars.args['ActionBar'..bar.id].args.ArtWorkOpts.args.backgroundInset = {
		name = L['Background offset'],
		type = "slider",
		order	= 7,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['bar'..id].artwork.background_inset = val
			AB:PositionAndSizeBar('bar'..id);
		end,
		get =function(info)
			return E.db.actionbars['bar'..id].artwork.background_inset
		end,
	}
	
	return bar
end

function AB:PLAYER_REGEN_ENABLED()
	self:UpdateButtonSettings()
	self:UnregisterEvent('PLAYER_REGEN_ENABLED')
end

local function Vehicle_OnEvent(self, event)
	if ( CanExitVehicle() and ActionBarController_GetCurrentActionBarState() == LE_ACTIONBAR_STATE_MAIN ) then
		self:Show()
		self:GetNormalTexture():SetVertexColor(1, 1, 1)
		self:EnableMouse(true)
	else
		self:Hide()
	end
end

local function Vehicle_OnClick(self)
	if ( UnitOnTaxi("player") ) then
		TaxiRequestEarlyLanding();
		self:GetNormalTexture():SetVertexColor(1, 0, 0)
		self:EnableMouse(false)
	else
		VehicleExit();
	end
end

function AB:UpdateVehicleLeave()
	local button = LeaveVehicleButton
	if not button then return; end
	
	local pos = "BOTTOMLEFT" --E.db.general.minimap.icons.vehicleLeave.position or
	local size = 26 --E.db.general.minimap.icons.vehicleLeave.size or 
	button:ClearAllPoints()
	button:SetPoint(pos, Minimap, pos, 2, 2)
	--E.db.general.minimap.icons.vehicleLeave.xOffset or 
	--E.db.general.minimap.icons.vehicleLeave.yOffset or 
	button:SetSize(size, size)
end

function AB:CreateVehicleLeave()
	local vehicle = CreateFrame("Button", 'LeaveVehicleButton', Minimap)
	vehicle:SetSize(26,26)
	vehicle:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 2, 2)
	vehicle:SetNormalTexture("Interface\\AddOns\\AleaUI\\media\\vehicleexit")
	vehicle:SetPushedTexture("Interface\\AddOns\\AleaUI\\media\\vehicleexit")
	vehicle:SetHighlightTexture("Interface\\AddOns\\AleaUI\\media\\vehicleexit")
---	vehicle:SetTemplate("Default")

	Skins.SetTemplate(vehicle, 'DARK')
	vehicle:RegisterForClicks("AnyUp")
	vehicle:SetFrameLevel(Minimap:GetFrameLevel()+1)
	vehicle:SetScript("OnClick", Vehicle_OnClick)
	vehicle:SetScript("OnEnter", MainMenuBarVehicleLeaveButton_OnEnter)
	vehicle:SetScript("OnLeave", GameTooltip_Hide)
	vehicle:RegisterEvent("PLAYER_ENTERING_WORLD");
	vehicle:RegisterEvent("UPDATE_BONUS_ACTIONBAR");
	vehicle:RegisterEvent("UPDATE_MULTI_CAST_ACTIONBAR");
	vehicle:RegisterEvent("UNIT_ENTERED_VEHICLE");
	vehicle:RegisterEvent("UNIT_EXITED_VEHICLE");
	vehicle:RegisterEvent("VEHICLE_UPDATE");
	vehicle:SetScript("OnEvent", Vehicle_OnEvent)
	
	self:UpdateVehicleLeave()

	vehicle:Hide()
end

function AB:ReassignBindings(event)
	if event == "UPDATE_BINDINGS" then
		self:UpdatePetBindings();
		self:UpdateStanceBindings();
	end

	self:UnregisterEvent("PLAYER_REGEN_DISABLED")

	if InCombatLockdown() then return end
	for _, bar in pairs(self["handledBars"]) do
		if not bar then return end

		ClearOverrideBindings(bar)
		for i = 1, #bar.buttons do
			local button = (bar.bindButtons.."%d"):format(i)
			local real_button = (bar:GetName().."Button%d"):format(i)
			for k=1, select('#', GetBindingKey(button)) do
				local key = select(k, GetBindingKey(button))
				if key and key ~= "" then
					SetOverrideBindingClick(bar, false, key, real_button)
				end
			end
		end
	end
end

AB.UPDATE_BINDINGS = AB.ReassignBindings
AB.PLAYER_REGEN_DISABLED = AB.ReassignBindings
AB.PET_BATTLE_CLOSE = AB.ReassignBindings

function AB:RemoveBindings()
	if InCombatLockdown() then return end
	for _, bar in pairs(self["handledBars"]) do
		if not bar then return end

		ClearOverrideBindings(bar)
	end

	self:RegisterEvent("PLAYER_REGEN_DISABLED")
end

AB.PET_BATTLE_OPENING_DONE = AB.RemoveBindings

function AB:UpdateBar1Paging()
	if E.db.actionbars.bar6.enabled then
		AB.barDefaults.bar1.conditions = format("[vehicleui] %d; [possessbar] %d; [overridebar] %d; [shapeshift] 13; [form,noform] 0; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6;", GetVehicleBarIndex(), GetVehicleBarIndex(), GetOverrideBarIndex())
	else
		AB.barDefaults.bar1.conditions = format("[vehicleui] %d; [possessbar] %d; [overridebar] %d; [shapeshift] 13; [form,noform] 0; [bar:2] 2; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6;", GetVehicleBarIndex(), GetVehicleBarIndex(), GetOverrideBarIndex())
	end

	if (false or InCombatLockdown()) or not self.isInitialized then return; end
	local bar2Option = InterfaceOptionsActionBarsPanelBottomRight
	local bar3Option = InterfaceOptionsActionBarsPanelBottomLeft
	local bar4Option = InterfaceOptionsActionBarsPanelRightTwo
	local bar5Option = InterfaceOptionsActionBarsPanelRight

	if (E.db.actionbars.bar2.enabled and not bar2Option:GetChecked()) or (not E.db.actionbars.bar2.enabled and bar2Option:GetChecked())  then
		bar2Option:Click()
	end

	if (E.db.actionbars.bar3.enabled and not bar3Option:GetChecked()) or (not E.db.actionbars.bar3.enabled and bar3Option:GetChecked())  then
		bar3Option:Click()
	end

	if not E.db.actionbars.bar5.enabled and not E.db.actionbars.bar4.enabled then
		if bar4Option:GetChecked() then
			bar4Option:Click()
		end

		if bar5Option:GetChecked() then
			bar5Option:Click()
		end
	elseif not E.db.actionbars.bar5.enabled then
		if not bar5Option:GetChecked() then
			bar5Option:Click()
		end

		if not bar4Option:GetChecked() then
			bar4Option:Click()
		end
	elseif (E.db.actionbars.bar4.enabled and not bar4Option:GetChecked()) or (not E.db.actionbars.bar4.enabled and bar4Option:GetChecked()) then
		bar4Option:Click()
	elseif (E.db.actionbars.bar5.enabled and not bar5Option:GetChecked()) or (not E.db.actionbars.bar5.enabled and bar5Option:GetChecked()) then
		bar5Option:Click()
	end
end

function AB:UpdateButtonSettings()
	if false then return end
	if InCombatLockdown() then self:RegisterEvent('PLAYER_REGEN_ENABLED'); return; end
	for button, _ in pairs(self["handledbuttons"]) do
		if button then
			self:StyleButton(button, button.noBackdrop)
			self:StyleFlyout(button)
		else
			self["handledbuttons"][button] = nil
		end
	end

	for i=1, 6 do
		self:PositionAndSizeBar('bar'..i)
	end
	self:PositionAndSizeBarPet()
	self:PositionAndSizeBarShapeShift()
	self:UpdatePetBindings()
	self:UpdateStanceBindings()
	for barName, bar in pairs(self["handledBars"]) do
		self:UpdateButtonConfig(bar, bar.bindButtons)
	end
end

E.OnLSMUpdateRegister(function()
	AB:UpdateButtonSettings()
end)	

function AB:GetPage(bar, defaultPage, condition)
	local page = E.db.actionbars[bar]['paging'][E.myclass]
	if not condition then condition = '' end
	if not page then page = '' end
	if page then
		condition = condition.." "..page
	end
	condition = condition.." "..defaultPage

	return condition
end

function AB:VehicleBarArtWorkVisability()
	
	if UnitHasVehicleUI('player')  then
		self:Hide()
		self.back:Hide()
	else
		self:Show()
		self.back:Show()
	end
--	print(self, self.back, event, unit, UnitHasVehicleUI('player'),  UnitHasVehiclePlayerFrameUI('player'))
end

function AB:StyleButton(button, noBackdrop, adjustChecked)
	local name = button:GetName();
	local icon = _G[name.."Icon"];
	local count = _G[name.."Count"];
	local flash	 = _G[name.."Flash"];
	local hotkey = _G[name.."HotKey"];
	local border  = _G[name.."Border"];
	local macroName = _G[name.."Name"];
	local normal  = _G[name.."NormalTexture"];
	local normal2 = button:GetNormalTexture()
	local shine = _G[name.."Shine"];
	local combat = InCombatLockdown()

	if flash then flash:SetTexture(nil); end
	if normal then normal:SetTexture(nil); normal:Hide(); normal:SetAlpha(0); end
	if normal2 then normal2:SetTexture(nil); normal2:Hide(); normal2:SetAlpha(0); end
	if border then border:Kill(); end

	if not button.noBackdrop then
		button.noBackdrop = noBackdrop;
	end

	if count then
		count:ClearAllPoints();
		count:SetPoint("BOTTOMRIGHT", 0, 2);
		count:SetFont(E:GetFont(E.db.actionbars.font), E.db.actionbars.fontSize, E.db.actionbars.fontOutline)
	end
	
	button._point = button
	
	if not button.noBackdrop and not button.artBorder then
		button.artBorder = CreateFrame("Frame", nil, button)
		button.artBorder:SetFrameLevel(button:GetFrameLevel()+1)
		button.artBorder:SetBackdrop({
		  edgeFile = [[Interface\Buttons\WHITE8x8]],
		  edgeSize = 1, 
		})
		button.artBorder:SetBackdropBorderColor(0,0,0,1)
		button.artBorder:SetPoint("TOPLEFT", icon, "TOPLEFT", 0, -0)
		button.artBorder:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", -0, 0)
	
		button.artBorder.back = button:CreateTexture()
		button.artBorder.back:SetDrawLayer('BACKGROUND', -2)
		button.artBorder.back:SetColorTexture(0, 0, 0, 0)
		button.artBorder.back:SetPoint("TOPLEFT", icon, "TOPLEFT", 0, 0)
		button.artBorder.back:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", 0, 0)
		
	end
	
	if icon then
		icon:SetTexCoord(unpack(E.media.texCoord));
		icon:SetInside()
	end

	if shine then
		shine:SetInside(nil, 3, 3)
	end
	
	if E.db.actionbars.macrotext then
		macroName:SetParent(button.artBorder)
		macroName:SetFont(E:GetFont(E.db.actionbars.font), E.db.actionbars.fontSize, E.db.actionbars.fontOutline)
	end
	
	if E.db.actionbars.hotkeytext then
		hotkey:SetParent(button.artBorder)
		hotkey:SetFont(E:GetFont(E.db.actionbars.font), E.db.actionbars.fontSize, E.db.actionbars.fontOutline)
	end

	--Extra Action Button
	if button.style then
		--button.style:SetParent(button.backdrop)
		button.style:SetDrawLayer('BACKGROUND', -7)
	end

	button.FlyoutUpdateFunc = AB.StyleFlyout
	self:FixKeybindText(button);
	button:StyleButton();

	if(not self.handledbuttons[button]) then
		self.handledbuttons[button] = true;
	end
end

function AB:Bar_OnEnter(bar)
--	E:UIFrameFadeIn(bar, 0.2, bar:GetAlpha(), bar.db.alpha)
end

function AB:Bar_OnLeave(bar)
--	E:UIFrameFadeOut(bar, 0.2, bar:GetAlpha(), 0)
end

function AB:Button_OnEnter(button)
	local bar = button:GetParent()
--	E:UIFrameFadeIn(bar, 0.2, bar:GetAlpha(), bar.db.alpha)
end

function AB:Button_OnLeave(button)
	local bar = button:GetParent()
--	E:UIFrameFadeOut(bar, 0.2, bar:GetAlpha(), 0)
end

function AB:BlizzardOptionsPanel_OnEvent()
	InterfaceOptionsActionBarsPanelBottomRight.Text:SetText(format("Remove Bar %d Action Page", 2))
	InterfaceOptionsActionBarsPanelBottomLeft.Text:SetText(format("Remove Bar %d Action Page", 3))
	InterfaceOptionsActionBarsPanelRightTwo.Text:SetText(format("Remove Bar %d Action Page", 4))
	InterfaceOptionsActionBarsPanelRight.Text:SetText(format("Remove Bar %d Action Page", 5))

	InterfaceOptionsActionBarsPanelBottomRight:SetScript('OnEnter', nil)
	InterfaceOptionsActionBarsPanelBottomLeft:SetScript('OnEnter', nil)
	InterfaceOptionsActionBarsPanelRightTwo:SetScript('OnEnter', nil)
	InterfaceOptionsActionBarsPanelRight:SetScript('OnEnter', nil)
end

function AB:DisableBlizzard()
	-- Hidden parent frame
	local UIHider = CreateFrame("Frame")
	UIHider:Hide()
	
	if ArtifactWatchBar then
		ArtifactWatchBar:SetParent(UIHider)
	end
	
	MultiBarBottomLeft:SetParent(UIHider)
	MultiBarBottomRight:SetParent(UIHider)
	MultiBarLeft:SetParent(UIHider)
	MultiBarRight:SetParent(UIHider)

	-- Hide MultiBar Buttons, but keep the bars alive
	for i=1,12 do
		_G["ActionButton" .. i]:Hide()
		_G["ActionButton" .. i]:UnregisterAllEvents()
		_G["ActionButton" .. i]:SetAttribute("statehidden", true)

		_G["MultiBarBottomLeftButton" .. i]:Hide()
		_G["MultiBarBottomLeftButton" .. i]:UnregisterAllEvents()
		_G["MultiBarBottomLeftButton" .. i]:SetAttribute("statehidden", true)

		_G["MultiBarBottomRightButton" .. i]:Hide()
		_G["MultiBarBottomRightButton" .. i]:UnregisterAllEvents()
		_G["MultiBarBottomRightButton" .. i]:SetAttribute("statehidden", true)

		_G["MultiBarRightButton" .. i]:Hide()
		_G["MultiBarRightButton" .. i]:UnregisterAllEvents()
		_G["MultiBarRightButton" .. i]:SetAttribute("statehidden", true)

		_G["MultiBarLeftButton" .. i]:Hide()
		_G["MultiBarLeftButton" .. i]:UnregisterAllEvents()
		_G["MultiBarLeftButton" .. i]:SetAttribute("statehidden", true)

		if _G["VehicleMenuBarActionButton" .. i] then
			_G["VehicleMenuBarActionButton" .. i]:Hide()
			_G["VehicleMenuBarActionButton" .. i]:UnregisterAllEvents()
			_G["VehicleMenuBarActionButton" .. i]:SetAttribute("statehidden", true)
		end

		if _G['OverrideActionBarButton'..i] then
			_G['OverrideActionBarButton'..i]:Hide()
			_G['OverrideActionBarButton'..i]:UnregisterAllEvents()
			_G['OverrideActionBarButton'..i]:SetAttribute("statehidden", true)
		end

		_G['MultiCastActionButton'..i]:Hide()
		_G['MultiCastActionButton'..i]:UnregisterAllEvents()
		_G['MultiCastActionButton'..i]:SetAttribute("statehidden", true)
	end

	ActionBarController:UnregisterAllEvents()
	ActionBarController:RegisterEvent('UPDATE_EXTRA_ACTIONBAR')

	MainMenuBar:EnableMouse(false)
	MainMenuBar:SetAlpha(0)
	MainMenuExpBar:UnregisterAllEvents()
	MainMenuExpBar:Hide()
	MainMenuExpBar:SetParent(UIHider)

	for i=1, MainMenuBar:GetNumChildren() do
		local child = select(i, MainMenuBar:GetChildren())
		if child then
			child:UnregisterAllEvents()
			child:Hide()
			child:SetParent(UIHider)
		end
	end

	ReputationWatchBar:UnregisterAllEvents()
	ReputationWatchBar:Hide()
	ReputationWatchBar:SetParent(UIHider)

	MainMenuBarArtFrame:UnregisterEvent("ACTIONBAR_PAGE_CHANGED")
	MainMenuBarArtFrame:UnregisterEvent("ADDON_LOADED")
	MainMenuBarArtFrame:Hide()
	MainMenuBarArtFrame:SetParent(UIHider)

	StanceBarFrame:UnregisterAllEvents()
	StanceBarFrame:Hide()
	StanceBarFrame:SetParent(UIHider)

	OverrideActionBar:UnregisterAllEvents()
	OverrideActionBar:Hide()
	OverrideActionBar:SetParent(UIHider)

	PossessBarFrame:UnregisterAllEvents()
	PossessBarFrame:Hide()
	PossessBarFrame:SetParent(UIHider)

	PetActionBarFrame:UnregisterAllEvents()
	PetActionBarFrame:Hide()
	PetActionBarFrame:SetParent(UIHider)

	MultiCastActionBarFrame:UnregisterAllEvents()
	MultiCastActionBarFrame:Hide()
	MultiCastActionBarFrame:SetParent(UIHider)

	--This frame puts spells on the damn actionbar, fucking obliterate that shit
	IconIntroTracker:UnregisterAllEvents()
	IconIntroTracker:Hide()
	IconIntroTracker:SetParent(UIHider)
	
	InterfaceOptionsActionBarsPanelAlwaysShowActionBars:EnableMouse(false)
	InterfaceOptionsActionBarsPanelPickupActionKeyDropDownButton:SetScale(0.0001)
	InterfaceOptionsActionBarsPanelLockActionBars:SetScale(0.0001)
	InterfaceOptionsActionBarsPanelAlwaysShowActionBars:SetAlpha(0)
	InterfaceOptionsActionBarsPanelPickupActionKeyDropDownButton:SetAlpha(0)
	InterfaceOptionsActionBarsPanelLockActionBars:SetAlpha(0)
	InterfaceOptionsActionBarsPanelPickupActionKeyDropDown:SetAlpha(0)
	InterfaceOptionsActionBarsPanelPickupActionKeyDropDown:SetScale(0.00001)

	hooksecurefunc('BlizzardOptionsPanel_OnEvent', AB.BlizzardOptionsPanel_OnEvent)
--	InterfaceOptionsFrameCategoriesButton6:SetScale(0.00001)
	if PlayerTalentFrame then
		PlayerTalentFrame:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	else
		hooksecurefunc("TalentFrame_LoadUI", function() PlayerTalentFrame:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED") end)
	end
end

function AB:UpdateButtonConfig(bar, buttonName)
	if InCombatLockdown() then self:RegisterEvent('PLAYER_REGEN_ENABLED'); return; end
	if not bar.buttonConfig then bar.buttonConfig = { hideElements = {}, colors = {} } end
	bar.buttonConfig.hideElements.macro = not E.db.actionbars.macrotext
	bar.buttonConfig.hideElements.hotkey = not E.db.actionbars.hotkeytext
	bar.buttonConfig.showGrid = E.db.actionbars.showGrid
	bar.buttonConfig.clickOnDown = E.db.actionbars.keyDown
	SetModifiedClick("PICKUPACTION", E.db.actionbars.movementModifier)
	bar.buttonConfig.colors.range = { E.db.actionbars.noRangeColor.r, E.db.actionbars.noRangeColor.g, E.db.actionbars.noRangeColor.b }
	bar.buttonConfig.colors.mana = { E.db.actionbars.noPowerColor.r, E.db.actionbars.noPowerColor.g, E.db.actionbars.noPowerColor.b }
	bar.buttonConfig.colors.hp = { E.db.actionbars.noPowerColor.r, E.db.actionbars.noPowerColor.g, E.db.actionbars.noPowerColor.b }

	for i, button in pairs(bar.buttons) do
		bar.buttonConfig.keyBoundTarget = format(buttonName.."%d", i)
		button.keyBoundTarget = bar.buttonConfig.keyBoundTarget
		button.postKeybind = AB.FixKeybindText
		button:SetAttribute("buttonlock", true)
		button:SetAttribute("checkselfcast", true)
		button:SetAttribute("checkfocuscast", true)

		button:UpdateConfig(bar.buttonConfig)
	end
end

function AB:FixKeybindText(button)
	local hotkey = _G[button:GetName()..'HotKey'];
	local text = hotkey:GetText();

	if text then
		text = gsub(text, 'SHIFT%-',"S");
		text = gsub(text, 'ALT%-', "A");
		text = gsub(text, 'CTRL%-', "C");
		text = gsub(text, 'BUTTON', "MB");
		text = gsub(text, 'MOUSEWHEELUP', "MWU");
		text = gsub(text, 'MOUSEWHEELDOWN', "MWD");
		text = gsub(text, 'NUMPAD', "NP");
		text = gsub(text, 'PAGEUP', "PU");
		text = gsub(text, 'PAGEDOWN', "PD");
		text = gsub(text, 'SPACE', "SP");
		text = gsub(text, 'INSERT', "INS");
		text = gsub(text, 'HOME', "HME");
		text = gsub(text, 'DELETE', "DLT");
		text = gsub(text, 'NMULTIPLY', "*");
		text = gsub(text, 'NMINUS', "N-");
		text = gsub(text, 'NPLUS', "N+");

		hotkey:SetText(text);
	end

	hotkey:ClearAllPoints()
	hotkey:SetPoint("TOPRIGHT", 0, -3);
end

local buttons = 0
local function SetupFlyoutButton()
	for i=1, buttons do
		--prevent error if you don't have max amount of buttons
		if _G["SpellFlyoutButton"..i] then
			AB:StyleButton(_G["SpellFlyoutButton"..i])
			_G["SpellFlyoutButton"..i]:StyleButton()
			_G["SpellFlyoutButton"..i]:HookScript('OnEnter', function(self)
				local parent = self:GetParent()
				local parentAnchorButton = select(2, parent:GetPoint())
				if not AB["handledbuttons"][parentAnchorButton] then return end

				local parentAnchorBar = parentAnchorButton:GetParent()
				if parentAnchorBar.mouseover then
					AB:Bar_OnEnter(parentAnchorBar)
				end
			end)
			_G["SpellFlyoutButton"..i]:HookScript('OnLeave', function(self)
				local parent = self:GetParent()
				local parentAnchorButton = select(2, parent:GetPoint())
				if not AB["handledbuttons"][parentAnchorButton] then return end

				local parentAnchorBar = parentAnchorButton:GetParent()

				if parentAnchorBar.mouseover then
					AB:Bar_OnLeave(parentAnchorBar)
				end
			end)
		end
	end

	SpellFlyout:HookScript('OnEnter', function(self)
		local anchorButton = select(2, self:GetPoint())
		if not AB["handledbuttons"][anchorButton] then return end

		local parentAnchorBar = anchorButton:GetParent()
		if parentAnchorBar.mouseover then
			AB:Bar_OnEnter(parentAnchorBar)
		end
	end)

	SpellFlyout:HookScript('OnLeave', function(self)
		local anchorButton = select(2, self:GetPoint())
		if not AB["handledbuttons"][anchorButton] then return end

		local parentAnchorBar = anchorButton:GetParent()
		if parentAnchorBar.mouseover then
			AB:Bar_OnLeave(parentAnchorBar)
		end
	end)
end

function AB:StyleFlyout(button)
	if(not button.FlyoutArrow or not button.FlyoutArrow:IsShown()) then return end

	if not LAB.buttonRegistry[button] then return end
	if not button.FlyoutBorder then return end
	local combat = InCombatLockdown()

	button.FlyoutBorder:SetAlpha(0)
	button.FlyoutBorderShadow:SetAlpha(0)

	SpellFlyoutHorizontalBackground:SetAlpha(0)
	SpellFlyoutVerticalBackground:SetAlpha(0)
	SpellFlyoutBackgroundEnd:SetAlpha(0)

	for i=1, GetNumFlyouts() do
		local x = GetFlyoutID(i)
		local _, _, numSlots, isKnown = GetFlyoutInfo(x)
		if isKnown then
			buttons = numSlots
			break
		end
	end

	--Change arrow direction depending on what bar the button is on
	local arrowDistance
	if ((SpellFlyout:IsShown() and SpellFlyout:GetParent() == button) or GetMouseFocus() == button) then
		arrowDistance = 5
	else
		arrowDistance = 2
	end

	if button:GetParent() and button:GetParent():GetParent() and button:GetParent():GetParent():GetName() and button:GetParent():GetParent():GetName() == "SpellBookSpellIconsFrame" then
		return
	end

	if button:GetParent() then
		local point = 'BOTTOM' --E:GetScreenQuadrant(button:GetParent())
		if point == "UNKNOWN" then return end
		
		button.FlyoutArrow:SetParent(button.artBorder)
		
		if strfind(point, "TOP") then
			button.FlyoutArrow:ClearAllPoints()
			button.FlyoutArrow:SetPoint("BOTTOM", button, "BOTTOM", 0, -arrowDistance)
			SetClampedTextureRotation(button.FlyoutArrow, 180)
			if not combat then button:SetAttribute("flyoutDirection", "DOWN") end
		elseif point == "RIGHT" then
			button.FlyoutArrow:ClearAllPoints()
			button.FlyoutArrow:SetPoint("LEFT", button, "LEFT", -arrowDistance, 0)
			SetClampedTextureRotation(button.FlyoutArrow, 270)
			if not combat then button:SetAttribute("flyoutDirection", "LEFT") end
		elseif point == "LEFT" then
			button.FlyoutArrow:ClearAllPoints()
			button.FlyoutArrow:SetPoint("RIGHT", button, "RIGHT", arrowDistance, 0)
			SetClampedTextureRotation(button.FlyoutArrow, 90)
			if not combat then button:SetAttribute("flyoutDirection", "RIGHT") end
		elseif point == "CENTER" or strfind(point, "BOTTOM") then
			button.FlyoutArrow:ClearAllPoints()
			button.FlyoutArrow:SetPoint("TOP", button, "TOP", 0, arrowDistance)
			SetClampedTextureRotation(button.FlyoutArrow, 0)
			if not combat then button:SetAttribute("flyoutDirection", "UP") end
		end
	end
end

function AB:VehicleFix()
	local barName = 'bar1'
	local bar = self["handledBars"][barName]
	local spacing = E.db.actionbars[barName].buttonspacing;
	local numButtons = E.db.actionbars[barName].buttons;
	local buttonsPerRow = E.db.actionbars[barName].buttonsPerRow;
	local size = E.db.actionbars[barName].buttonsize;
	local point = E.db.actionbars[barName].point;
	local numColumns = ceil(numButtons / buttonsPerRow);

	if (HasOverrideActionBar() or HasVehicleActionBar()) and numButtons == 12 then
		local widthMult = 1;
		local heightMult = 1;

	--	bar.backdrop:ClearAllPoints()
	--	bar.backdrop:SetPoint(E.db.actionbars[barName].point, bar, E.db.actionbars[barName].point)
	--	bar.backdrop:SetWidth(spacing + ((size * (buttonsPerRow * widthMult)) + ((spacing * (buttonsPerRow - 1)) * widthMult) + (spacing * widthMult)));
	--	bar.backdrop:SetHeight(spacing + ((size * (numColumns * heightMult)) + ((spacing * (numColumns - 1)) * heightMult) + (spacing * heightMult)));
	else
	--	bar.backdrop:SetAllPoints()
	end
end

AB.UPDATE_VEHICLE_ACTIONBAR = AB.VehicleFix
AB.UPDATE_OVERRIDE_ACTIONBAR = AB.VehicleFix
	
function AB:ActionButton_ShowOverlayGlow(frame)
	if not frame.overlay then return; end
	local size =  frame:GetWidth() / 3
	frame.overlay:SetOutside(frame, size, size)
end

function AB:Initialize()

	AB:DisableBlizzard()

--	AB:SetupExtraButton()
--	AB:SetupMicroBar()
	AB:UpdateBar1Paging()

	for i=1, 6 do
		AB:CreateBar(i)
	end
	AB:CreateBarPet()
	AB:CreateBarShapeShift()
	AB:CreateVehicleLeave()

	AB:UpdateButtonSettings()

	AB:LoadKeyBinder()
	AB:RegisterEvent("UPDATE_BINDINGS", "ReassignBindings")
	AB:RegisterEvent("PET_BATTLE_CLOSE", "ReassignBindings")
	AB:RegisterEvent('PET_BATTLE_OPENING_DONE', 'RemoveBindings')
	AB:RegisterEvent('UPDATE_VEHICLE_ACTIONBAR', 'VehicleFix')
	AB:RegisterEvent('UPDATE_OVERRIDE_ACTIONBAR', 'VehicleFix')

	if C_PetBattles.IsInBattle() then
		AB:RemoveBindings()
	else
		AB:ReassignBindings()
	end

	if not GetCVarBool('lockActionBars') then
		SetCVar('lockActionBars', 1)
	end

	SpellFlyout:HookScript("OnShow", SetupFlyoutButton)
	
	E.GUI.args.actionbars.args.FontGroup = {
		name = L["Fonts"],
		type = "group",
		order = 5,
		embend = true,
		args = {},	
	}
	
	E.GUI.args.actionbars.args.FontGroup.args.Font = {	
		name = L["Font"],
		order = 1,
		type = "font",
		values = E.GetFontList,
		set = function(self, value)
			E.db.actionbars.font = value
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.font
		end,
	}
	
	E.GUI.args.actionbars.args.FontGroup.args.Size = {
		name = L["Size"],
		order = 2,
		type = "slider",
		min = 1, max = 32, step = 1,
		set = function(self, value)
			E.db.actionbars.fontSize = value
			AB:UpdateButtonSettings()
		end,
		get = function(self) 
			return E.db.actionbars.fontSize
		end
	}
	
	E.GUI.args.actionbars.args.FontGroup.args.Flags = {
		name = L["Outline"],
		type = "dropdown",
		order = 3,
		values = {
			['NONE'] = NO,
			['OUTLINE'] = L["Outline"],
		},
		set = function(self, value)			
			E.db.actionbars.fontOutline = value
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.fontOutline
		end,	
	}
	
	E.GUI.args.actionbars.args.MacroText = {
		name = L["Macro name"],
		type = "toggle",
		order = 1,
		set = function(self, value)
			E.db.actionbars.macrotext = not E.db.actionbars.macrotext
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.macrotext
		end,	
	}
	
	E.GUI.args.actionbars.args.HotKeyText = {
		name = L["Key name"],
		type = "toggle",
		order = 2,
		set = function(self, value)
			E.db.actionbars.hotkeytext = not E.db.actionbars.hotkeytext
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.hotkeytext
		end,	
	}
	
	E.GUI.args.actionbars.args.keyDown = {
		name = L["Trigger on key down"],
		type = "toggle",
		order = 3,
		set = function(self, value)
			E.db.actionbars.keyDown = not E.db.actionbars.keyDown
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.keyDown
		end,	
	}
	
	E.GUI.args.actionbars.args.noRangeColor = {
		name = L['"No range" color'],
		order = 3.6,
		type = "color",
		hasAlpha = false,		
		set = function(self, r,g,b,a)
			E.db.actionbars.noRangeColor = { r=r, g=g, b=b, 1 }
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.noRangeColor.r, 
			E.db.actionbars.noRangeColor.g, 
			E.db.actionbars.noRangeColor.b,
			E.db.actionbars.noRangeColor.a or 1
		end,
	}
	
	E.GUI.args.actionbars.args.noPowerColor = {
		name = L['"No power" color'],
		order = 3.7,
		type = "color",
		hasAlpha = false,		
		set = function(self, r,g,b,a)
			E.db.actionbars.noPowerColor = { r=r, g=g, b=b, 1 }
			AB:UpdateButtonSettings()
		end,
		get = function(self)
			return E.db.actionbars.noPowerColor.r, 
			E.db.actionbars.noPowerColor.g, 
			E.db.actionbars.noPowerColor.b,
			E.db.actionbars.noPowerColor.a or 1
		end,
	}

end

E:OnInit2(AB.Initialize)

local bar = CreateFrame('Frame', 'AleaUI_BarPet', E.UIParent, 'SecureHandlerStateTemplate');

function AB:UpdatePet(event, unit)
	if(event == "UNIT_AURA" and unit ~= "pet") then return end

	for i=1, NUM_PET_ACTION_SLOTS, 1 do
		local buttonName = "PetActionButton"..i;
		local button = _G[buttonName];
		local icon = _G[buttonName.."Icon"];
		local autoCast = _G[buttonName.."AutoCastable"];
		local shine = _G[buttonName.."Shine"];
		local checked = button:GetCheckedTexture();
		local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(i);

		if not isToken then
			icon:SetTexture(texture);
			button.tooltipName = name;
		else
			icon:SetTexture(_G[texture]);
			button.tooltipName = _G[name];
		end

		button.isToken = isToken;
		button.tooltipSubtext = subtext;

		if isActive and name ~= "PET_ACTION_FOLLOW" then
			--button:GetCheckedTexture():SetColorTexture(1, 1, 1)
			button:SetChecked(true);

			if IsPetAttackAction(i) then
				PetActionButton_StartFlash(button);
			end
		else
			--button:SetCheckedTexture("")
			button:SetChecked(false);
			if IsPetAttackAction(i) then
				PetActionButton_StopFlash(button);
			end
		end

		if autoCastAllowed then
			autoCast:Show();
		else
			autoCast:Hide();
		end

		if autoCastEnabled then
			AutoCastShine_AutoCastStart(shine);
		else
			AutoCastShine_AutoCastStop(shine);
		end

		button:SetAlpha(1);

		if texture then
			if GetPetActionSlotUsable(i) then
				SetDesaturation(icon, nil);
			else
				SetDesaturation(icon, 1);
			end
			icon:Show();
		else
			icon:Hide();
		end

		if not PetHasActionBar() and texture and name ~= "PET_ACTION_FOLLOW" then
			PetActionButton_StopFlash(button);
			SetDesaturation(icon, 1);
			button:SetChecked(false);
		end

		checked:SetAlpha(0.3)
	end
end


AB.SPELLS_CHANGED = AB.UpdatePet
AB.PLAYER_CONTROL_GAINED = AB.UpdatePet
AB.PLAYER_ENTERING_WORLD = AB.UpdatePet
AB.PLAYER_CONTROL_LOST = AB.UpdatePet
AB.PET_BAR_UPDATE = AB.UpdatePet
AB.UNIT_PET = AB.UpdatePet
AB.UNIT_FLAGS = AB.UpdatePet
AB.UNIT_AURA = AB.UpdatePet
AB.PLAYER_FARSIGHT_FOCUS_CHANGED = AB.UpdatePet
AB.PET_BAR_UPDATE_COOLDOWN = PetActionBar_UpdateCooldowns

function AB:PositionAndSizeBarPet()
	local spacing = E.db.actionbars['barPet'].buttonspacing;
	local buttonsPerRow = E.db.actionbars['barPet'].buttonsPerRow;
	local numButtons = E.db.actionbars['barPet'].buttons;
	local size = E.db.actionbars['barPet'].buttonsize;
	local autoCastSize = (size / 2) - (size / 7.5) + 1
	local point = E.db.actionbars['barPet'].point;
	local numColumns = ceil(numButtons / buttonsPerRow);
	local widthMult = E.db.actionbars['barPet'].widthMult;
	local heightMult = E.db.actionbars['barPet'].heightMult;
	bar.db = E.db.actionbars['barPet']
	bar.db.position = nil; --Depreciated
	if numButtons < buttonsPerRow then
		buttonsPerRow = numButtons;
	end

	if numColumns < 1 then
		numColumns = 1;
	end

	bar:SetWidth(spacing + ((size * (buttonsPerRow * widthMult)) + ((spacing * (buttonsPerRow - 1)) * widthMult) + (spacing * widthMult)));
	bar:SetHeight(spacing + ((size * (numColumns * heightMult)) + ((spacing * (numColumns - 1)) * heightMult) + (spacing * heightMult)));
	bar.mouseover = E.db.actionbars['barPet'].mouseover
	if E.db.actionbars['barPet'].enabled then
		bar:SetScale(1);
		bar:SetAlpha(bar.db.alpha);
	else
		bar:SetScale(0.000001);
		bar:SetAlpha(0);
	end
	--[[
	if E.db.actionbars['barPet'].backdrop == true then
		bar.backdrop:Show();
	else
		bar.backdrop:Hide();
	end
	]]
	local horizontalGrowth, verticalGrowth;
	if point == "TOPLEFT" or point == "TOPRIGHT" then
		verticalGrowth = "DOWN";
	else
		verticalGrowth = "UP";
	end

	if point == "BOTTOMLEFT" or point == "TOPLEFT" then
		horizontalGrowth = "RIGHT";
	else
		horizontalGrowth = "LEFT";
	end

	local button, lastButton, lastColumnButton, autoCast;
	for i=1, NUM_PET_ACTION_SLOTS do
		button = _G["PetActionButton"..i];

		lastButton = _G["PetActionButton"..i-1];
		autoCast = _G["PetActionButton"..i..'AutoCastable'];
		lastColumnButton = _G["PetActionButton"..i-buttonsPerRow];
		button:SetParent(bar);
		button:ClearAllPoints();
		button:SetSize(size, size);

		autoCast:SetOutside(button, autoCastSize, autoCastSize)

		button:SetAttribute("showgrid", 1);
	--[[
		if E.db.actionbars['barPet'].mouseover == true then
			bar:SetAlpha(0);
			if not self.hooks[bar] then
				self:HookScript(bar, 'OnEnter', 'Bar_OnEnter');
				self:HookScript(bar, 'OnLeave', 'Bar_OnLeave');
			end

			if not self.hooks[button] then
				self:HookScript(button, 'OnEnter', 'Button_OnEnter');
				self:HookScript(button, 'OnLeave', 'Button_OnLeave');
			end
		else
			bar:SetAlpha(bar.db.alpha);
			if self.hooks[bar] then
				self:Unhook(bar, 'OnEnter');
				self:Unhook(bar, 'OnLeave');
			end

			if self.hooks[button] then
				self:Unhook(button, 'OnEnter');
				self:Unhook(button, 'OnLeave');
			end
		end
	]]
		if i == 1 then
			local x, y;
			if point == "BOTTOMLEFT" then
				x, y = spacing, spacing;
			elseif point == "TOPRIGHT" then
				x, y = -spacing, -spacing;
			elseif point == "TOPLEFT" then
				x, y = spacing, -spacing;
			else
				x, y = -spacing, spacing;
			end

			button:SetPoint(point, bar, point, x, y);
		elseif (i - 1) % buttonsPerRow == 0 then
			local x = 0;
			local y = -spacing;
			local buttonPoint, anchorPoint = "TOP", "BOTTOM";
			if verticalGrowth == 'UP' then
				y = spacing;
				buttonPoint = "BOTTOM";
				anchorPoint = "TOP";
			end
			button:SetPoint(buttonPoint, lastColumnButton, anchorPoint, x, y);
		else
			local x = spacing;
			local y = 0;
			local buttonPoint, anchorPoint = "LEFT", "RIGHT";
			if horizontalGrowth == 'LEFT' then
				x = -spacing;
				buttonPoint = "RIGHT";
				anchorPoint = "LEFT";
			end

			button:SetPoint(buttonPoint, lastButton, anchorPoint, x, y);
		end

		if i > numButtons then
			button:SetScale(0.000001);
			button:SetAlpha(0);
		else
			button:SetScale(1);
			button:SetAlpha(bar.db.alpha);
		end

		self:StyleButton(button);
		
		local opts = E.db.actionbars['barPet']
		
		button.artBorder:SetBackdrop({
		  edgeFile = E:GetBorder(opts.border.texture),
		  edgeSize = opts.border.size, 
		})
		button.artBorder:SetBackdropBorderColor(opts.border.color[1],opts.border.color[2],opts.border.color[3],opts.border.color[4])
		button.artBorder:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.inset, -opts.border.inset)
		button.artBorder:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.inset, opts.border.inset)

		button.artBorder.back:SetTexture(E:GetTexture(opts.border.background_texture))
		button.artBorder.back:SetVertexColor(opts.border.background_color[1],opts.border.background_color[2],opts.border.background_color[3],opts.border.background_color[4])
		button.artBorder.back:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.background_inset, -opts.border.background_inset)
		button.artBorder.back:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.background_inset, opts.border.background_inset)
		
		--wtf lol
		if not button.CheckFixed then
			hooksecurefunc(button:GetCheckedTexture(), 'SetAlpha', function(self, value)
				if value == 1 then
					self:SetAlpha(0.3)
				end
			end)
			button.CheckFixed = true;
		end
	end
	initial = false
	RegisterStateDriver(bar, "show", E.db.actionbars['barPet'].visibility);
end

function AB:UpdatePetBindings()
	for i=1, NUM_PET_ACTION_SLOTS do
		if E.db.actionbars.hotkeytext then
			local key = GetBindingKey("BONUSACTIONBUTTON"..i)
			_G["PetActionButton"..i.."HotKey"]:Show()
			_G["PetActionButton"..i.."HotKey"]:SetText(key)
			self:FixKeybindText(_G["PetActionButton"..i])
		else
			_G["PetActionButton"..i.."HotKey"]:Hide()
		end
	end
end

function AB:CreateBarPet()
--	bar:CreateBackdrop('Default');
--	bar.backdrop:SetAllPoints();
	if E.db.actionbars['bar4'].enabled then
		bar:SetPoint('RIGHT', AleaUI_Bar4, 'LEFT', -4, 0);
	else
		bar:SetPoint('RIGHT', E.UIParent, 'RIGHT', -4, 0);
	end

	bar:SetAttribute("_onstate-show", [[
		if newstate == "hide" then
			self:Hide();
		else
			self:Show();
		end
	]]);

	PetActionBarFrame.showgrid = 1;
	PetActionBar_ShowGrid();

	self:RegisterEvent('SPELLS_CHANGED')
	self:RegisterEvent('PLAYER_CONTROL_GAINED');
	self:RegisterEvent('PLAYER_ENTERING_WORLD');
	self:RegisterEvent('PLAYER_CONTROL_LOST');
	self:RegisterEvent('PET_BAR_UPDATE');
	self:RegisterEvent('UNIT_PET');
	self:RegisterEvent('UNIT_FLAGS');
	self:RegisterEvent('UNIT_AURA');
	self:RegisterEvent('PLAYER_FARSIGHT_FOCUS_CHANGED');
	self:RegisterEvent('PET_BAR_UPDATE_COOLDOWN');


	E:Mover(bar, 'PetAB');
	self:PositionAndSizeBarPet();
	self:UpdatePetBindings()
	
	E.numActonBars = ( E.numActonBars or 0 ) + 1
	
	E.GUI.args.actionbars.args['PetActionBar'] = {
		name = L['PetActionBar'],
		order = E.numActonBars,
		expand = true,
		type = "group",
		args = {}
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.unlock = {
		name = L["Unlock"],
		order = 2,
		type = "execute",
		set = function(self, value)
			E:UnlockMover('PetAB') 
		end,
		get = function(self)end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.enabled = {
		name = L['Enable'],
		order = 1.1,
		type = "toggle",
		set = function(self, value)
			E.db.actionbars['barPet'].enabled = not E.db.actionbars['barPet'].enabled
			AB:PositionAndSizeBarPet();
		end,
		get = function(self)
			return E.db.actionbars['barPet'].enabled
		end
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.perrow = {
		name = L["Per row"],
		order = 3.1,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['barPet'].buttonsPerRow = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self) 
			return E.db.actionbars['barPet'].buttonsPerRow
		end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.buttons = {
		name = L["Buttons"],
		order = 3,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['barPet'].buttons = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self) 
			return E.db.actionbars['barPet'].buttons
		end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.perrow = {
		name = L["Per row"],
		order = 3.1,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['barPet'].buttonsPerRow = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self) 
			return E.db.actionbars['barPet'].buttonsPerRow
		end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.size = {
		name = L["Size"],
		order = 4,
		type = "slider",
		min = 1, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['barPet'].buttonsize = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self) 
			return E.db.actionbars['barPet'].buttonsize
		end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.spacing = {
		name = L["Spacing"],
		order = 6,
		type = "slider",
		min = 0, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['barPet'].buttonspacing = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self) 
			return E.db.actionbars['barPet'].buttonspacing
		end
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.point = {
		name = L["Point"],
		type = "dropdown",
		order = 7,
		values = DD_points,
		set = function(self, value)
			E.db.actionbars['barPet'].point = value
			AB:PositionAndSizeBarPet();
		end,
		get = function(self)
			return E.db.actionbars['barPet'].point
		end,	
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts = {
		name = L["Borders"],
		order = 10,
		embend = true,
		type = "group",
		args = {}
	}
	
	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BorderTexture = {
		order = 1,
		type = 'border',
		name = L['Border texture'],
		values = E:GetBorderList(),
		set = function(info,value) 
			E.db.actionbars['barPet'].border.texture = value;
			AB:PositionAndSizeBarPet();
		end,
		get = function(info) return E.db.actionbars['barPet'].border.texture end,
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BorderColor = {
		order = 2,
		name = L['Border color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['barPet'].border.color={ r, g, b, a}; 
			AB:PositionAndSizeBarPet();
		end,
		get = function(info) 
			return E.db.actionbars['barPet'].border.color[1],
					E.db.actionbars['barPet'].border.color[2],
					E.db.actionbars['barPet'].border.color[3],
					E.db.actionbars['barPet'].border.color[4] 
		end,
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BorderSize = {
		name = L['Border size'],
		type = "slider",
		order	= 3,
		min		= 1,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['barPet'].border.size = val
			AB:PositionAndSizeBarPet();
		end,
		get =function(info)
			return E.db.actionbars['barPet'].border.size
		end,
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BorderInset = {
		name = L['Border inset'],
		type = "slider",
		order	= 4,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['barPet'].border.inset = val
			AB:PositionAndSizeBarPet();
		end,
		get =function(info)
			return E.db.actionbars['barPet'].border.inset
		end,
	}


	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BackgroundTexture = {
		order = 5,
		type = 'statusbar',
		name = L['Background texture'],
		values = E.GetTextureList,
		set = function(info,value) 
			E.db.actionbars['barPet'].border.background_texture = value;
			AB:PositionAndSizeBarPet();
		end,
		get = function(info) return E.db.actionbars['barPet'].border.background_texture end,
	}

	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.BackgroundColor = {
		order = 6,
		name = L['Background color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['barPet'].border.background_color={ r, g, b, a}
			AB:PositionAndSizeBarPet();
		end,
		get = function(info) 
			return E.db.actionbars['barPet'].border.background_color[1],
					E.db.actionbars['barPet'].border.background_color[2],
					E.db.actionbars['barPet'].border.background_color[3],
					E.db.actionbars['barPet'].border.background_color[4] 
		end,
	}


	E.GUI.args.actionbars.args['PetActionBar'].args.BorderOpts.args.backgroundInset = {
		name = L['Background offset'],
		type = "slider",
		order	= 7,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['barPet'].border.background_inset = val
			AB:PositionAndSizeBarPet();
		end,
		get =function(info)
			return E.db.actionbars['barPet'].border.background_inset
		end,
	}
end


local bar = CreateFrame('Frame', 'AleaUI_StanceBar', E.UIParent, 'SecureHandlerStateTemplate');

function AB:UPDATE_SHAPESHIFT_COOLDOWN()
	local numForms = GetNumShapeshiftForms();
	local start, duration, enable, cooldown
	for i = 1, NUM_STANCE_SLOTS do
		if i <= numForms then
			cooldown = _G["AleaUI_StanceBarButton"..i.."Cooldown"];
			start, duration, enable = GetShapeshiftFormCooldown(i);
			CooldownFrame_SetTimer(cooldown, start, duration, enable);
			cooldown:SetDrawBling(cooldown:GetEffectiveAlpha() > 0.5) --Cooldown Bling Fix
		end
	end

	self:StyleShapeShift("UPDATE_SHAPESHIFT_COOLDOWN")
end

function AB:StyleShapeShift(event)
	local numForms = GetNumShapeshiftForms();
	local texture, name, isActive, isCastable, _;
	local buttonName, button, icon, cooldown;
	local stance = GetShapeshiftForm();

	for i = 1, NUM_STANCE_SLOTS do
		buttonName = "AleaUI_StanceBarButton"..i;
		button = _G[buttonName];
		icon = _G[buttonName.."Icon"];
		cooldown = _G[buttonName.."Cooldown"];

		if i <= numForms then
			texture, name, isActive, isCastable = GetShapeshiftFormInfo(i);

			if not texture then
				texture = "Interface\\Icons\\Spell_Nature_WispSplode"
			end

			if (type(texture) == "string" and (lower(texture) == "interface\\icons\\spell_nature_wispsplode" or lower(texture) == "interface\\icons\\ability_rogue_envelopingshadows")) and E.db.actionbars.stanceBar.style == 'darkenInactive' then
				_, _, texture = GetSpellInfo(name)
			end

			icon:SetTexture(texture);

			if texture then
				cooldown:SetAlpha(1);
			else
				cooldown:SetAlpha(0);
			end

			if isActive then
				StanceBarFrame.lastSelected = button:GetID();
				if numForms == 1 then
					button.checked:SetColorTexture(1, 1, 1, 0.5)
					button:SetChecked(true);
				else
					button.checked:SetColorTexture(1, 1, 1, 0.5)
					button:SetChecked(E.db.actionbars.stanceBar.style ~= 'darkenInactive');
				end
			else
				if numForms == 1 or stance == 0 then
					button:SetChecked(false);
				else
					button:SetChecked(E.db.actionbars.stanceBar.style == 'darkenInactive');
					button.checked:SetAlpha(1)
					if E.db.actionbars.stanceBar.style == 'darkenInactive' then
						button.checked:SetColorTexture(0, 0, 0, 0.5)
					else
						button.checked:SetColorTexture(1, 1, 1, 0.5)
					end
				end
			end

			if isCastable then
				icon:SetVertexColor(1.0, 1.0, 1.0);
			else
				icon:SetVertexColor(0.4, 0.4, 0.4);
			end
		end
	end
end

function AB:PositionAndSizeBarShapeShift()
	local spacing = E.db.actionbars['stanceBar'].buttonspacing
	local buttonsPerRow = E.db.actionbars['stanceBar'].buttonsPerRow;
	local numButtons = E.db.actionbars['stanceBar'].buttons;
	local size = E.db.actionbars['stanceBar'].buttonsize
	local point = E.db.actionbars['stanceBar'].point;
	local widthMult = E.db.actionbars['stanceBar'].widthMult;
	local heightMult = E.db.actionbars['stanceBar'].heightMult;

	bar.db = E.db.actionbars['stanceBar']
	bar.db.position = nil; --Depreciated
	if bar.LastButton and numButtons > bar.LastButton then
		numButtons = bar.LastButton;
	end

	if bar.LastButton and buttonsPerRow > bar.LastButton then
		buttonsPerRow = bar.LastButton;
	end

	if numButtons < buttonsPerRow then
		buttonsPerRow = numButtons;
	end

	local numColumns = ceil(numButtons / buttonsPerRow);
	if numColumns < 1 then
		numColumns = 1;
	end

	bar:SetWidth(spacing + ((size * (buttonsPerRow * widthMult)) + ((spacing * (buttonsPerRow - 1)) * widthMult) + (spacing * widthMult)));
	bar:SetHeight(spacing + ((size * (numColumns * heightMult)) + ((spacing * (numColumns - 1)) * heightMult) + (spacing * heightMult)));
	bar.mouseover = E.db.actionbars['stanceBar'].mouseover
	if E.db.actionbars['stanceBar'].enabled then
		bar:SetScale(1);
		bar:SetAlpha(bar.db.alpha);
	else
		bar:SetScale(0.000001);
		bar:SetAlpha(0);
	end
	--[[
	if E.db.actionbars['stanceBar'].backdrop == true then
		bar.backdrop:Show();
	else
		bar.backdrop:Hide();
	end
	]]
	local horizontalGrowth, verticalGrowth;
	if point == "TOPLEFT" or point == "TOPRIGHT" then
		verticalGrowth = "DOWN";
	else
		verticalGrowth = "UP";
	end

	if point == "BOTTOMLEFT" or point == "TOPLEFT" then
		horizontalGrowth = "RIGHT";
	else
		horizontalGrowth = "LEFT";
	end

	local button, lastButton, lastColumnButton;
	for i=1, NUM_STANCE_SLOTS do
		button = _G["AleaUI_StanceBarButton"..i];
		lastButton = _G["AleaUI_StanceBarButton"..i-1];
		lastColumnButton = _G["AleaUI_StanceBarButton"..i-buttonsPerRow];
		button:SetParent(bar);
		button:ClearAllPoints();
		button:SetSize(size,size);
		--[[
		if E.db.actionbars['stanceBar'].mouseover == true then
			bar:SetAlpha(0);
			if not self.hooks[bar] then
				self:HookScript(bar, 'OnEnter', 'Bar_OnEnter');
				self:HookScript(bar, 'OnLeave', 'Bar_OnLeave');
			end

			if not self.hooks[button] then
				self:HookScript(button, 'OnEnter', 'Button_OnEnter');
				self:HookScript(button, 'OnLeave', 'Button_OnLeave');
			end
		else
			bar:SetAlpha(bar.db.alpha);
			if self.hooks[bar] then
				self:Unhook(bar, 'OnEnter');
				self:Unhook(bar, 'OnLeave');
			end

			if self.hooks[button] then
				self:Unhook(button, 'OnEnter');
				self:Unhook(button, 'OnLeave');
			end
		end
		]]
		if i == 1 then
			local x, y;
			if point == "BOTTOMLEFT" then
				x, y = spacing, spacing;
			elseif point == "TOPRIGHT" then
				x, y = -spacing, -spacing;
			elseif point == "TOPLEFT" then
				x, y = spacing, -spacing;
			else
				x, y = -spacing, spacing;
			end

			button:SetPoint(point, bar, point, x, y);
		elseif (i - 1) % buttonsPerRow == 0 then
			local x = 0;
			local y = -spacing;
			local buttonPoint, anchorPoint = "TOP", "BOTTOM";
			if verticalGrowth == 'UP' then
				y = spacing;
				buttonPoint = "BOTTOM";
				anchorPoint = "TOP";
			end
			button:SetPoint(buttonPoint, lastColumnButton, anchorPoint, x, y);
		else
			local x = spacing;
			local y = 0;
			local buttonPoint, anchorPoint = "LEFT", "RIGHT";
			if horizontalGrowth == 'LEFT' then
				x = -spacing;
				buttonPoint = "RIGHT";
				anchorPoint = "LEFT";
			end

			button:SetPoint(buttonPoint, lastButton, anchorPoint, x, y);
		end

		if i > numButtons then
			button:SetScale(0.000001);
			button:SetAlpha(0);
		else
			button:SetScale(1);
			button:SetAlpha(bar.db.alpha);
		end

		if(not button.FlyoutUpdateFunc) then
			self:StyleButton(button, nil, true);
			local opts = E.db.actionbars['stanceBar']
		
			button.artBorder:SetBackdrop({
			  edgeFile = E:GetBorder(opts.border.texture),
			  edgeSize = opts.border.size, 
			})
			button.artBorder:SetBackdropBorderColor(opts.border.color[1],opts.border.color[2],opts.border.color[3],opts.border.color[4])
			button.artBorder:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.inset, -opts.border.inset)
			button.artBorder:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.inset, opts.border.inset)

			button.artBorder.back:SetTexture(E:GetTexture(opts.border.background_texture))
			button.artBorder.back:SetVertexColor(opts.border.background_color[1],opts.border.background_color[2],opts.border.background_color[3],opts.border.background_color[4])
			button.artBorder.back:SetPoint("TOPLEFT", button._point, "TOPLEFT", opts.border.background_inset, -opts.border.background_inset)
			button.artBorder.back:SetPoint("BOTTOMRIGHT", button._point, "BOTTOMRIGHT", -opts.border.background_inset, opts.border.background_inset)
		
		end
	end
end

function AB:AdjustMaxStanceButtons(event)
	if InCombatLockdown() then return; end

	for i=1, #bar.buttons do
		bar.buttons[i]:Hide()
	end
	local initialCreate = false;
	local numButtons = GetNumShapeshiftForms()
	for i = 1, NUM_STANCE_SLOTS do
		if not bar.buttons[i] then
			bar.buttons[i] = CreateFrame("CheckButton", format(bar:GetName().."Button%d", i), bar, "StanceButtonTemplate")
			bar.buttons[i]:SetID(i)
			initialCreate = true;
		end

		if ( i <= numButtons ) then
			bar.buttons[i]:Show();
			bar.LastButton = i;
		else
			bar.buttons[i]:Hide();
		end
	end

	self:PositionAndSizeBarShapeShift();

	if event == 'UPDATE_SHAPESHIFT_FORMS' then
		self:StyleShapeShift()
	end

	if not C_PetBattles.IsInBattle() or initialCreate then
		if numButtons == 0 then
			UnregisterStateDriver(bar, "show");
			bar:Hide()
		else
			bar:Show()
			RegisterStateDriver(bar, "show", '[petbattle] hide;show');
		end
	end
end

function AB:UpdateStanceBindings()
	for i = 1, NUM_STANCE_SLOTS do
		if E.db.actionbars.hotkeytext then
			_G["AleaUI_StanceBarButton"..i.."HotKey"]:Show()
			_G["AleaUI_StanceBarButton"..i.."HotKey"]:SetText(GetBindingKey("CLICK AleaUI_StanceBarButton"..i..":LeftButton"))
			self:FixKeybindText(_G["AleaUI_StanceBarButton"..i])
		else
			_G["AleaUI_StanceBarButton"..i.."HotKey"]:Hide()
		end
	end
end

AB.UPDATE_SHAPESHIFT_FORMS = AB.AdjustMaxStanceButtons
AB.UPDATE_SHAPESHIFT_USABLE = AB.StyleShapeShift
AB.UPDATE_SHAPESHIFT_FORM = AB.StyleShapeShift
AB.ACTIONBAR_PAGE_CHANGED = AB.StyleShapeShift

function AB:CreateBarShapeShift()
--	bar:CreateBackdrop('Default');
--	bar.backdrop:SetAllPoints();
	bar:SetPoint('TOPLEFT', E.UIParent, 'TOPLEFT', 4, -4);
	bar.buttons = {};
	bar:SetAttribute("_onstate-show", [[
		if newstate == "hide" then
			self:Hide();
		else
			self:Show();
		end
	]]);
	
	self:RegisterEvent('UPDATE_SHAPESHIFT_FORMS');
	self:RegisterEvent('UPDATE_SHAPESHIFT_COOLDOWN');
	self:RegisterEvent('UPDATE_SHAPESHIFT_USABLE');
	self:RegisterEvent('UPDATE_SHAPESHIFT_FORM');
	self:RegisterEvent('ACTIONBAR_PAGE_CHANGED');

	E:Mover(bar, 'ShiftActionBar');
	
	self:AdjustMaxStanceButtons();
	self:PositionAndSizeBarShapeShift();
	self:StyleShapeShift();
	self:UpdateStanceBindings()
	
	E.numActonBars = ( E.numActonBars or 0 ) + 1
	
	E.GUI.args.actionbars.args['ShiftActionBar'] = {
		name = L['ShiftActionBar'],
		order = E.numActonBars,
		expand = true,
		type = "group",
		args = {}
	}

	E.GUI.args.actionbars.args['ShiftActionBar'].args.unlock = {
		name = L["Unlock"],
		order = 2,
		type = "execute",
		set = function(self, value)
			E:UnlockMover('ShiftActionBar') 
		end,
		get = function(self)end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.enabled = {
		name =L['Enable'],
		order = 1.1,
		type = "toggle",
		set = function(self, value)
			E.db.actionbars['stanceBar'].enabled = not E.db.actionbars['stanceBar'].enabled
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self)
			return E.db.actionbars['stanceBar'].enabled
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.DarkenInactive = {
		name = L["DarkenInactive"],
		order = 1.2,
		type = "toggle",
		set = function(self, value)	
			if E.db.actionbars['stanceBar'].style == 'darkenInactive' then
				E.db.actionbars['stanceBar'].style = 'reverted'
			else
				E.db.actionbars['stanceBar'].style = 'darkenInactive'
			end
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self)
			return E.db.actionbars['stanceBar'].style == 'darkenInactive' and true or false
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.perrow = {
		name = L["Per row"],
		order = 3.1,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['stanceBar'].buttonsPerRow = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self) 
			return E.db.actionbars['stanceBar'].buttonsPerRow
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.buttons = {
		name = L["Buttons"],
		order = 3,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['stanceBar'].buttons = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self) 
			return E.db.actionbars['stanceBar'].buttons
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.perrow = {
		name = L["Per row"],
		order = 3.1,
		type = "slider",
		min = 1, max = 10, step = 1,
		set = function(self, value)
			E.db.actionbars['stanceBar'].buttonsPerRow = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self) 
			return E.db.actionbars['stanceBar'].buttonsPerRow
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.size = {
		name = L["Size"],
		order = 4,
		type = "slider",
		min = 1, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['stanceBar'].buttonsize = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self) 
			return E.db.actionbars['stanceBar'].buttonsize
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.spacing = {
		name = L["Spacing"],
		order = 6,
		type = "slider",
		min = 0, max = 60, step = 1,
		set = function(self, value)
			E.db.actionbars['stanceBar'].buttonspacing = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self) 
			return E.db.actionbars['stanceBar'].buttonspacing
		end
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.point = {
		name = L["Point"],
		type = "dropdown",
		order = 7,
		values = DD_points,
		set = function(self, value)
			E.db.actionbars['stanceBar'].point = value
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(self)
			return E.db.actionbars['stanceBar'].point
		end,	
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts = {
		name = L['Borders'],
		order = 10,
		embend = true,
		type = "group",
		args = {}
	}
	
	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BorderTexture = {
		order = 1,
		type = 'border',
		name = L['Border texture'],
		values = E:GetBorderList(),
		set = function(info,value) 
			E.db.actionbars['stanceBar'].border.texture = value;
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(info) return E.db.actionbars['stanceBar'].border.texture end,
	}

	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BorderColor = {
		order = 2,
		name = L['Border color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['stanceBar'].border.color={ r, g, b, a}; 
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(info) 
			return E.db.actionbars['stanceBar'].border.color[1],
					E.db.actionbars['stanceBar'].border.color[2],
					E.db.actionbars['stanceBar'].border.color[3],
					E.db.actionbars['stanceBar'].border.color[4] 
		end,
	}

	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BorderSize = {
		name = L['Border size'],
		type = "slider",
		order	= 3,
		min		= 1,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['stanceBar'].border.size = val
			AB:PositionAndSizeBarShapeShift();
		end,
		get =function(info)
			return E.db.actionbars['stanceBar'].border.size
		end,
	}

	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BorderInset = {
		name = L['Border inset'],
		type = "slider",
		order	= 4,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['stanceBar'].border.inset = val
			AB:PositionAndSizeBarShapeShift();
		end,
		get =function(info)
			return E.db.actionbars['stanceBar'].border.inset
		end,
	}


	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BackgroundTexture = {
		order = 5,
		type = 'statusbar',
		name = L['Background texture'],
		values = E.GetTextureList,
		set = function(info,value) 
			E.db.actionbars['stanceBar'].border.background_texture = value;
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(info) return E.db.actionbars['stanceBar'].border.background_texture end,
	}

	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.BackgroundColor = {
		order = 6,
		name = L['Background color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.actionbars['stanceBar'].border.background_color={ r, g, b, a}
			AB:PositionAndSizeBarShapeShift();
		end,
		get = function(info) 
			return E.db.actionbars['stanceBar'].border.background_color[1],
					E.db.actionbars['stanceBar'].border.background_color[2],
					E.db.actionbars['stanceBar'].border.background_color[3],
					E.db.actionbars['stanceBar'].border.background_color[4] 
		end,
	}


	E.GUI.args.actionbars.args['ShiftActionBar'].args.BorderOpts.args.backgroundInset = {
		name = L['Background offset'],
		type = "slider",
		order	= 7,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.actionbars['stanceBar'].border.background_inset = val
			AB:PositionAndSizeBarShapeShift();
		end,
		get =function(info)
			return E.db.actionbars['stanceBar'].border.background_inset
		end,
	}
end


do

	local function UpdateItemButtonColor(button)
		if not button.itemcolor then
			local f = CreateFrame("Frame", nil, button)
			f:SetInside()
			f:SetFrameLevel(button:GetFrameLevel()+1)
			Skins.SetTemplate(f, 'BORDERED')
			f:SetBackdropBorderColor(0,0,0,0)
			button.itemcolor = f
		end	
		
	--	print('T',	button._state_type, button._state_action)
				
		button.itemcolor:SetBackdropBorderColor(0,0,0,0)
		
		if button._state_type == 'action' then
			local type, id, subType, spellID = GetActionInfo(button._state_action)	
			if type == 'item' then
				local name, link, quality = GetItemInfo(id)		
				if quality and quality > 1 then				
					local r, g, b, hex = GetItemQualityColor(quality);									
					button.itemcolor:SetBackdropBorderColor(r,g,b,1)
				end
			end
		end
	end

	local addon1 = CreateFrame("Frame")
	addon1:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
	addon1:RegisterEvent("PLAYER_LOGIN")
	addon1:RegisterEvent("SPELLS_CHANGED")
	addon1:RegisterEvent("PLAYER_TALENT_UPDATE")
	addon1:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	addon1:SetScript("OnEvent", function(self, event, buttonID)
		self:Show()
	end)
	addon1:SetScript('OnUpdate', function(self, elapsed)
		self.elapsed = (self.elapsed or 0 ) + elapsed
		if self.elapsed < 0.05 then return end
		self.elapsed = 0
		
		for button in pairs(AB.handledbuttons or {}) do
			UpdateItemButtonColor(button)		
		end
		self:Hide()
	end)
	E:OnInit2(function()
		addon1:Show()
	end)
end