local E = AleaUI
local OT = E:Module("ObjectivesTracker")
local Skins = E:Module("Skins")

local defaults = {
	classColor = true,
	personalColor = false,
	color = { 0.8, 0.8, 0.2, 1 },
	
	setResizable = true,
	width = 200,
	height = 800,
	
	hideOnBossFights = true,
}

E.default_settings.objectFrame = defaults

	
local status = false

local WatchFrameHolder = CreateFrame("Frame", "WatchFrameHolder", E.UIParent)
WatchFrameHolder:SetWidth(130)
WatchFrameHolder:SetHeight(22)
WatchFrameHolder:Show()
WatchFrameHolder:SetClampedToScreen(true)

local WatchFrameUpdater = CreateFrame("Frame", "WatchFrameUpdater", E.UIParent)
WatchFrameUpdater:Show()
WatchFrameUpdater:SetScript('OnUpdate', function(self, elapsed)
	self.elapsed = ( self.elapsed or 0 ) + elapsed
	if self.elapsed < 1 then return end
	self.elapsed = 0
	local show = true

	for i=1, 5 do
		local unit = 'boss'..i		
		if UnitExists(unit) then
			WatchFrameHolder:Hide()
			show = false
			break
		end
	end

	if show and not WatchFrameHolder:IsShown() then
		WatchFrameHolder:Show()
	end
end)

local header = { 
	["QuestHeader"] = true,
	["AchievementHeader"] = true,
	["ScenarioHeader"] = true,
}

	
	--[==[
	SCENARIO_CONTENT_TRACKER_MODULE,
	AUTO_QUEST_POPUP_TRACKER_MODULE,
	QUEST_TRACKER_MODULE,
	BONUS_OBJECTIVE_TRACKER_MODULE,
	WORLD_QUEST_TRACKER_MODULE,
	ACHIEVEMENT_TRACKER_MODULE
					]==]	
					
local function Load()
	E:Mover(WatchFrameHolder, "watchFrameMover")
	
	ObjectiveTrackerFrame:SetIgnoreFramePositionManager(true)
	ObjectiveTrackerFrame:SetParent(WatchFrameHolder)
	ObjectiveTrackerFrame:ClearAllPoints()
	ObjectiveTrackerFrame:SetPoint('TOPRIGHT', WatchFrameHolder, 'TOPRIGHT', 0, 0)
	ObjectiveTrackerFrame:SetHeight(500)
	ObjectiveTrackerFrame:SetClampedToScreen(true)
	
	ObjectiveTrackerFrame.bg = ObjectiveTrackerFrame:CreateTexture(nil, "BACKGROUND", nil, 1)
	ObjectiveTrackerFrame.bg:SetAllPoints()
	ObjectiveTrackerFrame.bg:SetColorTexture(1,1,1,0.2)	
	ObjectiveTrackerFrame.bg:Hide()
		
	hooksecurefunc(ObjectiveTrackerFrame,"SetPoint",function(_,_,parent)
		if parent ~= WatchFrameHolder then
			ObjectiveTrackerFrame:ClearAllPoints()
			ObjectiveTrackerFrame:SetPoint('TOPRIGHT', WatchFrameHolder, 'TOPRIGHT', 0, 0)
		end
	end)
	hooksecurefunc(ObjectiveTrackerFrame,"SetParent",function(_,parent)
		if parent ~= WatchFrameHolder then
			ObjectiveTrackerFrame:SetParent(WatchFrameHolder)
		end
	end)
	
	
	local minWidth =  243
	local minHeight = 300
	
	local maxWidth = 500
	local maxHeight = 700
	
	local lastDiments_width = -1
	local lastDiments_height = -1
	
	local function MovingFrameOnUpdate(self, elapsed)
		
		local x, y = GetCursorPosition();
				x = x / self:GetEffectiveScale();
				y = y / self:GetEffectiveScale();
			
		self:SetPoint("BOTTOMLEFT", E.UIParent, "BOTTOMLEFT", x, y)
			
		if ( self:GetLeft() > ( ObjectiveTrackerFrame:GetRight() - minWidth ) ) then
			x = ObjectiveTrackerFrame:GetLeft()
		end
		
		if ( self:GetBottom() > ( ObjectiveTrackerFrame:GetTop() - minHeight ) ) then
			y = ObjectiveTrackerFrame:GetBottom()
		end
		
		if ( self:GetLeft() < ( ObjectiveTrackerFrame:GetRight() - maxWidth ) ) then		
			x = ObjectiveTrackerFrame:GetLeft()
		end
		
		if ( self:GetBottom() < ( ObjectiveTrackerFrame:GetTop() - maxHeight ) ) then
			y = ObjectiveTrackerFrame:GetBottom()
		end
		
		self:SetPoint("BOTTOMLEFT", E.UIParent, "BOTTOMLEFT", x, y)
		
		
		
		local diments_width = math.min(maxWidth, math.max(0, (ObjectiveTrackerFrame:GetRight()  - self:GetLeft())))
		local width = math.max(minWidth, diments_width)
		
		local diments_height = math.min(maxHeight , math.max(0, (ObjectiveTrackerFrame:GetTop()  - self:GetBottom())))
		local height = math.max(minHeight, diments_height)
		
		if lastDiments_width ~= diments_width then
			lastDiments_width = diments_width
			ObjectiveTrackerFrame:SetWidth(width)
		end
		
		if lastDiments_height ~= diments_height then
			lastDiments_height = diments_height
			ObjectiveTrackerFrame:SetHeight(height)
		end
		
	end
	
	
	local sizing = CreateFrame("Frame", nil, E.UIParent)
		sizing:SetSize(18, 18)
		sizing.isMouseDown = false
		sizing:SetMovable(true)
		sizing:SetFrameLevel(15)
		sizing:RegisterForDrag("LeftButton")
		sizing:SetPoint("BOTTOMLEFT", E.UIParent, "BOTTOMLEFT", ObjectiveTrackerFrame:GetLeft(), ObjectiveTrackerFrame:GetBottom())
		sizing:SetScript("OnDragStart", function(self)
			self:SetScript('OnUpdate', MovingFrameOnUpdate)
		end)
		sizing:SetScript("OnDragStop", function(self)
			
			ObjectiveTrackerFrame.bg:Hide()
			self:SetScript('OnUpdate', nil)
			self.isMouseDown = false
			self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 1)
		end)
		sizing:SetScript("OnEnter", function(self)
			ObjectiveTrackerFrame.bg:Show()
			self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight") --0.4, 0.4, 0.4, 0.8)
		end)
		sizing:SetScript("OnLeave", function(self)
			if not self.isMouseDown then
				ObjectiveTrackerFrame.bg:Hide()
				self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 0.4)
			end
		end)
		sizing:SetScript("OnMouseUp", function(self)
			self.isMouseDown = false
			ObjectiveTrackerFrame.bg:Hide()
			self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 1)
		end)
		sizing:SetScript("OnMouseDown", function(self)
			self.isMouseDown = true
			self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down") --0.4, 0.4, 0.4, 0.8)
		end)

		local sizing_bg = sizing:CreateTexture(nil, "BACKGROUND", nil, 1)
		sizing_bg:SetAllPoints()
		sizing_bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 0.4)	
		sizing.bg = sizing_bg
		E.SetTextureRotation(sizing.bg, 270)
		
	local function QuestItemButtonSkin(self)
		
		self:SetFrameLevel(10)
		self.HotKey:SetFont(E.media.default_font2, 18, 'OUTLINE')
		self.HotKey:SetText('x')
		
		self:SetNormalTexture(nil)
		self:SetPushedTexture(nil)
		
		self.icon:SetTexCoord(0.07, 0.93,  0.07, 0.93)
		self.icon:SetDrawLayer("OVERLAY")
		
		if not self._texture then
			self._texture = self:CreateTexture()
			self._texture:SetAllPoints(self)
			self._texture:SetDrawLayer("OVERLAY")
			self._texture:SetColorTexture(1, 1, 1, 0.3)
			
			self:SetHighlightTexture(self._texture)
		end
		
		if not self._bg then
			self._bg = self:CreateTexture()
			self._bg:SetOutside(self)
			self._bg:SetDrawLayer("BACKGROUND")
			self._bg:SetColorTexture(0, 0, 0, 1)
		end
	end
	
	hooksecurefunc('QuestObjectiveItem_OnShow', QuestItemButtonSkin)
	hooksecurefunc('QuestObjectiveItem_OnHide', QuestItemButtonSkin)
	
	function OT:UpdateStyle()	
		local statusBar = self.Bar
		local IconBG 	= statusBar.IconBG
		local Icon		= statusBar.Icon
		local BarFrame 	= statusBar.BarFrame
		local BarFrame2 = statusBar.BarFrame2
		local BarFrame3 = statusBar.BarFrame3
		local BarBG		= statusBar.BarBG -- background
		
		IconBG:SetAlpha(0); IconBG:SetShown(false);
		BarFrame:SetAlpha(0); BarFrame:SetShown(false);
		BarFrame2:SetAlpha(0); BarFrame2:SetShown(false);
		BarFrame3:SetAlpha(0); BarFrame3:SetShown(false);
			
		statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
		BarBG:SetTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
		BarBG:SetVertexColor(32/255, 61/255, 116/255, 0.7)
	end
	
	function OT:ThemeBar(progressBar, themeType)	
		local statusBar = progressBar.Bar
		local IconBG 	= statusBar.IconBG
		local Icon		= statusBar.Icon
		local BarFrame 	= statusBar.BarFrame
		local BarFrame2 = statusBar.BarFrame2
		local BarFrame3 = statusBar.BarFrame3
		local BarBG		= statusBar.BarBG -- background

		if not progressBar.skinned then
			progressBar.skinned = true
			
			progressBar.themeType = themeType
			
			IconBG:SetAlpha(0); IconBG:SetShown(false);
			BarFrame:SetAlpha(0); BarFrame:SetShown(false);
			BarFrame2:SetAlpha(0); BarFrame2:SetShown(false);
			BarFrame3:SetAlpha(0); BarFrame3:SetShown(false);
			
			
			statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
			BarBG:SetTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
		
			BarBG:SetVertexColor(32/255, 61/255, 116/255, 0.7)
			E:CreateBackdrop(statusBar, BarBG, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)				
			E:CreateBackdrop(progressBar, progressBar.Bar.Icon, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)
		
			statusBar.Starburst:Kill()
			statusBar.BarGlow:Kill()
			statusBar.Sheen:Kill()
			
			progressBar.Flare1:Kill()
			progressBar.Flare2:Kill()
			progressBar.SmallFlare1:Kill()
			progressBar.SmallFlare2:Kill()
			progressBar.FullBarFlare1:Kill()
			progressBar.FullBarFlare2:Kill()
			
			progressBar:HookScript('OnShow', OT.UpdateStyle)
		end
		
	--	print('T', 'ThemeBar', themeType, progressBar.themeType)
	
		OT.UpdateStyle(progressBar)
	end
	
	ObjectiveTrackerFrame.HeaderMenu.Title:SetFont(E.media.default_font, 12,'NONE')
	ObjectiveTrackerFrame.HeaderMenu.Title:SetTextColor(1, 1, 1, 1)
	for frame in pairs(header) do
		ObjectiveTrackerBlocksFrame[frame]:StripTextures("ARTWORK")
		ObjectiveTrackerBlocksFrame[frame].Text:SetWordWrap(true)
		ObjectiveTrackerBlocksFrame[frame].Text:SetFont(E.media.default_font, 12,'NONE')
		ObjectiveTrackerBlocksFrame[frame].Text:SetTextColor(1, 1, 1, 1)
	end
	
	BONUS_OBJECTIVE_TRACKER_MODULE.Header:StripTextures("ARTWORK")
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Text:SetWordWrap(true)
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Text:SetFont(E.media.default_font2, 14)
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Text:SetTextColor(1,1,1,1)
	
	WORLD_QUEST_TRACKER_MODULE.Header:StripTextures("ARTWORK")
	WORLD_QUEST_TRACKER_MODULE.Header.Text:SetWordWrap(true)
	WORLD_QUEST_TRACKER_MODULE.Header.Text:SetFont(E.media.default_font2, 14)
	WORLD_QUEST_TRACKER_MODULE.Header.Text:SetTextColor(1,1,1,1)
	
	local baseAlpha = 0.6
	local function ThemeRewardItem_OnShow(self)
		self.bgIcon:SetAlpha(0)
		self.bgText:SetAlpha(0)
		self.bgIcon.alpha = -1
	end
	local function ThemeRewardItem_OnUpdate(self, elapsed)
		self.bgIcon.alpha = self.bgIcon.alpha + elapsed
		
		local a = self.bgIcon.alpha 
	
		if a < 1 and a > 0 then
			self.bgIcon:SetAlpha(baseAlpha*a)
			self.bgText:SetAlpha(baseAlpha*a)
		elseif a > 2 then
			self.bgIcon:SetAlpha(baseAlpha*(3-a))
			self.bgText:SetAlpha(baseAlpha*(3-a))
		elseif a > 1 then
			self.bgIcon:SetAlpha(baseAlpha)
			self.bgText:SetAlpha(baseAlpha)
		else
			self.bgIcon:SetAlpha(0)
			self.bgText:SetAlpha(0)
		end		
	end
	local function ThemeRewardItem_OnHide(self)
		self.bgIcon:SetAlpha(0)
		self.bgText:SetAlpha(0)
		self.bgIcon.alpha = 0
	end
	
	function OT:ThemeRewardItem()
	
		self.Count:SetFont(E.media.default_font2, 12, 'OUTLINE')
		self.Label:SetFont(E.media.default_font2, 12, 'OUTLINE')
		self.ItemIcon:SetTexCoord(unpack(E.media.texCoord))
		
		local parent = self:GetParent()
	
		if not self.bgIcon then
			local bgIcon = self:CreateTexture()
			bgIcon:SetColorTexture(0,0,0,1)
			bgIcon:SetOutside(self.ItemIcon)
			bgIcon:SetDrawLayer('BORDER', 0)
			
			self:HookScript('OnShow', ThemeRewardItem_OnShow)
			self:HookScript('OnHide', ThemeRewardItem_OnHide)
			self:HookScript('OnUpdate', ThemeRewardItem_OnUpdate)

			local bgText = self:CreateTexture()
			bgText:SetColorTexture(0,0,0,1)
			bgText:SetPoint('TOPLEFT', self.ItemIcon, 'TOPRIGHT', 3, 0)
			bgText:SetPoint('BOTTOMLEFT', self.ItemIcon, 'BOTTOMRIGHT', 3, 0)
			bgText:SetSize(76, 76)
			bgText:SetDrawLayer('BORDER', 0)
			
			self.bgText = bgText
			self.bgIcon = bgIcon
			
			ThemeRewardItem_OnShow(self)
		end
		
		self.ItemOverlay:SetShown(false)
		self.ItemBorder:SetShown(false)
	end
	
	hooksecurefunc('BonusObjectiveTracker_AnimateReward', function(block)
		local rewardsFrame = ObjectiveTrackerBonusRewardsFrame;
	--	rewardsFrame:StripTextures()
		for i=1, #rewardsFrame.Rewards do
			OT.ThemeRewardItem(rewardsFrame.Rewards[i])
		end
	end)
	
	hooksecurefunc('ScenarioObjectiveTracker_AnimateReward', function()
		local rewardsFrame = ObjectiveTrackerScenarioRewardsFrame;
	--	rewardsFrame:StripTextures()
		for i=1, #rewardsFrame.Rewards do
			OT.ThemeRewardItem(rewardsFrame.Rewards[i])
		end
	end)
	
	local colorGradient = { '|cFFFF0000', '|cFFFF7a00', '|cFFFFFF00', '|cFF00FF00' }
	
	local stringHooked = {}
	
	local listOfModules = {		 
		SCENARIO_CONTENT_TRACKER_MODULE,
		AUTO_QUEST_POPUP_TRACKER_MODULE,
		QUEST_TRACKER_MODULE,
		BONUS_OBJECTIVE_TRACKER_MODULE,
		WORLD_QUEST_TRACKER_MODULE,
		ACHIEVEMENT_TRACKER_MODULE }
	
	local tempFrame = CreateFrame('Frame')
	
	function OT:UpdateUsedBlocksWidth()		
		for i=1, #listOfModules do		
			for id, frame in pairs( listOfModules[i].usedBlocks ) do	
				
				local mod = 0
				if WORLD_QUEST_TRACKER_MODULE == listOfModules[i] then
					mod = 22
				end
				
				if frame:GetWidth()+mod ~=  ObjectiveTrackerFrame:GetWidth()+mod then
					frame:SetWidth( ObjectiveTrackerFrame:GetWidth()+mod )
					--[==[
					if not frame.bg_test then
						local bg_test = frame:CreateTexture()
						bg_test:SetColorTexture(0,0,0,0.2)
						bg_test:SetOutside(frame)
						bg_test:SetDrawLayer('BORDER', 0)
						frame.bg_test = bg_test
					end
					]==]			
				end
			end	
		end
		
		for frame in pairs(stringHooked) do

			local mod = 0
			if OT.IsHeader(frame) then
				mod = -30
			end
			
		--	print('Point:', frame:GetPoint())
		--	print('Parent:', frame:GetParent())
			
			if frame:GetWidth() ~=  ObjectiveTrackerFrame:GetWidth()+mod then	
				frame:SetWidth( ObjectiveTrackerFrame:GetWidth()+mod )	
				--[==[
				if not frame.bg_test then
					local bg_test = tempFrame:CreateTexture()
					bg_test:SetColorTexture(1,0,0,0.2)
					bg_test:SetOutside(frame)
					bg_test:SetDrawLayer('BORDER', 0)
					frame.bg_test = bg_test
				end
				]==]
			end
		end
	end
	
	
	function OT:UpdateStringWidth()	
		OT:UpdateUsedBlocksWidth()		
	end
	
	ObjectiveTrackerFrame:HookScript('OnSizeChanged', function(self, w, h)		
		OT:UpdateUsedBlocksWidth()
	end)
	
	local function HandlerForObjectiveString(self, text)
		if stringHooked[self] ~= text then
			stringHooked[self] = text
		
		--	print('Update text', text)
			
			local a1, a2, a3, a4 = string.find(text, '(%d+)/(%d+)')
			
			if a3 and a4 then
				local currentValue = tonumber(a3)
				local maxValue = tonumber(a4)
				
				if currentValue and maxValue then
					local percent = currentValue/maxValue
					
					local color = colorGradient[4]
					
					if percent < 0.4 then
						color = colorGradient[1]
					elseif percent < 0.7 then
						color = colorGradient[2]
					elseif percent < 1 then
						color = colorGradient[3]
				--	elseif percent < 0.8 then
				--		color = colorGradient[4]
					end
					
					local pattern = format('%d/%d', currentValue, maxValue)
				
					local tempText = text:gsub(pattern, color..pattern..'|r')
					
					stringHooked[self] = tempText
					
					self:SetText(tempText)
					
				--	print('T', percent, currentValue, maxValue, color, pattern, text)
				end
			end
		end
	end
	
	local colorWhite = 1
	local colorWhiteHighlight = 1.3
	local classR, classG, classB = RAID_CLASS_COLORS[E.myclass].r, RAID_CLASS_COLORS[E.myclass].g, RAID_CLASS_COLORS[E.myclass].b
	local classRH, classGH, classBH = RAID_CLASS_COLORS[E.myclass].r, RAID_CLASS_COLORS[E.myclass].g, RAID_CLASS_COLORS[E.myclass].b
	
	if E.myclass == 'PRIEST' then
		colorWhite = 0.7
		colorWhiteHighlight = 1
		
		classR = 120/255
		classG = 153/255
		classB = 243/255
		
		classRH, classGH, classBH = classR, classG, classB
	elseif E.myclass == 'SHAMAN' then
		colorWhite = 1.3
		colorWhiteHighlight = 1.6
	end
	--[==[
	OBJECTIVE_TRACKER_COLOR["Header"].r = classR*colorWhite
	OBJECTIVE_TRACKER_COLOR["Header"].g = classG*colorWhite
	OBJECTIVE_TRACKER_COLOR["Header"].b = classB*colorWhite
	
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].r = classRH*colorWhiteHighlight
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].g = classGH*colorWhiteHighlight
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].b = classBH*colorWhiteHighlight
	]==]
	
	local customColorText = { Header = {}, HeaderHighlight = {} }
	
	customColorText["Header"].r = classR*colorWhite
	customColorText["Header"].g = classG*colorWhite
	customColorText["Header"].b = classB*colorWhite
	
	customColorText["HeaderHighlight"].r = classRH*colorWhiteHighlight
	customColorText["HeaderHighlight"].g = classGH*colorWhiteHighlight
	customColorText["HeaderHighlight"].b = classBH*colorWhiteHighlight
	
	local function IsHeader(self)
		local r,g,b = self:GetTextColor()
		
		
		return ( OBJECTIVE_TRACKER_COLOR["Header"].r == r and OBJECTIVE_TRACKER_COLOR["Header"].g == g and	OBJECTIVE_TRACKER_COLOR["Header"].b == b ) or			
			   ( OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].r == r and OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].g == g and OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].b == b )			
	end
	
	OT.IsHeader = IsHeader
	
	local function UpdateCustomTextColor(self, r,g,b,a)
		
		
		if OBJECTIVE_TRACKER_COLOR["Header"].r == r and
			OBJECTIVE_TRACKER_COLOR["Header"].g == g and
			OBJECTIVE_TRACKER_COLOR["Header"].b == b then
			
			self:SetTextColor(customColorText["Header"].r, customColorText["Header"].g,customColorText["Header"].b)
		elseif OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].r == r and
			OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].g == g and
			OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].b == b then
			
			self:SetTextColor(customColorText["HeaderHighlight"].r, customColorText["HeaderHighlight"].g,customColorText["HeaderHighlight"].b)
		elseif ( OBJECTIVE_TRACKER_COLOR["Normal"].r == r and
				 OBJECTIVE_TRACKER_COLOR["Normal"].g == g and
				 OBJECTIVE_TRACKER_COLOR["Normal"].b == b ) then
				 
		--	self:SetTextColor(OBJECTIVE_TRACKER_COLOR["Normal"].r, OBJECTIVE_TRACKER_COLOR["Normal"].g,OBJECTIVE_TRACKER_COLOR["Normal"].b)
		elseif ( OBJECTIVE_TRACKER_COLOR["NormalHighlight"].r == r and
				 OBJECTIVE_TRACKER_COLOR["NormalHighlight"].g == g and
				 OBJECTIVE_TRACKER_COLOR["NormalHighlight"].b == b ) then
		
		--	self:SetTextColor(OBJECTIVE_TRACKER_COLOR["NormalHighlight"].r, OBJECTIVE_TRACKER_COLOR["NormalHighlight"].g,OBJECTIVE_TRACKER_COLOR["NormalHighlight"].b)
		end
	end
	
	local function UpdateCustomFont(self, font, size, outline)
	
		if IsHeader(self) then			
			if font ~= E.media.default_font2 and size ~= 14 and outline ~= 'NONE' then
				self:SetFont(E.media.default_font2, 14)	
				self:SetShadowColor(0,0,0,1)
				self:SetShadowOffset(1, -1)
			end
			
			self:SetWordWrap(false)
		else
			if font ~= E.media.default_font2 and size ~= 12 and outline ~= 'NONE' then
				self:SetFont(E.media.default_font2, 12)	
				self:SetShadowColor(0,0,0,1)
				self:SetShadowOffset(1, -1)
			end
			
			self:SetWordWrap(true)
		end
	end
	
	
	function OT:OnSetStringText(fontString, text, useFullHeight, colorStyle, useHighlight)
		
		if colorStyle == OBJECTIVE_TRACKER_COLOR["Header"] then
		--	fontString:SetFont(E.media.default_font2, 14, 'NONE')				
			UpdateCustomTextColor(fontString, OBJECTIVE_TRACKER_COLOR["Header"].r, OBJECTIVE_TRACKER_COLOR["Header"].g, OBJECTIVE_TRACKER_COLOR["Header"].b)
		else
		--	fontString:SetFont(E.media.default_font2, 12, 'NONE')
		end
		
		OT.UpdateStringWidth(fontString)
		
		if not stringHooked[fontString] then
			stringHooked[fontString] = true
			
			
			
	--		hooksecurefunc(fontString, 'SetFont', UpdateCustomFont)				
			hooksecurefunc(fontString, 'SetTextColor', UpdateCustomTextColor)	
			hooksecurefunc(fontString, 'SetText', HandlerForObjectiveString)
		end
	end
	
	hooksecurefunc(DEFAULT_OBJECTIVE_TRACKER_MODULE, 'SetStringText', OT.OnSetStringText)
	
	hooksecurefunc(SCENARIO_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, criteriaIndex)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]

		if progressBar then
			
			OT:ThemeBar(progressBar, 'Scenario')
	
			
			progressBar.Bar.Icon:Hide();
			progressBar.Bar.IconBG:Hide();
			progressBar.Bar.Icon:SetUIBorderAlpha(0)
			
			if (not criteriaIndex) then
				local rewardQuestID = select(10, C_Scenario.GetStepInfo());

				if (rewardQuestID ~= 0) then
					-- reward icon; try the first item
					local _, texture = GetQuestLogRewardInfo(1, rewardQuestID);
					-- artifact xp
					local artifactXP, artifactCategory = GetQuestLogRewardArtifactXP(rewardQuestID);
					if ( not texture and artifactXP > 0 ) then
						local name, icon = C_ArtifactUI.GetArtifactXPRewardTargetInfo(artifactCategory);
						texture = icon or "Interface\\Icons\\INV_Misc_QuestionMark";
					end
					-- currency
					if ( not texture and GetNumQuestLogRewardCurrencies(rewardQuestID) > 0 ) then
						_, texture = GetQuestLogRewardCurrencyInfo(1, rewardQuestID);
					end
					-- money?
					if ( not texture and GetQuestLogRewardMoney(rewardQuestID) > 0 ) then
						texture = "Interface\\Icons\\inv_misc_coin_02";
					end
					-- xp
					if ( not texture and GetQuestLogRewardXP(rewardQuestID) > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) then
						texture = "Interface\\Icons\\xp_icon";
					end
					if ( texture ) then
						progressBar.Bar.Icon:SetTexture(texture);
						progressBar.Bar.Icon:Show();
						progressBar.Bar.IconBG:Show();
						progressBar.Bar.Icon:SetUIBorderAlpha(1)
						progressBar.Bar.BarGlow:SetAtlas("bonusobjectives-bar-glow-ring", true);
					end
				end
			end
		end
	end)
	hooksecurefunc(DEFAULT_OBJECTIVE_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, questID)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]

		if progressBar then
			local statusBar = progressBar.Bar
			local BorderLeft 	= statusBar.BorderLeft
			local BorderMid		= statusBar.BorderMid
			local BorderRight 	= statusBar.BorderRight
		
			if not progressBar.skinned then
				progressBar.skinned = true
				
				progressBar.themeType = 'Default'
	
				BorderLeft:Kill()
				BorderMid:Kill()
				BorderRight:Kill()
				
				statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
				statusBar:SetStatusBarColor(32/255, 61/255, 116/255, 0.7)
				E:CreateBackdrop(statusBar, nil, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)	
			end
			
			statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
			statusBar:SetStatusBarColor(32/255, 61/255, 116/255, 0.7)
				
	--		print('T', 'ThemeBar', 'Default', progressBar.themeType)			
		end
	end)
	
	hooksecurefunc(BONUS_OBJECTIVE_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, questID, finished)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]

		if progressBar then
		
			OT:ThemeBar(progressBar, 'BONUS_OBJECT')
			
			local _, texture = GetQuestLogRewardInfo(1, questID);
			-- next, currency
			if ( not texture and GetNumQuestLogRewardCurrencies(questID) > 0 ) then
				_, texture = GetQuestLogRewardCurrencyInfo(1, questID);
			end
			-- money?
			if ( not texture and GetQuestLogRewardMoney(questID) > 0 ) then
				texture = "Interface\\Icons\\inv_misc_coin_02";
			end
			-- xp
			if ( not texture and GetQuestLogRewardXP(questID) > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) then
				texture = "Interface\\Icons\\xp_icon";
			end

			if not texture then			
				progressBar.Bar.Icon:SetUIBorderAlpha(0)
			else
				progressBar.Bar.Icon:SetUIBorderAlpha(1)
				progressBar.Bar.Icon:SetSize(17, 17)
				progressBar.Bar.Icon:SetMask(nil)
				progressBar.Bar.Icon:SetTexture(texture)
				progressBar.Bar.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
				progressBar.Bar.Icon:ClearAllPoints()
				progressBar.Bar.Icon:SetPoint("TOPLEFT", progressBar.Bar.BarBG, "TOPRIGHT", 3, 0)
			end			
		end
	end)	
	
	hooksecurefunc(WORLD_QUEST_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, questID, finished)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]

		if progressBar then
		
			OT:ThemeBar(progressBar, 'WORLD')
			
			local _, texture = GetQuestLogRewardInfo(1, questID);
			-- next, currency
			if ( not texture and GetNumQuestLogRewardCurrencies(questID) > 0 ) then
				_, texture = GetQuestLogRewardCurrencyInfo(1, questID);
			end
			-- money?
			if ( not texture and GetQuestLogRewardMoney(questID) > 0 ) then
				texture = "Interface\\Icons\\inv_misc_coin_02";
			end
			-- xp
			if ( not texture and GetQuestLogRewardXP(questID) > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) then
				texture = "Interface\\Icons\\xp_icon";
			end

			if not texture then			
				progressBar.Bar.Icon:SetUIBorderAlpha(0)
			else
				progressBar.Bar.Icon:SetUIBorderAlpha(1)
				progressBar.Bar.Icon:SetSize(17, 17)
				progressBar.Bar.Icon:SetMask(nil)
				progressBar.Bar.Icon:SetTexture(texture)
				progressBar.Bar.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
				progressBar.Bar.Icon:ClearAllPoints()
				progressBar.Bar.Icon:SetPoint("TOPLEFT", progressBar.Bar.BarBG, "TOPRIGHT", 3, 0)
			end			
		end
	end)
--	hooksecurefunc(WORLD_QUEST_TRACKER_MODULE, 'SetStringText', OT.OnSetStringText)	
	
end

E:OnInit2(Load)