local E = AleaUI
local addOn = ...

--if true then return end

local NP = E:Module('NamePlates')
local UF = E:Module("UnitFrames")

local plateFrameOffset = 10
local plateFrameWidthOffset = 20

NP.CreatedPlates = {}
--[==[
	["nameplateOtherAtBase"] = "Position other nameplates at the base, rather than overhead -1=bottom 0=top 1=middle",
	["nameplateOverlapH"] = "Percentage amount for horizontal overlap of nameplates",
	["nameplateOverlapV"] = "Percentage amount for vertical overlap of nameplates",
	["nameplateResourceOnTarget"] = "Nameplate class resource overlay mode. 0=self, 1=target",

	["nameplateShowSelf"] = "",
	["nameplateShowEnemies"] = "",
	["nameplateShowEnemyMinions"] = "",
	["nameplateShowEnemyPets"] = "",
	["nameplateShowEnemyGuardians"] = "",
	["nameplateShowEnemyTotems"] = "",
	["nameplateShowEnemyMinus"] = "",
	["nameplateShowFriends"] = "",
	["nameplateShowFriendlyMinions"] = "",
	["nameplateShowFriendlyPets"] = "",
	["nameplateShowFriendlyGuardians"] = "",
	["nameplateShowFriendlyTotems"] = "",
	["nameplateShowAll"] = "",
	
	["nameplateMaxDistance"] = "The max distance to show nameplates.",
	["nameplateTargetBehindMaxDistance"] = "The max distance to show the target nameplate when the target is behind the camera.",
	["nameplateMotion"] = "Defines the movement/collision model for nameplates",
	["nameplateMotionSpeed"] = "Controls the rate at which nameplate animates into their target locations [0.0-1.0] default=0.025",
	["nameplateGlobalScale"] = "Applies global scaling to non-self nameplates, this is applied AFTER selected, min, and max scale.",
	["nameplateMinScale"] = "The minimum scale of nameplates. default=0.8",
	["nameplateMaxScale"] = "The max scale of nameplates.",
	["nameplateLargerScale"] = "An additional scale modifier for important monsters.",
	["nameplateMinScaleDistance"] = "The distance from the max distance that nameplates will reach their minimum scale.",
	["nameplateMaxScaleDistance"] = "The distance from the camera that nameplates will reach their maximum scale.",
	["nameplateMinAlpha"] = "The minimum alpha of nameplates.",
	["nameplateMaxAlpha"] = "The max alpha of nameplates.",
	["nameplateMinAlphaDistance"] = "The distance from the max distance that nameplates will reach their minimum alpha.",
	["nameplateMaxAlphaDistance"] = "The distance from the camera that nameplates will reach their maximum alpha.",
	["nameplateSelectedScale"] = "The scale of the selected nameplate.",
	["nameplateSelectedAlpha"] = "The alpha of the selected nameplate.",
	["nameplateSelfScale"] = "The scale of the self nameplate.",
	["nameplateSelfAlpha"] = "The alpha of the self nameplate.",
	["nameplateSelfBottomInset"] = "The inset from the bottom (in screen percent) that the self nameplate is clamped to.",
	["nameplateSelfTopInset"] = "The inset from the top (in screen percent) that the self nameplate is clamped to.",
	["nameplateOtherBottomInset"] = "The inset from the bottom (in screen percent) that the non-self nameplates are clamped to.",
	["nameplateOtherTopInset"] = "The inset from the top (in screen percent) that the non-self nameplates are clamped to.",
	["nameplateLargeBottomInset"] = "The inset from the bottom (in screen percent) that large nameplates are clamped to.",
	["nameplateLargeTopInset"] = "The inset from the top (in screen percent) that large nameplates are clamped to.",
	["nameplateClassResourceTopInset"] = "The inset from the top (in screen percent) that nameplates are clamped to when class resources are being displayed on them.",
	["ShowClassColorInNameplate"] = "use this to display the class color in the nameplate health bar",
	["ShowNamePlateLoseAggroFlash"] = "When enabled, if you are a tank role and lose aggro, the nameplate with briefly flash.",
	["NamePlateHorizontalScale"] = "Applied to horizontal size of all nameplates.",
	["NamePlateVerticalScale"] = "Applied to vertical size of all nameplates.",
]==]
local colorToTypeName = {
	["color1"] = "Physical",
	["color2"] = "Magic",
	["color3"] = "Curse",
	["color4"] = "Disease",
	["color5"] = "Poison",
	["color6"] = "Buff",
	["purge"] = "Purge",
}

local spellstringcache = {}

function NP:SpellString(spellid)
	if not spellstringcache[spellid] then
		local name, _, icon = GetSpellInfo(spellid)
		spellstringcache[spellid] = "\124T"..icon..":10\124t "..name
	end
	
	return spellstringcache[spellid]
end

local disableMouseover = true
local nop = function()end

local function DisableBlizzardNamePlates()
	NamePlateDriverFrame:UnregisterAllEvents()
	
	NamePlateDriverFrame.SetBaseNamePlateSize = function()
	--	print('T', 'NamePlateDriverFrame.SetBaseNamePlateSize')
	end
	
	
	ClassNameplateBarRogueDruidFrame:UnregisterAllEvents()
	ClassNameplateBarRogueDruidFrame:SetScript('OnEvent', nop)
	
	DeathKnightResourceOverlayFrame:UnregisterAllEvents()
	DeathKnightResourceOverlayFrame:SetScript('OnEvent', nop)
	
	ClassNameplateBarMageFrame:UnregisterAllEvents()
	ClassNameplateBarMageFrame:SetScript('OnEvent', nop)
	
	if ClassNameplateBarMonkFrame then
	ClassNameplateBarMonkFrame:UnregisterAllEvents()
	ClassNameplateBarMonkFrame:SetScript('OnEvent', nop)
	end
	
	ClassNameplateBarPaladinFrame:UnregisterAllEvents()
	ClassNameplateBarPaladinFrame:SetScript('OnEvent', nop)
	
	ClassNameplateBarWarlockFrame:UnregisterAllEvents()
	ClassNameplateBarWarlockFrame:SetScript('OnEvent', nop)
end

AleaUI:OnInit2(function()
	if NamePlateDriverMixin then
		DisableBlizzardNamePlates()	
	else
		AleaUI:OnAddonLoad('Blizzard_NamePlates', DisableBlizzardNamePlates)
	end
end)

local defaults = {

	reactions = {		
		tapped  		= { r = 0.6, g = 0.6, b = 0.6 },
		neutral 		= { r = 218/255, g = 197/255, b = 92/255 },
		enemy 			= { r = 0.78, g = 0.25, b = 0.25 },
		friendlyNPC 	= { r = 0.31, g = 0.45, b = 0.63},
		friendlyPlayer 	= { r = 75/255,  g = 175/255, b = 76/255},
	},
	
	
	showfromspelllist	= true,
	showmyauras			= true,
	showpurge			= true,
	
	backdropfadecolor 	= { .06, .06, .06,  0.3 },
	bordercolor 		= { 0,0,0,1 },
	font 				= E.media.default_font_name,
	fontSize 			= 12,
	fontOutline 		= 'OUTLINE',
	nonTargetAlpha 		= 0.6,
	healthBar_width 	= 108,
	healthBar_height 	= 18,
	healthBar_texture 	= E.media.default_bar_texture_name1,
	
	healthBar_name_xOffset = 0,
	healthBar_name_yOffset = 0,
	
	healthBar_hp_xOffset = 0,
	healthBar_hp_yOffset = 0,
	healthBar_hp_format  = 4,
	
	healthBar_lvl_xOffset = 3,
	healthBar_lvl_yOffset = 10,
	
	castBar_height 		= 6,
	castBar_color 		= {1,208/255,0 },
	castBar_noInterrupt = {0.78,0.25,0.25 },
	
	raidIcon_xOffset 	= -4,
	raidIcon_yOffset	= 6,
	raidIcon_size 		= 28,
	raidIcon_attachTo 	= 'LEFT',
	raidIcon_attachFrom = "RIGHT",
	
	auraSize			= 12,
	
	aura_xOffset = 0,
	aura_yOffset = 1,
	
	buffs = {
		numAuras 		= 6,
		stretchTexture 	= true,
		showPersonal	= false,
	},
	debuffs = {
		numAuras 		= 6,
		stretchTexture 	= true,
		showPersonal	= false,
	},
	colorByType 		= false,
	
	typecolors 			= {
		color1		= {0.80,0,0},
		color2		= {0.20,0.60,1.00},
		color3		= {0.60,0.00,1.00 },
		color4		= {0.60,0.40, 0 },
		color5		= {0.00,0.60,0 },
		color6		= {0.00,1.00,0 },
		purge		= { 1, 1, 1 },
	},
	
	spelllist = {},
	namelist = {},
	blacklist = {},
	
	nameplateMaxDistance = 60,
	nameplateOffsets = false,
	
	nameplateShowSelf = false,
	nameplateMotion = "STACKED",
	nameplateShowAll = true,
	nameplateShowFriendlyMinions = true,
	nameplateShowEnemyMinions = true,
	nameplateShowEnemyMinus = true,
}

local defaultSpells1 = {--Important spells, add them with huge icons.
	205369, -- Priest Shadow bomb
	6358, --Seduction

	33786, --Cyclone
	5782, --Fear
	5484, --Howl of Terror
	45438, --Ice Block
	642, --Divine Shield
	8122, --Psychic Scream

	23335, -- Silverwing Flag (alliance WSG flag)
	23333, -- Warsong Flag (horde WSG flag)
	34976, -- Netherstorm Flag (EotS flag)
	2094, --Blind
	33206, --Pain Suppression (priest) 
	29166, --Innervate (druid) 
	47585, --Dispersion (priest)

	87204, -- Shadow Priest 4pPVP

	108416, --Warlock Gertvenniy dogovor
	104773, --Warlock Tverdia reshimost
		
	871, --Warrior Shield Wall
	
	19263, --Hunter Deterance
	
	61336, --Druid
	
	31230, --Rogue
	
	6940, --Paladin
	31821, --Paladin
	
	48707, --DK
	
	108271, --Shaman

	53480, --Hunter

	15286, --Priest
	
	122783, --Monk
	122278, --Monk
	122465, --Monk

}

local defaultSpells2 = {--semi-important spells, add them with mid size icons.
	118038, -- warrior 
	45243, -- priest
	116849, -- monk poolp
	15487, --Silence (priest)
	10060, --Power Infusion (priest) 
	2825, --Bloodlust
	5246, --Intimidating Shout (warrior) 
	31224, --Cloak of Shadows (rogue) 
	498, --Divine Protection
	47476, --Strangulate (warlock) 
	31884, --Avenging Wrath (pally) 
	37587, --Bestial Wrath (hunter) 
	12472, --Icy Veins (mage)
	49039, --Lichborne (DK)
	48792, --Icebound Fortitude (DK)
	5277, --Evasion (rogue)
	53563, --Beacon of Light (pally)
	22812, --Barkskin (druid)
	67867, --Trampled (ToC arena spell when you run over someone)

    1022, --Hand of Protection (pally)

	46968, --Shockwave (warrior)
	46924, --Bladestorm (warrior)

	2983, --Sprint (rogue)
	2335, --Swiftness Potion
	6624, --Free Action Potion
	3448, --Lesser Invisibility Potion
	11464, --Invisibility Potion
	17634, --Potion of Petrification
	53905, --Indestructible Potion
	54221, --Potion of Speed
	1850, --Dash
	87204, --

	23920, --Warrior Reflect
	1719, --Warrior Recklessness
	18499, --Warrior Berserk
	71, --Warrior  Defence Stance
	107574, --Warrior Avatar
	12292, --Warrior Bleed
	114028, --Warriot Mass Reflect	

	124974, --Druid 

	102342, --Druid

	1966, -- Rogue faint	
	48263, --DK blood presens	
	77535, --DK Shield blood	

	3045, --Hunter
	19574, --Hunter

	--[[ INCAPACITATES ]]--
	-- Druid
	99, -- Incapacitating Roar (talent)
	-- Hunter
	3355,	-- Freezing Trap
	19386,  -- Wyvern Sting
	-- Mage
	118, -- Polymorph
	28272, -- Polymorph (pig)
	28271, -- Polymorph (turtle)
	61305, -- Polymorph (black cat)
	61025, -- Polymorph (serpent) -- FIXME: gone ?
	61721, -- Polymorph (rabbit)
	61780, -- Polymorph (turkey)
	82691, -- Ring of Frost
	31661, -- Dragon's Breath
	-- Monk
	115078, -- Paralysis
	-- Paladin
	20066, -- Repentance
	-- Priest
	605, -- Dominate Mind
	9484, -- Shackle Undead
	64044, -- Psychic Horror (Horror effect)
	88625, -- Holy Word: Chastise
	-- Rogue
	1776, -- Gouge
	6770, -- Sap
	-- Shaman
	51514, -- Hex
	-- Warlock
	710, -- Banish
	6789, -- Mortal Coil
	-- Pandaren
	107079, -- Quaking Palm

	--[[ SILENCES ]]--
	-- Death Knight
	47476, -- Strangulate
	-- Druid

	-- Mage

	-- Paladin
	31935, -- Avenger's Shield
	-- Priest
	15487, -- Silence
	-- Rogue
	1330, -- Garrote
	-- Blood Elf
	25046, -- Arcane Torrent (Energy version)
	28730, -- Arcane Torrent (Mana version)
	50613, -- Arcane Torrent (Runic power version)
	69179, -- Arcane Torrent (Rage version)
	80483, -- Arcane Torrent (Focus version)

	--[[ DISORIENTS ]]--
	-- Druid
	33786, -- Cyclone
	-- Paladin
	105421, -- Blinding Light -- FIXME: is this the right category? Its missing from blizzard's list

	-- Priest
	8122, -- Psychic Scream
	-- Rogue
	2094, -- Blind
	-- Warlock
	5782, -- Fear -- probably unused
	118699, -- Fear -- new debuff ID since MoP
	130616, -- Fear (with Glyph of Fear)
	5484, -- Howl of Terror (talent)
	115268, -- Mesmerize (Shivarra)
	6358, -- Seduction (Succubus)
	-- Warrior
	5246, -- Intimidating Shout (main target)

	--[[ STUNS ]]--
	-- Death Knight
	108194, -- Asphyxiate
	91800, -- Gnaw (Ghoul)
	91797, -- Monstrous Blow (Dark Transformation Ghoul)

	-- Druid
	22570, -- Maim
	5211, -- Mighty Bash
	1822, -- Rake (Stun from Prowl)
	163505, -- Rake (Stun from Prowl)
	-- Hunter
	109248, -- Binding Shot
	117526, -- Binding Shot
	19577, -- Intimidation
	24394, -- Intimidation
	-- Mage
	44572, -- Deep Freeze
	-- Monk

	113656, -- Fists of Fury
	120086, -- Fists of Fury
	119381, -- Leg Sweep
	-- Paladin
	853, -- Hammer of Justice

	-- Rogue
	1833, -- Cheap Shot
	408, -- Kidney Shot
	-- Shaman
	118345, -- Pulverize (Primal Earth Elemental)
	118905, -- Static Charge (Capacitor Totem)
	-- Warlock
	89766, -- Axe Toss (Felguard)
	30283, -- Shadowfury
	1122, --  Summon Infernal
	22703, -- Summon Infernal
	-- Warrior
	132168, -- Shockwave
	132169, -- Storm Bolt
	-- Tauren
	20549, -- War Stomp


	--[[ ROOTS ]]--
	-- Death Knight
	96294, -- Chains of Ice (Chilblains Root)
	-- Druid
	339, -- Entangling Roots
	102359, -- Mass Entanglement (talent)

	-- Hunter
	61685, 
	53148, -- Charge (Tenacity pet)
	135373, -- Entrapment (passive)
	136634, -- Narrow Escape (passive talent)
	-- Mage
	122, -- Frost Nova
	33395, -- Freeze (Water Elemental)
	-- Monk
	116095, --
	116706, -- Disable
	-- Priest
	114404, -- Void Tendrils
	-- Shaman
	64695, -- Earthgrab Totem
}

local bannedspells = {
	[15407] = true,
	[146198] = true,
}

local specific_defaultSpellsOpts = {
	[12292]  = { show = 1, spellID = 12292,  checkID = true, size = 1.5 },
	[163505] = { show = 1, spellID = 163505, checkID = true, size = 1.5 },
	[123393] = { show = 1, spellID = 123393, checkID = true, size = 2 },
}
for k,v in pairs(defaultSpells1) do	
	if GetSpellInfo(v) then
		defaults.spelllist[GetSpellInfo(v)] = specific_defaultSpellsOpts[v] or { show = 1, spellID = v, checkID = false, size = 2 }
	else
		print('defaultSpells1', v, 'is Unknown')
	end	
end
wipe(defaultSpells1)
for k,v in pairs(defaultSpells2) do	
	if GetSpellInfo(v) then
		defaults.spelllist[GetSpellInfo(v)] = specific_defaultSpellsOpts[v] or { show = 1, spellID = v, checkID = false, size = 1.5 }
	else
		print('defaultSpells2', v, 'is Unknown')
	end
end
wipe(defaultSpells2)
for k,v in pairs(bannedspells) do
	if GetSpellInfo(k) then
		defaults.blacklist[GetSpellInfo(k)] = specific_defaultSpellsOpts[v] or { show = 2, spellID = k, checkID = false, size = 1 }
	else
		print('bannedspells', k, 'is Unknown')
	end
end
wipe(bannedspells)

E.default_settings.nameplates = defaults

local GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit

NP:RegisterEvent("NAME_PLATE_CREATED");
NP:RegisterEvent("NAME_PLATE_UNIT_ADDED");
NP:RegisterEvent("NAME_PLATE_UNIT_REMOVED");
NP:RegisterEvent("PLAYER_TARGET_CHANGED");
NP:RegisterEvent("DISPLAY_SIZE_CHANGED");
NP:RegisterEvent("CVAR_UPDATE");
NP:RegisterEvent("RAID_TARGET_UPDATE");
NP:RegisterEvent('UPDATE_MOUSEOVER_UNIT')

NP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
NP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
NP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
NP:RegisterEvent("UNIT_SPELLCAST_START")
NP:RegisterEvent("UNIT_SPELLCAST_DELAYED")
NP:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
NP:RegisterEvent("UNIT_SPELLCAST_STOP")
NP:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
NP:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
NP:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
	
	
function NP:NAME_PLATE_CREATED(event, realFrame)
	NP:CreateNamePlateFrame(realFrame)
end

function NP:NAME_PLATE_UNIT_ADDED(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP.plateNamePlate = nil
		realFrame.AleaNP:Show()
		realFrame.AleaNP.unit = unit
		realFrame.AleaNP.guid = UnitGUID(unit)
		
		realFrame.AleaNP.isPlayer = UnitIsPlayer(unit)
		realFrame.AleaNP.isFriend = UnitIsFriend("player",unit)
		
		
		realFrame.AleaNP:RegisterEvent('UNIT_AURA')
		realFrame.AleaNP:RegisterEvent('UNIT_HEALTH_FREQUENT')
		realFrame.AleaNP:RegisterEvent('UNIT_MAXHEALTH')
		realFrame.AleaNP:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		realFrame.AleaNP:RegisterEvent('UNIT_NAME_UPDATE')
		realFrame.AleaNP:RegisterEvent('UNIT_FACTION')
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_START")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_STOP")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
		realFrame.AleaNP:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")

		realFrame.AleaNP:OnShowUpdate()
		
		if UnitIsUnit(unit, 'target') and not UnitIsUnit(unit, 'player') then
			realFrame.AleaNP:SetTargetWidth()
		else
			realFrame.AleaNP:SetNormalWidth()
		end
	end
end

function NP:NAME_PLATE_UNIT_REMOVED(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:Hide()
		
		realFrame.AleaNP.guid = nil
		realFrame.AleaNP.isPlayer = false
		realFrame.AleaNP.isFriend = false
		
		NP:RemovePowerBar(realFrame.AleaNP)
		
		realFrame.AleaNP:UnregisterAllEvents()
		
		realFrame.AleaNP.unit = nil
		realFrame.AleaNP.plateNamePlate = nil
		realFrame.AleaNP:SetNormalWidth()
	end
end

function NP:UNIT_SPELLCAST_INTERRUPTIBLE(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP.castBar.notInterruptible = false
	end
end

function NP:UNIT_SPELLCAST_NOT_INTERRUPTIBLE(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP.castBar.notInterruptible = true
	end
end

function NP:UNIT_SPELLCAST_CHANNEL_START(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateCast()
	end
end
function NP:UNIT_SPELLCAST_CHANNEL_STOP(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:StopCast()
	end
end

function NP:UNIT_SPELLCAST_CHANNEL_UPDATE(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateCast()
	end
end

function NP:UNIT_SPELLCAST_START(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateCast()
	end
end
function NP:UNIT_SPELLCAST_DELAYED(event, unit) 
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateCast()
	end
end
function NP:UNIT_SPELLCAST_INTERRUPTED(event, unit) 
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:StopCast()
	end
end
function NP:UNIT_SPELLCAST_STOP(event, unit) 
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:StopCast()
	end
end
function NP:UNIT_SPELLCAST_SUCCEEDED(event, unit) 
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:StopCast(true)
	end
end

function NP:PLAYER_TARGET_CHANGED(event)
	if UnitExists('target') then
		local realFrame = GetNamePlateForUnit('target')		
		for _, frame in pairs(C_NamePlate.GetNamePlates()) do		
			if frame ~= realFrame then
				frame.AleaNP:SetAlpha(0.6)
				frame.AleaNP:UpdateFrameLevel()
				frame.AleaNP:SetNormalWidth()
			end
		end
		
		if ( realFrame ) then
			realFrame.AleaNP:SetAlpha(1)	
			realFrame.AleaNP:UpdateAuras()
			realFrame.AleaNP:UpdateFrameLevel()
			
			if not UnitIsUnit(realFrame.AleaNP.unit, 'player') then
				realFrame.AleaNP:SetTargetWidth()
			else
				realFrame.AleaNP:SetNormalWidth()
			end
		end
	else
		for _, frame in pairs(C_NamePlate.GetNamePlates()) do	
			frame.AleaNP:SetAlpha(1)
			frame.AleaNP:UpdateFrameLevel()
			frame.AleaNP:SetNormalWidth()
		end
	end
end

function NP:DISPLAY_SIZE_CHANGED(event)

end

function NP:UpdateCustomName()
	for _, frame in pairs(C_NamePlate.GetNamePlates()) do	
		if frame.AleaNP.unit then
			frame.AleaNP:UpdateName()
		end
	end
end
	

function NP.EventHandler(self, event, unit)
	if event == 'UNIT_THREAT_SITUATION_UPDATE' then
		self:UpdateAggro()
		self:UpdateFaction()
	elseif unit and unit == self.unit then
		if event == 'UNIT_AURA' then
			self:UpdateAuras()
		elseif event == 'UNIT_HEALTH' or event == 'UNIT_MAXHEALTH' or event == 'UNIT_HEALTH_FREQUENT' then
			self:UpdateHealth()
		elseif event == 'UNIT_FACTION' then
			self:UpdateName()
			self:UpdateFaction()
		elseif event == 'UNIT_NAME_UPDATE' then
			self:UpdateName()		
		elseif event == "UNIT_SPELLCAST_CHANNEL_START" or
				event == "UNIT_SPELLCAST_CHANNEL_UPDATE" or
				event == "UNIT_SPELLCAST_START" or
				event == "UNIT_SPELLCAST_DELAYED" then
			self:UpdateCast()
		elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" or
				event == "UNIT_SPELLCAST_INTERRUPTED" or 
				event == "UNIT_SPELLCAST_STOP" then
			self:StopCast()
		elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
			self:StopCast(true)
		elseif event == "UNIT_SPELLCAST_INTERRUPTIBLE" then
			self.castBar.notInterruptible = false
		elseif event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" then
			self.castBar.notInterruptible = true
		end
	end
end

--[==[
function NP:UNIT_AURA(event, unit)
	local realFrame = GetNamePlateForUnit(unit)	
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateAuras()
	end
end

function NP:UNIT_HEALTH(event, unit)
	local realFrame = GetNamePlateForUnit(unit)	
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateHealth()
	end
end
NP.UNIT_MAXHEALTH = NP.UNIT_HEALTH
]==]

local raidIndexCoord = {
	[1] = { 0, .25, 0, .25 }, --"STAR"
	[2] = { .25, .5, 0, .25}, --MOON
	[3] = { .5, .75, 0, .25}, -- CIRCLE
	[4] = { .75, 1, 0, .25}, -- SQUARE
	[5] = { 0, .25, .25, .5}, -- DIAMOND
	[6] = { .25, .5, .25, .5}, -- CROSS
	[7] = { .5, .75, .25, .5}, -- TRIANGLE
	[8] = { .75, 1, .25, .5}, --  SKULL
}

function NP:RAID_TARGET_UPDATE(event)
	for _, frame in pairs(C_NamePlate.GetNamePlates()) do
		local icon = frame.AleaNP.raidIcon
		local index = GetRaidTargetIndex(frame.AleaNP.unit)

		if ( index and raidIndexCoord[index] ) then
			icon:Show()
			icon:SetTexCoord(raidIndexCoord[index][1], raidIndexCoord[index][2], raidIndexCoord[index][3], raidIndexCoord[index][4])
		else
			icon:Hide()
		end
	end
end

function NP:CVAR_UPDATE(event, name)
--	print('NP:CVAR_UPDATE', name)
end

function NP:UNIT_NAME_UPDATE(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateName()
	end
end

local RAID_CLASS_COLORS = RAID_CLASS_COLORS

local function IsTapDenied(unit)
	return not UnitPlayerControlled(unit) and UnitIsTapDenied(unit);
end

local function IsOnThreatList(threatStatus)
	return threatStatus ~= nil
end

local function IsTapped(unit)
	local _, threatStatus = UnitDetailedThreatSituation("player", unit);
	return IsOnThreatList(threatStatus);
end

local mouseoverUpdate = CreateFrame('Frame')
mouseoverUpdate:Hide()
mouseoverUpdate:SetScript('OnUpdate', function(self, elapsed)
	if UnitExists('mouseover') then 
	
		if mouseoverUpdate.overlayFrame then
			if MouseIsOver(mouseoverUpdate.overlayFrame) then
				if not mouseoverUpdate.overlayFrame.AleaNP.mouseIsOver then
					mouseoverUpdate.overlayFrame.AleaNP.mouseIsOver = true				
					mouseoverUpdate.overlayFrame.AleaNP.overlay:Show()
					mouseoverUpdate.overlayFrame.AleaNP:UpdateFrameLevel()					
				end
			else
				mouseoverUpdate.overlayFrame.AleaNP.mouseIsOver = false
				mouseoverUpdate.overlayFrame.AleaNP.overlay:Hide()
				mouseoverUpdate.overlayFrame.AleaNP:UpdateTargetAlpha()
				mouseoverUpdate.overlayFrame.AleaNP:UpdateFrameLevel()
			end
		end
		return 
	end
	
	for _, frame in pairs(C_NamePlate.GetNamePlates()) do
		frame.AleaNP.mouseIsOver = false
		frame.AleaNP.overlay:Hide()
	end

	self:Hide()
end)

function NP:UPDATE_MOUSEOVER_UNIT()
	local realFrame = GetNamePlateForUnit('mouseover')
	
	if disableMouseover then
		for _, frame in pairs(C_NamePlate.GetNamePlates()) do
			if frame ~= realFrame then
				frame.AleaNP.mouseIsOver = false
				frame.AleaNP.overlay:Hide()
			end
		end
		
		mouseoverUpdate.overlayFrame = nil
	end
	
	if realFrame then
		if disableMouseover and not realFrame.AleaNP.plateNamePlate then
			mouseoverUpdate.overlayFrame = realFrame
			mouseoverUpdate:Show()	
		end
		
		realFrame.AleaNP:UpdateAuras()
	end
end

function NP:UNIT_FACTION(event, unit)
	local realFrame = GetNamePlateForUnit(unit)
	
	if ( realFrame ) then
		realFrame.AleaNP:UpdateName()
		realFrame.AleaNP:UpdateFaction()
	end
end

function NP:UpdateFaction()
	if not self.unit then return end
	
	local localizedClass, englishClass = UnitClass(self.unit);
	local classColor = RAID_CLASS_COLORS[englishClass];

	local r, g, b

	if self.plateNamePlate then	
		r, g, b = 0.1, 0.6, 0.1
	elseif self.isPlayer and classColor then
		-- Use class colors for players if class color option is turned on
		r, g, b = classColor.r, classColor.g, classColor.b;
	elseif ( IsTapDenied(self.unit) ) then
		-- Use grey if not a player and can't get tap on unit
		r, g, b = E.db.nameplates.reactions.tapped.r, E.db.nameplates.reactions.tapped.g, E.db.nameplates.reactions.tapped.b
	elseif ( true ) then
		-- Use color based on the type of unit (neutral, etc.)
		if ( IsTapped(self.unit) ) then
			r, g, b = E.db.nameplates.reactions.enemy.r, E.db.nameplates.reactions.enemy.g, E.db.nameplates.reactions.enemy.b
		else
			r, g, b = UnitSelectionColor(self.unit, true);
			
			if r > 0.9 and g == 0 and b == 0 then
				r, g, b = E.db.nameplates.reactions.enemy.r, E.db.nameplates.reactions.enemy.g, E.db.nameplates.reactions.enemy.b
			elseif r > 0.9 and g > 0.9 and b == 0 then		
				r, g, b = E.db.nameplates.reactions.neutral.r, E.db.nameplates.reactions.neutral.g, E.db.nameplates.reactions.neutral.b
			end
		end
	elseif self.isFriend then
		r, g, b = E.db.nameplates.reactions.friendlyNPC.r, E.db.nameplates.reactions.friendlyNPC.g, E.db.nameplates.reactions.friendlyNPC.b
	else
		r, g, b = E.db.nameplates.reactions.enemy.r, E.db.nameplates.reactions.enemy.g, E.db.nameplates.reactions.enemy.b
	end
	
	local classif = UnitClassification(self.unit)
	
	if classif == 'elite' or classif == 'worldboss' then
		self.eliteIcon:Show()
		self.eliteIcon:SetDesaturated(false)
	elseif classif == 'rare' or classif == 'rareelite' then
		self.eliteIcon:Show()
		self.eliteIcon:SetDesaturated(true)
	else
		self.eliteIcon:Hide()
	end
	
	local level = UnitLevel(self.unit)
	if ( UnitIsWildBattlePet(self.unit) or UnitIsBattlePetCompanion(self.unit) ) then
		self.levelText:SetText(UnitBattlePetLevel(self.unit))
	elseif level == UnitLevel('player') then
		self.levelText:SetText('')
	elseif (level > UnitLevel('player')+3) then
		self.levelText:SetText('|cffff0000'..level.."|r")
	elseif(level > UnitLevel('player')) then
		self.levelText:SetText('|cffF2F266'..level.."|r")
	elseif level > 0 then
		self.levelText:SetText('|cff4baf4c'..level.."|r")
	else
		self.levelText:SetText('|cffcc7f00??|r')
	end

	if self._color_r ~= r or
		self._color_g ~= g or
		self._color_b ~= b then
	
		self:SetStatusBarColor(r, g, b, 1)
		
		self._color_r, self._color_g, self._color_b = r, g, b
	end
end

function NP:UpdateName()
	if not self.unit then return end
	
	local name = UnitName(self.unit)
	
	if E.db.nameplates.namelist[name] and E.db.nameplates.namelist[name].show then
		name = E.db.nameplates.namelist[name].subname or name
	end
	
	self.nameText:SetText(name)
end
function NP:UpdateTargetAlpha()
	if not self.unit then return end
	
	if UnitExists('target') then
		if UnitIsUnit('target', self.unit) then
			self:SetAlpha(1)
		else
			self:SetAlpha(0.6)
		end
	else
		self:SetAlpha(1)
	end
end

function NP:UpdateFrameLevel()
	if not self.unit then return end
	--if true then return end
	
	if UnitIsUnit('target', self.unit) then
		self:SetFrameLevel(4)
	--elseif UnitIsUnit('mouseover', self.unit) and self.mouseIsOver then
	--	self:SetFrameLevel(4)	
	else
		self:SetFrameLevel(2)
	end
end

do
	function NP:ShortValue(v)
		if v >= 1e9 then
			return ("%.1fG"):format(v / 1e9)
		elseif v >= 1e6 then
			return ("%.1fM"):format(v / 1e6)
		elseif v >= 1e3 or v <= -1e3 then
			return ("%.1fk"):format(v / 1e3)
		else
			return v
		end
	end
	
	local floor = floor
	function NP:ShortPercValue(v)

		if v == 100 then
			return ('%d'):format(v)
		end

		return ('%.1f'):format(v)
	end
end

function NP:UpdateHealth()
	if not self.unit then return end
	
	local maxHP = UnitHealthMax(self.unit)
	local curHP = UnitHealth(self.unit)
	
	self:SetMinMaxValues(0, maxHP)
	self:SetValue(curHP)
	
	if E.db.nameplates.healthBar_hp_format == 1 then
		self.healthText:SetText('')
	elseif E.db.nameplates.healthBar_hp_format == 2 then
		self.healthText:SetFormattedText('%s%%', NP:ShortPercValue(curHP/maxHP*100))
	elseif E.db.nameplates.healthBar_hp_format == 3 then
		self.healthText:SetFormattedText('%s', NP:ShortValue(curHP))
	else
		self.healthText:SetFormattedText('%s - %s%%', NP:ShortValue(curHP), NP:ShortPercValue(curHP/maxHP*100))
	end
end

--options.spelllist[GetSpellInfo(v)] = specific_defaultSpellsOpts[v] or { show = 1, spellID = v, checkID = false, size = 2 }
--options.blacklist[GetSpellInfo(k)] = specific_defaultSpellsOpts[v] or { show = 2, spellID = k, checkID = false, size = 1 }

do

	local icons = {}
	
	local timeout = 0
	local function OnUpdate(self, elapsed)
		timeout = timeout + elapsed
		if timeout < 0.1 then return end
		timeout = 0
		
		local getTime = GetTime()
		local hide = true
		for frame, expiration in pairs(icons) do
			hide = false
			frame.timer:SetFormattedText(NP:formatTimeRemaining2(expiration-getTime))		
		end
		
		if hide then
			self:Hide()
		end
	end
	
	local onUpdater = CreateFrame('Frame')
	onUpdater:Hide()
	onUpdater:SetScript('OnUpdate', OnUpdate)
	
	function NP:AuraIcon_Add(frame, expiration)
		icons[frame] = expiration
		onUpdater:Show()
	end
	
	function NP:AuraIcon_Remove(frame)
		if icons[frame] then
			icons[frame] = nil
			OnUpdate(onUpdater, 1)
		end
	end
	
end

do
	
	local hour, minute = 3600, 60
	local format = string.format
    local ceil = math.ceil
	local floor = math.floor
	local fmod = math.fmod

	function NP:formatTimeRemaining2(msecs)
		if msecs < 0 then msecs = 0 end
		
		if msecs >= hour then
			return "%dh", ceil(msecs / hour)
		elseif msecs >= minute then
			return "%dm", ceil(msecs / minute)
		else
			return "%.0f", ceil(msecs)
		end
    end
end

local types_to_color = {
	["BUFF"] = "color6",
	["DEBUFF"] = "color1",
	["Poison"] = "color5",
	["Magic"] = "color2",
	["Disease"] = "color4",
	["Curse"] = "color3",
	["purge"] = "purge",
}

local function UpdateBorderColor(self, types)
	if types == "black" or types == nil then
		self:SetBorderColor(E.db.nameplates.bordercolor[1],E.db.nameplates.bordercolor[2],E.db.nameplates.bordercolor[3],1)
	else
		local c = E.db.nameplates.typecolors[types_to_color[types] or "color1"]
		self:SetBorderColor(c[1],c[2],c[3],1)
	end
end

local function FillAuraFrame(frame, unit, filter)

	local index = 1
	local frameIndex = 1
	local frameSize = 1
	local isfriend = UnitIsFriend(unit, "player")
	
	while true do
	
		local name, rank, texture, count, debuffType, duration, expirationTime, caster, isStealable, nameplateShowPersonal, spellID, _, _, _, nameplateShowAll = UnitAura(unit, index, filter)
		
		if not name then break end
		if not frame.icons[frameIndex] then break end
		--[==[
		if E.StunList[spellID] == nil then
			E.StunList[spellID] = false
			
			local tt = E.GetCustomTooltip()
			tt:SetUnitAura(unit, index, filter)
			
			local result, text = E.GetAuraControllType(tt)
		
			if result then
			--	print('Aura', name, result, text)
				if result == 'STUN' then
					E.StunList[spellID] = result
				end
				
				if AleaUIDB then	
					if not AleaUIDB.spellContol then						
						AleaUIDB.spellContol = {}						
					end
					
					if not AleaUIDB.spellContol[spellID] then
						AleaUIDB.spellContol[spellID] = result
					end
				end
			end
		end
		]==]
		
		local skip = false
		
		local arg = E.db.nameplates.blacklist[name]
		if arg then
			if arg.checkID then
				if arg.spellID == spellID then	
					skip = true;
				end
			else
				skip = true;
			end
		end
	
	
		if skip then
			break
		else
			local arg = E.db.nameplates.spelllist[name]
			local skip2 = false
			local localsize = E.db.nameplates.auraSize
			
		--	print('T1', name, arg)
			
			if arg then
				if true then --C.db.profile.showspellformlist
					if arg.checkID then
						if arg.spellID == spellID then			
							if arg.show == 1 then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
							elseif arg.show == 3 and ( caster and UnitIsUnit(caster, 'player') ) then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
							elseif arg.show == 4 and auraTarget == AURA_TARGET_HOSTILE then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
							elseif arg.show == 5 and auraTarget == AURA_TARGET_FRIENDLY then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
							else skip2 = false;
							end
						else skip2 = false;
						end
					else
						if arg.show == 1 then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
						elseif arg.show == 3 and ( caster and UnitIsUnit(caster, 'player') ) then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
						elseif arg.show == 4 and auraTarget == AURA_TARGET_HOSTILE then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
						elseif arg.show == 5 and auraTarget == AURA_TARGET_FRIENDLY then skip2 = true; localsize = E.db.nameplates.auraSize*arg.size;
						else skip2 = false;
						end	
					end
				end
			end
			
		--	print('T2', name, arg, skip2, arg.show)
			
			if ( caster and UnitIsUnit(caster, 'player') ) or
			   ( nameplateShowAll ) or
			   ( filter == 'HELPFUL' and not isfriend and isStealable ) or
			   ( skip2 ) or E.spellContol[spellID] then
			
			
				frame.icons[frameIndex].icon:SetTexture(texture)
				
				
				if E.db.nameplates.showpurge and isStealable then
					UpdateBorderColor(frame.icons[frameIndex], 'purge')
				elseif E.db.nameplates.colorByType then
					UpdateBorderColor(frame.icons[frameIndex], debuffType)
				else
					UpdateBorderColor(frame.icons[frameIndex], "black")
				end
				
				
				if duration and duration > 0 then
					frame.icons[frameIndex].timer:SetFormattedText(NP:formatTimeRemaining2(expirationTime-GetTime()))
					frame.icons[frameIndex].timer:Show()
					
					NP:AuraIcon_Add(frame.icons[frameIndex], expirationTime)
				else
					frame.icons[frameIndex].timer:Hide()					
					NP:AuraIcon_Remove(frame.icons[frameIndex])
				end
				
				if count and count > 1 then
					frame.icons[frameIndex].stack:SetText(count)
					frame.icons[frameIndex].stack:Show()
				else
					frame.icons[frameIndex].stack:Hide()
				end
				
				frame.icons[frameIndex]:Show()	
				frame.icons[frameIndex]:SetSize(localsize, localsize)
				
				if frameSize < localsize then
					frameSize = localsize
				end
				
				frameIndex = frameIndex + 1
			end
		end		
		index = index + 1
	end
	if frameSize > 1 then	
		frame:SetSize(frameSize+6, frameSize+6)
	else
		frame:SetSize(frameSize, frameSize)
	end
	
	for i=frameIndex, 6 do
		frame.icons[i]:Hide()
		NP:AuraIcon_Remove(frame.icons[i])
	end
end

local function FillAuraFramePersonal(frame, unit, filter)

	local index = 1
	local frameIndex = 1
	local frameSize = 1
	
	while true do
	
		local name, rank, texture, count, debuffType, duration, expirationTime, caster, isStealable, nameplateShowPersonal, spellID, _, _, _, nameplateShowAll = UnitAura(unit, index, filter)
		
		if not name then break end
		if not frame.icons[frameIndex] then break end
		
		local skip = false
		
		local arg = E.db.nameplates.blacklist[name]
		if arg then
			if arg.checkID then
				if arg.spellID == spellID then	
					skip = true;
				end
			else
				skip = true;
			end
		end
	
	
		if skip then
			break
		else
			local localsize = E.db.nameplates.auraSize
		
			if ( nameplateShowAll or (nameplateShowPersonal and (caster == "player" or caster == "pet" or caster == "vehicle")) ) then
			
				frame.icons[frameIndex].icon:SetTexture(texture)
				
				UpdateBorderColor(frame.icons[frameIndex], "black")
				
				if duration and duration > 0 then
					frame.icons[frameIndex].timer:SetFormattedText(NP:formatTimeRemaining2(expirationTime-GetTime()))
					frame.icons[frameIndex].timer:Show()
					
					NP:AuraIcon_Add(frame.icons[frameIndex], expirationTime)
				else
					frame.icons[frameIndex].timer:Hide()					
					NP:AuraIcon_Remove(frame.icons[frameIndex])
				end
				
				if count and count > 1 then
					frame.icons[frameIndex].stack:SetText(count)
					frame.icons[frameIndex].stack:Show()
				else
					frame.icons[frameIndex].stack:Hide()
				end
				
				frame.icons[frameIndex]:Show()	
				frame.icons[frameIndex]:SetSize(localsize, localsize)
				
				if frameSize < localsize then
					frameSize = localsize
				end
				
				frameIndex = frameIndex + 1
			end
		end		
		index = index + 1
	end
	if frameSize > 1 then	
		frame:SetSize(frameSize+6, frameSize+6)
	else
		frame:SetSize(frameSize, frameSize)
	end
	
	for i=frameIndex, 6 do
		frame.icons[i]:Hide()
		NP:AuraIcon_Remove(frame.icons[i])
	end
end


function NP:UpdateAuras()
	if not self.unit then return end	
	
	if self.plateNamePlate then
		FillAuraFramePersonal(self.DebuffFrame, self.unit, 'HELPFUL')
	else
		FillAuraFrame(self.DebuffFrame, self.unit, 'HARMFUL')
		FillAuraFrame(self.BuffFrame, self.unit, 'HELPFUL')	
	end
end

local green =  {0, 1, 0}

local function NamePlate_CastBarOnUpdate(self)
	
	
	local color
	
	if self.castType == 'channel' then
		local curValue = self.duration - ( GetTime() - self.startTime )
		
		if curValue >= 0 then
			self:SetValue(curValue)
			self.time:SetFormattedText('%.1fc', curValue)
			
			if curValue > 0 and (curValue/self.duration) <= 0.02 then
				color = green
			elseif self.notInterruptible then
				color = E.db.nameplates.castBar_noInterrupt
			else
				color = E.db.nameplates.castBar_color
			end

			self:SetStatusBarColor(color[1], color[2], color[3])
		else
			self.plate:StopCast()
		end
	else
		local curValue = GetTime() - self.startTime
		if curValue >= 0 then
			self:SetValue(curValue)
			self.time:SetFormattedText('%.1fc', curValue)
			
			if curValue > 0 and (curValue/self.duration) >= 0.98 then
				color = green
			elseif self.notInterruptible then
				color = E.db.nameplates.castBar_noInterrupt
			else
				color = E.db.nameplates.castBar_color
			end
			
			self:SetStatusBarColor(color[1], color[2], color[3])
		else
			self.plate:StopCast()
		end
	end
end

function NP:UpdateCast()
	if not self.unit then return end
	if self.plateNamePlate then 
		self.castBar.castType = nil
		self.castBar.startTime = nil
		self.castBar.endTime = nil
		self.castBar.notInterruptible = nil
		self.castBar:Hide()
		return 
	end
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(self.unit)
	
	if name then		
		self.castBar.castType = 'channel'
		self.castBar.startTime = startTime*0.001
		self.castBar.endTime = endTime*0.001
		self.castBar.duration = ( endTime - startTime ) * 0.001
		self.castBar.notInterruptible = notInterruptible
		
		self.castBar:SetMinMaxValues(0, self.castBar.duration )
		self.castBar:SetValue(0)
		self.castBar.name:SetText(name)
		self.castBar.icon:SetTexture(texture)
		
		self.castBar:Show()
		NamePlate_CastBarOnUpdate(self.castBar)
		return
	end
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo(self.unit)
	if name then 		
		self.castBar.castType = 'normal'
		self.castBar.startTime = startTime*0.001
		self.castBar.endTime = endTime*0.001
		self.castBar.duration = ( endTime - startTime ) * 0.001
		self.castBar.notInterruptible = notInterruptible
		
		self.castBar:SetMinMaxValues(0, self.castBar.duration )
		self.castBar:SetValue(0)
		self.castBar.name:SetText(name)
		self.castBar.icon:SetTexture(texture)
		
		self.castBar:Show()
		NamePlate_CastBarOnUpdate(self.castBar)
		return 
	end
	
	self:StopCast()
end

function NP:StopCast(notChannel)

	if notChannel and self.castBar.castType == 'channel' then
		return
	end
	
	self.castBar.castType = nil
	self.castBar.startTime = nil
	self.castBar.endTime = nil
	self.castBar.notInterruptible = nil
	self.castBar:Hide()
end

function NP:UpdateRaidIcon()
	if not self.unit then return end
	
	local index = GetRaidTargetIndex(self.unit)

	if ( index and raidIndexCoord[index] ) then
		self.raidIcon:Show()
		self.raidIcon:SetTexCoord(raidIndexCoord[index][1], raidIndexCoord[index][2], raidIndexCoord[index][3], raidIndexCoord[index][4])
	else
		self.raidIcon:Hide()
	end
end

function NP:SetupPlayerPlate(plate)
	if UnitIsUnit('player', plate.unit) then
		plate.plateNamePlate = true
		plate.levelText:Hide()
		plate.nameText:Hide()
		plate.healthText:Hide()
		NP:SetupPowerBar(plate)
	else
		plate.plateNamePlate = false
		plate.levelText:Show()
		plate.nameText:Show()
		plate.healthText:Show()
		
		NP:RemovePowerBar(plate)
	end
end

function NP:OnShowUpdate()
	NP:SetupPlayerPlate(self)
	
	self:UpdateName()
	self:UpdateFaction()
	self:UpdateTargetAlpha()
	self:UpdateFrameLevel()
	self:UpdateHealth()
	self:UpdateAuras()
	self:UpdateCast()
	self:UpdateRaidIcon()
	self:UpdateAggro()
end


NP.PerClassPowerBar = {
	['WARLOCK'] = {
		powerToken = SPELL_POWER_SOUL_SHARDS,
		maxValue = 5,
		color = { RAID_CLASS_COLORS['WARLOCK'].r, RAID_CLASS_COLORS['WARLOCK'].g, RAID_CLASS_COLORS['WARLOCK'].b, 1 },
	},
}

local hiden = CreateFrame('Frame')
hiden:Hide()

function NP:RemovePowerBar(plate)
	if plate.havePowerBar then
		plate.havePowerBar = false
		
		if NP.powerBar.parent == plate then
			NP.powerBar:SetParent(hiden)
			NP.powerBar.parent = nil
		end
		
		plate:SetHeight(E.db.nameplates.healthBar_height)
--		plate:SetPoint('BOTTOMLEFT', plate.owner, 'BOTTOMLEFT', 0, 0)
--		plate:SetPoint('BOTTOMRIGHT', plate.owner, 'BOTTOMRIGHT', 0, 0)		
		plate:SetPoint('BOTTOM', plate.owner, 'BOTTOM', 0, 10)
		
		plate.DebuffFrame:SetPoint("BOTTOMLEFT", plate, 'TOPLEFT', 0, 12)
		plate.DebuffFrame:SetPoint("BOTTOMRIGHT", plate, 'TOPRIGHT', 0, 12)
	end
end

function NP:UpdatePowerBar()
	local maxV = UnitPowerMax("player", NP.powerBar.powerToken)		
	self:SetMinMaxValues(0, NP.powerBar.maxValue or maxV or 100)
	self:SetValue(UnitPower("player", NP.powerBar.powerToken))
end

function NP:SetupPowerBar(plate)
	
	if not NP.powerBar then
		NP.powerBar = CreateFrame('StatusBar')
		NP.powerBar:RegisterUnitEvent("UNIT_POWER_FREQUENT", "player");
		NP.powerBar:SetScript('OnEvent', NP.UpdatePowerBar)
		NP.powerBar:SetScript('OnShow', NP.UpdatePowerBar)
		NP:CreateBackdrop(NP.powerBar)
	end
	
	if not plate.havePowerBar or NP.powerBar.parent ~= plate then		
		plate.havePowerBar = true
		
		NP.UpdatePowerBar(NP.powerBar)
		
		NP.powerBar:SetParent(plate)
		
		NP.powerBar.parent = plate
		
		plate:SetHeight(5)
		
		plate.DebuffFrame:SetPoint("BOTTOMLEFT", plate, 'TOPLEFT', 0, 2)
		plate.DebuffFrame:SetPoint("BOTTOMRIGHT", plate, 'TOPRIGHT', 0, 2)
	
	--	plate:SetPoint('BOTTOMLEFT', plate.owner, 'BOTTOMLEFT', 0, 5)
	--	plate:SetPoint('BOTTOMRIGHT', plate.owner, 'BOTTOMRIGHT', 0, 5)
		plate:SetPoint('BOTTOM', plate.owner, 'BOTTOM', 0, 0)
		
		NP.powerBar:SetPoint('TOPLEFT', plate, 'BOTTOMLEFT', 0, 0)
		NP.powerBar:SetPoint('TOPRIGHT', plate, 'BOTTOMRIGHT', 0, 0)
		NP.powerBar:SetSize(5, 5)
	
		NP.powerBar:SetStatusBarTexture(E:GetTexture(E.db.nameplates.healthBar_texture))
		
		if NP.PerClassPowerBar[E.myclass] then
			local color = NP.PerClassPowerBar[E.myclass].color or UF:PowerColorRGB("player")		
			NP.powerBar:SetStatusBarColor(color[1], color[2], color[3])
			
			NP.powerBar.powerToken = NP.PerClassPowerBar[E.myclass].powerToken or UnitPowerType('player')
			NP.powerBar.maxValue = NP.PerClassPowerBar[E.myclass].maxValue
		else
			local color = UF:PowerColorRGB("player")
		
			NP.powerBar:SetStatusBarColor(color[1], color[2], color[3])
			NP.powerBar.powerToken = UnitPowerType('player')
			NP.powerBar.maxValue = nil
		end	
	end
end



function NP.SetTargetWidth(self)
	if not self.isTarget then
		self.isTarget = true
		self.sizer.dir = 1
		self.sizer.step = max(self.defaultIncrease/GetFramerate(), self.defaultIncrease/30) * 5
		self.sizer.to = E.db.nameplates.healthBar_width
		self.sizer:Show()
	end
end


function NP.SetNormalWidth(self)
	if self.isTarget then
		self.isTarget = false
		self.sizer.dir = -1
		self.sizer.step = max(self.defaultIncrease/GetFramerate(), self.defaultIncrease/30) * 5
		self.sizer.to = self.defaultWidth
		self.sizer:Show()
	end
end

local ThreatStatusColor = {
	[0] = { 0.7, 0.7, 0.7 },
	[1] = { 1, 1, 0.47 },
	[2] = { 1, 0.5, 0 },
	[3] = { 1, 0.3, 0.3 },
}
function NP.UpdateAggro(self)
	local status = UnitThreatSituation('player', self.unit)

	if (status and status > 0) then
		self.glowIndicator:SetBackdropBorderColor(ThreatStatusColor[status][1], ThreatStatusColor[status][2], ThreatStatusColor[status][3])
		self.glowIndicator:Show()
	else
		self.glowIndicator:Hide()
	end
end

local NamePlate_DRBarOnUpdate = function(self, elapsed)
	local guid = self.plate.guid
	
	if not E.StunTimer[guid] or E.StunTimer[guid] < GetTime() then
		E.StunTimer[guid] = nil
		E.StunDRs[guid] = 1
		self.dr = nil
		
		self:Hide()
		return
	end
	
	if self.dr ~= E.StunDRs[guid]  then
		if E.StunDRs[guid] == 0.25 then
			self.time:SetText('25')
			self:SetStatusBarColor(0.9,0.4,0.2,1)
		elseif E.StunDRs[guid] == 0.5 then
			self.time:SetText('50')
			self:SetStatusBarColor(0.9,0.9,0.2,1)
		elseif E.StunDRs[guid] == 1 then
			self.time:SetText('100')
			self:SetStatusBarColor(0.9,0.9,0.9,1)
		elseif E.StunDRs[guid] < 0.25 then
			self.time:SetText('imm')
			self:SetStatusBarColor(0.9,0.2,0.2,1)
		end
	end
	
	self.dr = E.StunDRs[guid]
	
	self:SetValue(E.StunTimer[guid]-GetTime())
end

function NP.UpdateDRs(self, text)
	local guid = self.guid
	
	if not E.StunTimer[guid] or E.StunTimer[guid] < GetTime() then
		E.StunTimer[guid] = nil
		E.StunDRs[guid] = 1
		
		self:Hide()
		return
	end
	
	self.drBar:Show()
	self.drBar.name:SetText(text)
	self.drBar.time:SetText('100')
end

function NP:CreateNamePlateFrame(frame)

	local plate = CreateFrame('StatusBar', nil, WorldFrame)

--	plate:SetPoint('BOTTOMLEFT', frame, 'BOTTOMLEFT', 0, 0)
--	plate:SetPoint('BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	plate:SetPoint('BOTTOM', frame, 'BOTTOM', 0, plateFrameOffset)

	plate.owner = frame
	
	plate:SetAlpha(1)
	plate:SetStatusBarTexture(E.media.default_bar_texture1)
	plate:SetSize(100, 8)
	plate:SetStatusBarColor(0.7, 0, 0, 1)
	plate:SetMinMaxValues(0, 100)
	plate:SetValue(100)
	plate:SetFrameStrata('LOW')
	plate:SetFrameLevel(2)
	plate:EnableMouse(false)
	plate:GetStatusBarTexture():SetDrawLayer('ARTWORK', 0)
	plate:SetScript('OnEvent', NP.EventHandler)
	
	plate.sizer = CreateFrame('Frame')
	plate.sizer.plate = plate
	plate.sizer:Hide()
	plate.sizer:SetScript('OnUpdate', function(self, elapsed)
		if self.dir == 1 then
			if self.plate:GetWidth() < self.to then
				if self.plate:GetWidth()+self.step > self.to then
					self.plate:SetWidth(self.to)
				else
					self.plate:SetWidth(self.plate:GetWidth()+self.step)
				end
				return
			end
		else
			if self.plate:GetWidth() > self.to then
				if self.plate:GetWidth()-self.step < self.to then
					self.plate:SetWidth(self.to)
				else
					self.plate:SetWidth(self.plate:GetWidth()-self.step)
				end
				return
			end
		end	
		self:Hide()
	end)
	
	--[==[
	plate.backArt = plate:CreateTexture()
	plate.backArt:SetAllPoints(frame)
	plate.backArt:SetColorTexture(1, 0, 0, 0.4)
	plate.backArt:SetDrawLayer('OVERLAY')
	]==]
	
	plate.glowIndicator = CreateFrame("Frame", nil, plate)
	plate.glowIndicator:SetFrameStrata("LOW")
	plate.glowIndicator:SetFrameLevel(1)
	plate.glowIndicator:SetBackdrop( {	
 		edgeFile = "Interface\\AddOns\\AleaUI\\media\\glow", edgeSize = 3,
 		insets = {left = 5, right = 5, top = 5, bottom = 5},
 	})		
	plate.glowIndicator:SetBackdropBorderColor(0, 1, 0, 1)
	plate.glowIndicator:SetScale(1.5)
	plate.glowIndicator:SetPoint("TOPLEFT", plate, "TOPLEFT", -3, 3)
	plate.glowIndicator:SetPoint("BOTTOMRIGHT", plate, "BOTTOMRIGHT", 3, -3)
	plate.glowIndicator:SetAlpha(0.8)
	plate.glowIndicator:Hide()
	
	plate.overlay = plate:CreateTexture()
	plate.overlay:SetAllPoints()
	plate.overlay:SetColorTexture(1, 1, 1, 0.4)
	plate.overlay:SetDrawLayer('ARTWORK', 1)
	plate.overlay:Hide()
	
	plate.levelText = plate:CreateFontString()
	plate.levelText:SetFont(E.media.default_font2, 10, 'OUTLINE')
	plate.levelText:SetText('100')
	plate.levelText:SetWidth(30)
	plate.levelText:SetJustifyH('RIGHT')
	plate.levelText:SetPoint('BOTTOMRIGHT', plate, 'BOTTOMRIGHT', 0, 10)
	plate.levelText:SetDrawLayer('ARTWORK', 2)
	plate.levelText:Show()
	
	plate.nameText = plate:CreateFontString()
	plate.nameText:SetFont(E.media.default_font2, 10, 'OUTLINE')
	plate.nameText:SetText('NameText')
	plate.nameText:SetPoint('BOTTOMLEFT', plate, 'BOTTOMLEFT', 0, 10)
	plate.nameText:SetPoint('RIGHT', plate.levelText, 'LEFT', -1, 0)
	plate.nameText:SetDrawLayer('ARTWORK', 2)
	plate.nameText:SetJustifyH('LEFT')
	plate.nameText:SetWordWrap(false)
	plate.nameText:Show()

	plate.healthText = plate:CreateFontString()
	plate.healthText:SetFont(E.media.default_font2, 10, 'OUTLINE')
	plate.healthText:SetText('100%')
	plate.healthText:SetPoint('CENTER', plate, 'CENTER', 0, 0)
	plate.healthText:SetAllPoints(plate)
	plate.healthText:SetDrawLayer('ARTWORK', 2)
	plate.healthText:SetJustifyH('CENTER')
	plate.healthText:SetWordWrap(false)
	plate.healthText:Show()
	
	plate.DebuffFrame = CreateFrame('Frame', nil, plate)
	plate.DebuffFrame:SetSize(20, 20)
	plate.DebuffFrame:SetFrameStrata('LOW')
	plate.DebuffFrame:SetPoint("BOTTOMLEFT", plate, 'TOPLEFT', 0, 8)
	plate.DebuffFrame:SetPoint("BOTTOMRIGHT", plate, 'TOPRIGHT', 0, 8)
	plate.DebuffFrame.icons = {}
	for i=1, 6 do
		local iconf = CreateFrame("Frame", nil, plate.DebuffFrame)
		iconf:SetSize(14, 14)
		iconf:SetFrameStrata('LOW')
		if i==1 then
			iconf:SetPoint('BOTTOMLEFT', plate.DebuffFrame, 'BOTTOMLEFT', 0, 0)
		else
			iconf:SetPoint('BOTTOMLEFT', plate.DebuffFrame.icons[i-1], 'BOTTOMRIGHT', 3, 0)
		end
		
		iconf.icon = iconf:CreateTexture(nil, 'ARTWORK')
		iconf.icon:SetAllPoints()
		iconf.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
		
	--	iconf.iconbg = iconf:CreateTexture(nil, 'ARTWORK', nil, -1)
	--	iconf.iconbg:SetAllPoints()
	--	iconf.iconbg:SetColorTexture(0,0,0,1)
		
		iconf.timer = iconf:CreateFontString()
		iconf.timer:SetFont(E.media.default_font2, 10, 'OUTLINE')
		iconf.timer:SetText('10')
		iconf.timer:SetPoint('BOTTOMLEFT', iconf, 'TOPLEFT', 0, -4)
		iconf.timer:SetDrawLayer('ARTWORK', 0)
		iconf.timer:SetJustifyH('LEFT')
		iconf.timer:Show()
		
		iconf.stack = iconf:CreateFontString()
		iconf.stack:SetFont(E.media.default_font2, 10, 'OUTLINE')
		iconf.stack:SetText('10')
		iconf.stack:SetPoint('BOTTOMRIGHT', iconf, 'BOTTOMRIGHT', 0, 0)
		iconf.stack:SetDrawLayer('ARTWORK', 0)
		iconf.stack:SetJustifyH('RIGHT')
		iconf.stack:Show()
		
		iconf:Hide()
		NP:CreateBackdrop(iconf)
		
		plate.DebuffFrame.icons[i] = iconf	
	end
	
	plate.BuffFrame = CreateFrame('Frame', nil, plate)
	plate.BuffFrame:SetSize(20, 20)
	plate.BuffFrame:SetFrameStrata('LOW')
	plate.BuffFrame:SetPoint("BOTTOMLEFT", plate.DebuffFrame, 'TOPLEFT', 0, 1)
	plate.BuffFrame:SetPoint("BOTTOMRIGHT", plate.DebuffFrame, 'TOPRIGHT', 0, 1)
	plate.BuffFrame.icons = {}
	for i=1, 6 do	
		local iconf = CreateFrame("Frame", nil, plate.BuffFrame)
		iconf:SetSize(14, 14)
		iconf:SetFrameStrata('LOW')
		if i==1 then
			iconf:SetPoint('BOTTOMRIGHT', plate.BuffFrame, 'BOTTOMRIGHT', 0, 0)
		else
			iconf:SetPoint('BOTTOMRIGHT', plate.BuffFrame.icons[i-1], 'BOTTOMLEFT', -3, 0)
		end
		
		iconf.icon = iconf:CreateTexture(nil, 'ARTWORK')
		iconf.icon:SetAllPoints()
		iconf.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
		
	--	iconf.iconbg = iconf:CreateTexture(nil, 'ARTWORK', nil, -1)
	--	iconf.iconbg:SetAllPoints()
	--	iconf.iconbg:SetColorTexture(0,0,0,1)
		
		iconf.timer = iconf:CreateFontString()
		iconf.timer:SetFont(E.media.default_font2, 10, 'OUTLINE')
		iconf.timer:SetText('10')
		iconf.timer:SetPoint('BOTTOMLEFT', iconf, 'TOPLEFT', 0, -4)
		iconf.timer:SetDrawLayer('ARTWORK', 0)
		iconf.timer:SetJustifyH('LEFT')
		iconf.timer:Show()
		
		iconf.stack = iconf:CreateFontString()
		iconf.stack:SetFont(E.media.default_font2, 10, 'OUTLINE')
		iconf.stack:SetText('10')
		iconf.stack:SetPoint('BOTTOMRIGHT', iconf, 'BOTTOMRIGHT', 0, 0)
		iconf.stack:SetDrawLayer('ARTWORK', 0)
		iconf.stack:SetJustifyH('RIGHT')
		iconf.stack:Show()
		
		iconf:Hide()
		
		NP:CreateBackdrop(iconf)
		
		plate.BuffFrame.icons[i] = iconf	
	end
	
	-- Dragon Border 
	plate.eliteIcon = plate:CreateTexture(nil, 'ARTWORK', nil, 3)
	plate.eliteIcon:SetSize(42.602336883545*1.3, 31.154767990112*1.3)
	plate.eliteIcon:SetTexture([[Interface\Tooltips\EliteNameplateIcon]])
	plate.eliteIcon:Hide()
	plate.eliteIcon:SetPoint("RIGHT", plate, "RIGHT", 37, -4)	
	
	-- Raid mark
	plate.raidIcon = plate:CreateTexture(nil, 'ARTWORK')
	plate.raidIcon:SetSize(26, 26)
	plate.raidIcon:SetTexture([[Interface\TargetingFrame\UI-RaidTargetingIcons]])
	plate.raidIcon:Hide()
	plate.raidIcon:SetPoint(E.db.nameplates.raidIcon_attachFrom, plate, E.db.nameplates.raidIcon_attachTo, E.db.nameplates.raidIcon_xOffset, E.db.nameplates.raidIcon_yOffset)
	
	--CastBar
	plate.castBar = CreateFrame("StatusBar", nil, plate)
	plate.castBar.plate = plate
	plate.castBar:SetPoint('TOPLEFT', plate, 'BOTTOMLEFT', 0, -5)	
	plate.castBar:SetPoint('TOPRIGHT', plate, 'BOTTOMRIGHT', 0, -5)	
	plate.castBar:SetFrameStrata("BACKGROUND")
	plate.castBar:SetFrameLevel(2)
	plate.castBar:SetScript('Onupdate', NamePlate_CastBarOnUpdate)
	plate.castBar:Hide()
	NP:CreateBackdrop(plate.castBar)

	plate.castBar.time = plate.castBar:CreateFontString(nil, 'OVERLAY')
	plate.castBar.time:SetPoint("TOPRIGHT", plate.castBar, "BOTTOMRIGHT", 6, -2)
	plate.castBar.time:SetJustifyH("RIGHT")
	plate.castBar.time:SetWordWrap(false)

	plate.castBar.name = plate.castBar:CreateFontString(nil, 'OVERLAY')
	plate.castBar.name:SetPoint("TOPLEFT", plate.castBar, "BOTTOMLEFT", 0, -2)
	plate.castBar.name:SetPoint("TOPRIGHT", plate.castBar.time, "TOPLEFT", 0, -2)
	plate.castBar.name:SetJustifyH("LEFT")
	plate.castBar.name:SetWordWrap(false)

	plate.castBar.icon = plate.castBar:CreateTexture(nil, 'OVERLAY')
	plate.castBar.icon:SetTexCoord(.07, .93, .07, .93)
	plate.castBar.icon:SetDrawLayer("OVERLAY")
	plate.castBar.icon:SetPoint("TOPLEFT", plate, "TOPRIGHT", 5, 0)
	NP:CreateBackdrop(plate.castBar, plate.castBar.icon)
	
	-- Opts
	plate.castBar:SetSize(100, 6)
	plate.castBar:SetStatusBarTexture(E.media.default_bar_texture1)
	plate.castBar.time:SetFont(E.media.default_font2, 10, 'OUTLINE')
	plate.castBar.name:SetFont(E.media.default_font2, 10, 'OUTLINE')
	plate.castBar.icon:SetSize(6 + 10 + 5, 6 + 10 + 5)	
	
	
	plate.drBar = CreateFrame("StatusBar", nil, plate)
	plate.drBar.plate = plate
	plate.drBar:SetPoint('BOTTOMLEFT', plate, 'TOPLEFT', 16, 2)	
	plate.drBar:SetPoint('BOTTOMRIGHT', plate, 'TOPRIGHT', 0, 2)	
	plate.drBar:SetFrameStrata("BACKGROUND")
	plate.drBar:SetFrameLevel(2)
	plate.drBar:SetMinMaxValues(0, 18)
	plate.drBar:SetScript('Onupdate', NamePlate_DRBarOnUpdate)
	plate.drBar:Hide()
--	NP:CreateBackdrop(plate.drBar)
	
	plate.drBar.bg = plate.drBar:CreateTexture(nil, 'BACKGROUND', nil, -1)
	plate.drBar.bg:SetAllPoints()
	plate.drBar.bg:SetColorTexture(0,0,0,0.6)
	
	plate.drBar.time = plate.drBar:CreateFontString(nil, 'OVERLAY')
	plate.drBar.time:SetPoint("BOTTOMLEFT", plate.drBar, "BOTTOMRIGHT", 2, 0)
	plate.drBar.time:SetJustifyH("RIGHT")
	plate.drBar.time:SetWordWrap(false)
	plate.drBar.time:Hide()

	plate.drBar.name = plate.drBar:CreateFontString(nil, 'OVERLAY')
--	plate.drBar.name:SetPoint("BOTTOMLEFT", plate.drBar, "TOPLEFT", 0, 0)
--	plate.drBar.name:SetPoint("BOTTOMRIGHT", plate.drBar.time, "BOTTOMLEFT", 0, 0)
	plate.drBar.name:SetPoint("BOTTOMRIGHT", plate.drBar, "BOTTOMLEFT", -2, -2)
	plate.drBar.name:SetJustifyH("RIGHT")
	plate.drBar.name:SetWordWrap(false)
	
	plate.drBar:SetSize(100, 3)
	plate.drBar:SetStatusBarTexture(E.media.default_bar_texture1)
	plate.drBar:SetStatusBarColor(0.9,0.9,0.9,1)
	plate.drBar.time:SetFont("Fonts\\ARIALN.TTF", 8, 'NONE')
	plate.drBar.name:SetFont("Fonts\\ARIALN.TTF", 8, 'NONE')
	
	--[==[
	plate.drBar.icon = plate.drBar:CreateTexture(nil, 'OVERLAY')
	plate.drBar.icon:SetTexCoord(.07, .93, .07, .93)
	plate.drBar.icon:SetDrawLayer("OVERLAY")
	plate.drBar.icon:SetPoint("TOPLEFT", plate, "TOPRIGHT", 5, 0)
	NP:CreateBackdrop(plate.drBar, plate.drBar.icon)	
	]==]
	
	
	
	plate.UpdateFaction = NP.UpdateFaction
	plate.UpdateName = NP.UpdateName
	plate.OnShowUpdate = NP.OnShowUpdate
	plate.UpdateTargetAlpha = NP.UpdateTargetAlpha
	plate.UpdateFrameLevel = NP.UpdateFrameLevel
	plate.UpdateHealth = NP.UpdateHealth
	plate.UpdateAuras = NP.UpdateAuras
	plate.UpdateCast = NP.UpdateCast
	plate.StopCast = NP.StopCast
	plate.UpdateRaidIcon = NP.UpdateRaidIcon
	plate.SetTargetWidth = NP.SetTargetWidth
	plate.SetNormalWidth = NP.SetNormalWidth
	plate.UpdateAggro = NP.UpdateAggro
	plate.UpdateDRs = NP.UpdateDRs
	
	NP:CreateBackdrop(plate)
	
	NP.CreatedPlates[#NP.CreatedPlates+1] = plate
	
	frame.AleaNP = plate
	
	NP.UpdateSettings(plate)
end

function NP.UpdateSettings(plate)
	
	local font = E:GetFont(E.db.nameplates.font)
	local fontSize = E.db.nameplates.fontSize --* UIParent:GetScale()/WorldFrame:GetScale()
	
	local horizontalScale = tonumber(GetCVar("NamePlateHorizontalScale"))
	
	plate.defaultWidth = E.db.nameplates.healthBar_width*0.8
	plate.defaultHeight = E.db.nameplates.healthBar_height
	plate.defaultIncrease = E.db.nameplates.healthBar_width - plate.defaultWidth
	
	plate:SetSize(plate.defaultWidth, plate.defaultHeight)
	
	plate:SetStatusBarTexture(E:GetTexture(E.db.nameplates.healthBar_texture))
	plate:SetBorderColor(E.db.nameplates.backdropfadecolor[1], E.db.nameplates.backdropfadecolor[2],E.db.nameplates.backdropfadecolor[3],E.db.nameplates.backdropfadecolor[4])
	
	plate.nameText:ClearAllPoints()
	plate.nameText:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	plate.nameText:SetPoint("BOTTOMLEFT", plate, "BOTTOMLEFT", 1+E.db.nameplates.healthBar_name_xOffset, E.db.nameplates.healthBar_name_yOffset)
	plate.nameText:SetPoint("BOTTOMRIGHT", plate, "BOTTOMRIGHT", -15+E.db.nameplates.healthBar_name_xOffset, E.db.nameplates.healthBar_name_yOffset)
	
	plate.levelText:ClearAllPoints()
	plate.levelText:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	plate.levelText:SetPoint('BOTTOMRIGHT', plate, 'BOTTOMRIGHT', E.db.nameplates.healthBar_lvl_xOffset, E.db.nameplates.healthBar_lvl_yOffset)
	
	plate.healthText:ClearAllPoints()
	plate.healthText:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	plate.healthText:SetPoint("BOTTOM", plate, "BOTTOM", E.db.nameplates.healthBar_hp_xOffset, E.db.nameplates.healthBar_hp_yOffset)
	
	plate.castBar:SetSize(E.db.nameplates.healthBar_width, E.db.nameplates.castBar_height)
	plate.castBar:SetStatusBarTexture(E:GetTexture(E.db.nameplates.healthBar_texture))
	plate.castBar.time:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	plate.castBar.name:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	plate.castBar.icon:SetSize(E.db.nameplates.castBar_height + E.db.nameplates.healthBar_height + 5, E.db.nameplates.castBar_height + E.db.nameplates.healthBar_height + 5)	

	plate.raidIcon:SetPoint(E.db.nameplates.raidIcon_attachFrom, plate, E.db.nameplates.raidIcon_attachTo, E.db.nameplates.raidIcon_xOffset, E.db.nameplates.raidIcon_yOffset)

	for i=1, 6 do
		plate.DebuffFrame.icons[i]:SetSize(E.db.nameplates.auraSize, E.db.nameplates.auraSize)
		plate.DebuffFrame.icons[i].timer:SetFont(font, fontSize, E.db.nameplates.fontOutline)
		plate.DebuffFrame.icons[i].stack:SetFont(font, fontSize, E.db.nameplates.fontOutline)
		
		plate.BuffFrame.icons[i]:SetSize(E.db.nameplates.auraSize, E.db.nameplates.auraSize )
		plate.BuffFrame.icons[i].timer:SetFont(font, fontSize, E.db.nameplates.fontOutline)
		plate.BuffFrame.icons[i].stack:SetFont(font, fontSize, E.db.nameplates.fontOutline)
	end
end

local function SetBorderColor(self, r, g, b,a)	
	self.bordertop:SetColorTexture(r, g, b,a)
	self.borderbottom:SetColorTexture(r, g, b,a)
	self.borderleft:SetColorTexture(r, g, b,a)
	self.borderright:SetColorTexture(r, g, b,a)
end

function NP:CreateBackdrop(parent, point)
	point = point or parent
	local noscalemult = 1 * UIParent:GetScale()
	
--	print(noscalemult)
--	print(1 * UIParent:GetScale()/WorldFrame:GetScale())
	
	if point.bordertop then return end

	point.backdrop = parent:CreateTexture(nil, "BORDER")
	point.backdrop:SetDrawLayer("BORDER", -4)
	point.backdrop:SetAllPoints(point)
	point.backdrop:SetColorTexture(unpack(E.db.nameplates.backdropfadecolor))		

	point.bordertop = parent:CreateTexture(nil, "BORDER")
	point.bordertop:SetPoint("TOPLEFT", point, "TOPLEFT", -noscalemult, noscalemult)
	point.bordertop:SetPoint("TOPRIGHT", point, "TOPRIGHT", noscalemult, noscalemult)
	point.bordertop:SetHeight(noscalemult)
	point.bordertop:SetColorTexture(unpack(E.db.nameplates.bordercolor))	
	point.bordertop:SetDrawLayer("BORDER", 1)
		
	point.borderbottom = parent:CreateTexture(nil, "BORDER")
	point.borderbottom:SetPoint("BOTTOMLEFT", point, "BOTTOMLEFT", -noscalemult, -noscalemult)
	point.borderbottom:SetPoint("BOTTOMRIGHT", point, "BOTTOMRIGHT", noscalemult, -noscalemult)
	point.borderbottom:SetHeight(noscalemult)
	point.borderbottom:SetColorTexture(unpack(E.db.nameplates.bordercolor))	
	point.borderbottom:SetDrawLayer("BORDER", 1)
		
	point.borderleft = parent:CreateTexture(nil, "BORDER")
	point.borderleft:SetPoint("TOPLEFT", point, "TOPLEFT", -noscalemult, noscalemult)
	point.borderleft:SetPoint("BOTTOMLEFT", point, "BOTTOMLEFT", noscalemult, -noscalemult)
	point.borderleft:SetWidth(noscalemult)
	point.borderleft:SetColorTexture(unpack(E.db.nameplates.bordercolor))	
	point.borderleft:SetDrawLayer("BORDER", 1)
		
	point.borderright = parent:CreateTexture(nil, "BORDER")
	point.borderright:SetPoint("TOPRIGHT", point, "TOPRIGHT", noscalemult, noscalemult)
	point.borderright:SetPoint("BOTTOMRIGHT", point, "BOTTOMRIGHT", -noscalemult, -noscalemult)
	point.borderright:SetWidth(noscalemult)
	point.borderright:SetColorTexture(unpack(E.db.nameplates.bordercolor))
	point.borderright:SetDrawLayer("BORDER", 1)	
	
	point.SetBorderColor = SetBorderColor
end

function NP:UpdateAllPlates()
	NP:ForEachPlate("UpdateSettings")
end

function NP:ForEachPlate(functionToRun, ...)
	for i=1, #NP.CreatedPlates do
		NP[functionToRun](NP.CreatedPlates[i], ...)
	end
end


function NP:UpdateBasePlateSize()
	local horizontalScale = tonumber(GetCVar("NamePlateHorizontalScale"))
	
--	C_NamePlate.SetNamePlateOtherSize(E.db.nameplates.healthBar_width * horizontalScale, E.db.nameplates.healthBar_height + 20)
	
--	C_NamePlate.SetNamePlateSelfSize(E.db.nameplates.healthBar_width-20, 40);
	
	
--	["player"] = C_NamePlate.SetNamePlateSelfPreferredClickInsets,
--	["friendly"] =  C_NamePlate.SetNamePlateFriendlyPreferredClickInsets,
--	["enemy"] = C_NamePlate.SetNamePlateEnemyPreferredClickInsets,
	
--	C_NamePlate.SetNamePlateSelfPreferredClickInsets(0,0,0,0)
--	C_NamePlate.SetNamePlateEnemyPreferredClickInsets(0,0,-20,20)
--	C_NamePlate.SetNamePlateFriendlyPreferredClickInsets(0,0,-20,20)
	
	local plateHeight = E.db.nameplates.healthBar_height + plateFrameWidthOffset
	
	C_NamePlate.SetNamePlateFriendlySize(E.db.nameplates.healthBar_width * horizontalScale, plateHeight);
	C_NamePlate.SetNamePlateEnemySize(E.db.nameplates.healthBar_width * horizontalScale, plateHeight);
	C_NamePlate.SetNamePlateSelfSize(E.db.nameplates.healthBar_width-20, 40);
	--[==[
	C_Timer.After(0.5, function()
		E:LockCVar('nameplateOverlapV', 1.4)
		E:LockCVar('nameplateOverlapH', 1.2)
	end)
	]==]
end

function NP:CVarUpdate()
--	E:LockCVar("nameplateShowSelf", E.db.nameplates.nameplateShowSelf and 0 or 1)
--	E:LockCVar("nameplateMotion", E.db.nameplates.nameplateMotion == "STACKED" and 1 or 0)
	SetCVar("nameplateShowAll", E.db.nameplates.nameplateShowAll and '1' or '0')
	
	SetCVar("nameplateMaxDistance", E.db.nameplates.nameplateMaxDistance)

	SetCVar("nameplateOtherTopInset", E.db.nameplates.nameplateOffsets  and '.08' or '-1') 
	SetCVar("nameplateOtherBottomInset",  E.db.nameplates.nameplateOffsets  and '.1' or '-1')
	
	SetCVar("nameplateLargeBottomInset", '.08')
	SetCVar("nameplateLargeTopInset", '.08')
		
--	E:LockCVar("nameplateShowFriendlyMinions", E.db.nameplates.nameplateShowFriendlyMinions and 1 or 0)
--	E:LockCVar("nameplateShowEnemyMinions", E.db.nameplates.nameplateShowEnemyMinions and 1 or 0)
--	E:LockCVar("nameplateShowEnemyMinus", E.db.nameplates.nameplateShowEnemyMinus and 1 or 0)
end

function NP.OnInit()
	NP:UpdateBasePlateSize()
end

function NP:UpdateProfileSettings()
	NP:UpdateAllPlates()	
	NP:UpdateBasePlateSize()
	NP:CVarUpdate()
end

E:OnInit2(NP.OnInit)
E:OnInit(function()
	NP:CVarUpdate()
end)
-----------------------
-- Interface
-----------------------
E.GUI.args.NamePlates = {
	name = "Панели здоровья",
	type = "group",
	order = 5,
	args = {},
}

E.GUI.args.NamePlates.args.blizzardSettings = {
	name = "Blizzard Settings",
	type = "group",
	order = 1,
	embend = true,
	args = {},	
}


E.GUI.args.NamePlates.args.generals = {
	name = "Основное",
	type = "group",
	order = 1.2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateMaxDistance = {
	name = "Дальность отображения",
	order = 1,
	type = "slider",
	min = 30, max = 60, step = 1,
	set = function(self, value)
		E.db.nameplates.nameplateMaxDistance = value
		SetCVar("nameplateMaxDistance", value)
	end,
	get = function(self)
		return E.db.nameplates.nameplateMaxDistance
	end,
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateShowAll = {
	name = "Показывать все панели",
	order = 1.1,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.nameplateShowAll = not E.db.nameplates.nameplateShowAll
		SetCVar("nameplateShowAll", E.db.nameplates.nameplateShowAll and 1 or 0)
	end,
	get = function(self)
		return E.db.nameplates.nameplateShowAll
	end,
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateOffsets = {
	name = "Включить залипание",
	order = 1.2,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.nameplateOffsets = not E.db.nameplates.nameplateOffsets
		
		SetCVar("nameplateOtherTopInset", E.db.nameplates.nameplateOffsets  and '.08' or '-1') 
		SetCVar("nameplateOtherBottomInset",  E.db.nameplates.nameplateOffsets  and '.1' or '-1')

	end,
	get = function(self)
		return E.db.nameplates.nameplateOffsets
	end,
}

--[==[
E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateShowFriendlyMinions = {
	name = "Дружественные питомцы",
	order = 1.2,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.nameplateShowFriendlyMinions = not E.db.nameplates.nameplateShowFriendlyMinions
		E:LockCVar("nameplateShowFriendlyMinions", E.db.nameplates.nameplateShowFriendlyMinions and 1 or 0)
	end,
	get = function(self)
		return E.db.nameplates.nameplateShowFriendlyMinions
	end,
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateShowEnemyMinions = {
	name = "Вражеские питомцы",
	order = 1.2,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.nameplateShowEnemyMinions = not E.db.nameplates.nameplateShowEnemyMinions
		E:LockCVar("nameplateShowEnemyMinions", E.db.nameplates.nameplateShowEnemyMinions and 1 or 0)
	end,
	get = function(self)
		return E.db.nameplates.nameplateShowEnemyMinions
	end,
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateShowEnemyMinus = {
	name = "Вражеские слабые мобы",
	order = 1.3,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.nameplateShowEnemyMinus = not E.db.nameplates.nameplateShowEnemyMinus
		E:LockCVar("nameplateShowEnemyMinus", E.db.nameplates.nameplateShowEnemyMinus and 1 or 0)
	end,
	get = function(self)
		return E.db.nameplates.nameplateShowEnemyMinus
	end,
}

E.GUI.args.NamePlates.args.blizzardSettings.args.nameplateMotion = {	
	name = "Расположение панелей",
	order = 1.4,
	type = "dropdown",
	values = {
		['STACKED'] = UNIT_NAMEPLATES_TYPE_2,
		['OVERLAP'] = UNIT_NAMEPLATES_TYPE_1,
	},
	set = function(self, value)
		E.db.nameplates.nameplateMotion = value
		E:LockCVar("nameplateMotion", E.db.nameplates.nameplateMotion == "STACKED" and 1 or 0)
	end,
	get = function(self)
		return E.db.nameplates.nameplateMotion
	end,
}
]==]
E.GUI.args.NamePlates.args.generals.args.buff_amount = {
	name = "Кол-во баффов",
	order = 3,
	type = "slider",
	min = 1, max = 10, step = 1,
	set = function(self, value)
		E.db.nameplates.buffs.numAuras = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.buffs.numAuras
	end,
}

E.GUI.args.NamePlates.args.generals.args.debuff_amount = {
	name = "Кол-во дебаффов",
	order = 4,
	type = "slider",
	min = 1, max = 10, step = 1,
	set = function(self, value)
		E.db.nameplates.debuffs.numAuras = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.debuffs.numAuras
	end,
}

E.GUI.args.NamePlates.args.generals.args.showmyauras = {
	name = "Показать свои ауры",
	order = 1,
	type = "toggle",

	set = function(self, value)
		E.db.nameplates.showmyauras = not E.db.nameplates.showmyauras
	end,
	get = function(self)
		return E.db.nameplates.showmyauras
	end,
}

E.GUI.args.NamePlates.args.generals.args.showfromspelllist = {
	name = "Показать ауры из списка",
	order = 2,
	type = "toggle",
	set = function(self, value)
		E.db.nameplates.showfromspelllist = not E.db.nameplates.showfromspelllist
	end,
	get = function(self)
		return E.db.nameplates.showfromspelllist
	end,
}

E.GUI.args.NamePlates.args.generals.args.showpurge = {
	name = "Показать ауры для рассеивания",
	order = 2,
	type = "toggle",
	set = function(self, value)
		E.db.nameplates.showpurge = not E.db.nameplates.showpurge
	end,
	get = function(self)
		return E.db.nameplates.showpurge
	end,
}

E.GUI.args.NamePlates.args.plateColors = {
	name = "Reaction Colors",
	type = "group",
	order = 1.3,
	embend = true,
	args = {},	
}

for k,v in pairs(defaults.reactions) do

	E.GUI.args.NamePlates.args.plateColors.args[k] = {
		name = k, desc = k,
		order = 1,
		type = "color",
		set = function(self, r,g,b)
			E.db.nameplates.reactions[k] = { r=r, g=g, b=b, 1 }
		end,
		get = function(self)
			return E.db.nameplates.reactions[k].r, E.db.nameplates.reactions[k].g, E.db.nameplates.reactions[k].b, 1
		end,
	}
end

E.GUI.args.NamePlates.args.auratypeColors = {
	name = "Aura type Colors",
	type = "group",
	order = 1.5,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.auratypeColors.args.colorbytype = {
	name = "Color by type",
	order = 1,
	width = "full",
	type = "toggle",
	set = function(self, value)
		E.db.nameplates.colorByType = not E.db.nameplates.colorByType
	end,
	get = function(self)
		return E.db.nameplates.colorByType
	end,		
}

for k,v in pairs(defaults.typecolors) do

	E.GUI.args.NamePlates.args.auratypeColors.args[k] = {
		name = colorToTypeName[k], desc = k,
		order = 2,
		type = "color",
		set = function(self, r,g,b)
			E.db.nameplates.typecolors[k] = {r,g,b,1}
		end,
		get = function(self)
			return E.db.nameplates.typecolors[k][1], E.db.nameplates.typecolors[k][2], E.db.nameplates.typecolors[k][3], 1
		end,
	}
end


E.GUI.args.NamePlates.args.plateHealth = {
	name = "Healths",
	type = "group",
	order = 2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.plateHealth.args.border = {
	name = "Border", desc = "Border",
	order = 1,
	type = "color",
	set = function(self, r,g,b)
		E.db.nameplates.bordercolor = { r, g, b, 1 }
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.bordercolor[1], E.db.nameplates.bordercolor[2], E.db.nameplates.bordercolor[3], 1
	end,
}

E.GUI.args.NamePlates.args.plateHealth.args.backgroundcolor = {
	name = "Background", desc = "Background",
	order = 2,
	type = "color",
	hasAlpha = true,		
	set = function(self, r,g,b,a)
		E.db.nameplates.backdropfadecolor = { r, g, b, a }
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.backdropfadecolor[1], E.db.nameplates.backdropfadecolor[2], E.db.nameplates.backdropfadecolor[3], E.db.nameplates.backdropfadecolor[4]
	end,
}

E.GUI.args.NamePlates.args.plateHealth.args.width = {	
	name = "Width",
	order = 3,
	type = "slider",
	min = 1, max = 300, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_width = value		
		
		NP:UpdateBasePlateSize()

		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.healthBar_width
	end,
}
E.GUI.args.NamePlates.args.plateHealth.args.height = {	
	name = "Height",
	order = 4,
	type = "slider",
	min = 1, max = 50, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_height = value
		
		NP:UpdateBasePlateSize()
		
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.healthBar_height
	end,
}

E.GUI.args.NamePlates.args.plateHealth.args.texture = {	
	name = "Texture",
	order = 5,
	type = "statusbar",
	values = E.GetTextureList,
	set = function(self, value)
		E.db.nameplates.healthBar_texture = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.healthBar_texture
	end,
}

E.GUI.args.NamePlates.args.plateHealth.args.auraSize = {	
	name = "Aura Size",
	order = 6,
	type = "slider",
	min = 1, max = 32, step = 1,
	set = function(self, value)
		E.db.nameplates.auraSize = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.auraSize
	end,
}
--[==[
E.GUI.args.NamePlates.args.AuraParen = {
	name = "Крепление аур",
	type = "group",
	order = 2.2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.AuraParen.args.xOffset = {	
	name = "xOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.aura_xOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.aura_xOffset
	end,
}
E.GUI.args.NamePlates.args.AuraParen.args.yOffset = {	
	name = "yOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.aura_yOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.aura_yOffset
	end,
}
]==]
E.GUI.args.NamePlates.args.plateCastBar = {
	name = "CastBar",
	type = "group",
	order = 3,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.plateCastBar.args.height = {	
	name = "Height",
	order = 1,
	type = "slider",
	min = 1, max = 50, step = 1,
	set = function(self, value)
		E.db.nameplates.castBar_height = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.castBar_height
	end,
}

E.GUI.args.NamePlates.args.plateCastBar.args.color = {
	name = "Normal", desc = "Normal",
	order = 2,
	type = "color",
	set = function(self, r,g,b)
		E.db.nameplates.castBar_color = { r, g, b, 1 }
	end,
	get = function(self)
		return E.db.nameplates.castBar_color[1], E.db.nameplates.castBar_color[2], E.db.nameplates.castBar_color[3], 1
	end,
}

E.GUI.args.NamePlates.args.plateCastBar.args.noInterrupt = {
	name = "Not Interruptable", desc = "Not Interruptable",
	order = 3,
	type = "color",
	set = function(self, r,g,b)
		E.db.nameplates.castBar_noInterrupt = { r, g, b, 1 }
	end,
	get = function(self)
		return E.db.nameplates.castBar_noInterrupt[1], E.db.nameplates.castBar_noInterrupt[2], E.db.nameplates.castBar_noInterrupt[3], 1
	end,
}


E.GUI.args.NamePlates.args.healthBar_nameText = {
	name = "Текст имени",
	type = "group",
	order = 3.1,
	embend = true,
	args = {},	
}
E.GUI.args.NamePlates.args.healthBar_nameText.args.xOffset = {	
	name = "xOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_name_xOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_name_xOffset
	end,
}
E.GUI.args.NamePlates.args.healthBar_nameText.args.yOffset = {	
	name = "yOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_name_yOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_name_yOffset
	end,
}

E.GUI.args.NamePlates.args.healthBar_hpText = {
	name = "Текст здоровья",
	type = "group",
	order = 3.2,
	embend = true,
	args = {},	
}
E.GUI.args.NamePlates.args.healthBar_hpText.args.textFormat = {
	name = 'Формат',
	type = 'dropdown',
	order = 0.1,
	width = 'full',
	values = {
		NO,
		'Проценты',
		'Текущее значение',
		'Оба',
	},
	set = function(info, value)
		E.db.nameplates.healthBar_hp_format = value
	end,
	get = function(info)
		return E.db.nameplates.healthBar_hp_format
	end,
}
E.GUI.args.NamePlates.args.healthBar_hpText.args.xOffset = {	
	name = "xOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_hp_xOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_hp_xOffset
	end,
}
E.GUI.args.NamePlates.args.healthBar_hpText.args.yOffset = {	
	name = "yOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_hp_yOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_hp_yOffset
	end,
}

E.GUI.args.NamePlates.args.healthBar_lvlText = {
	name = "Текст уровня",
	type = "group",
	order = 3.2,
	embend = true,
	args = {},	
}
E.GUI.args.NamePlates.args.healthBar_lvlText.args.xOffset = {	
	name = "xOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_lvl_xOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_lvl_xOffset
	end,
}
E.GUI.args.NamePlates.args.healthBar_lvlText.args.yOffset = {	
	name = "yOffset",
	order = 1,
	type = "slider",
	min = -200, max = 200, step = 1,
	set = function(self, value)
		E.db.nameplates.healthBar_lvl_yOffset = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.healthBar_lvl_yOffset
	end,
}

E.GUI.args.NamePlates.args.plateFont = {
	name = "Fonts",
	type = "group",
	order = 4,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.plateFont.args.font = {	
	name = "Font",
	order = 1,
	type = "font",
	values = E.GetFontList,
	set = function(self, value)
		E.db.nameplates.font = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.font
	end,
}

E.GUI.args.NamePlates.args.plateFont.args.fontSize = {	
	name = "Size",
	order = 1,
	type = "slider",
	min = 1, max = 32, step = 1,
	set = function(self, value)
		E.db.nameplates.fontSize = value
		NP:UpdateAllPlates()	
	end,
	get = function(self)
		return E.db.nameplates.fontSize
	end,
}
E.GUI.args.NamePlates.args.plateFont.args.fontOutline = {	
	name = "Flags",
	order = 3,
	type = "dropdown",
	values = {			
		[""] = NO,
		["OUTLINE"] = "OUTLINE",
	},
	set = function(self, value)
		E.db.nameplates.fontOutline = value
		NP:UpdateAllPlates()
	end,
	get = function(self)
		return E.db.nameplates.fontOutline
	end,
}

--NP:UpdateAllPlates()

E.GUI.args.NamePlates.args.spellList = {
	name = "Spell List",
	type = "group",
	order = 1,
	args = {},	
}

E.GUI.args.NamePlates.args.spellList.args.create = {
	name = "",
	type = "group",
	order = 1,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.spellList.args.settings = {
	name = "",
	type = "group",
	order = 2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.spellList.args.create.args.spellid = {
	name = "SpellID",
	type = "editbox",
	order = 1,
	set = function(self, value)
		local num = tonumber(value)			
		if num and GetSpellInfo(num) then
			if not E.db.nameplates.spelllist[GetSpellInfo(num)] then
			
				E.db.nameplates.spelllist[GetSpellInfo(num)] = {}
				E.db.nameplates.spelllist[GetSpellInfo(num)].show = 1
				E.db.nameplates.spelllist[GetSpellInfo(num)].size = 1
				E.db.nameplates.spelllist[GetSpellInfo(num)].checkID = false
				E.db.nameplates.spelllist[GetSpellInfo(num)].spellID = num
			end
			
			selectedspell = GetSpellInfo(num)
		end
	end,
	get = function(self)
		return ''
	end,

}

E.GUI.args.NamePlates.args.spellList.args.create.args.spelllist = {
	name = "Select spell",
	type = "dropdown",
	order = 2,
	values = function()
		local t = {}
		
		for spellname,params in pairs(E.db.nameplates.spelllist) do		

			t[spellname] = NP:SpellString(params.spellID)
			
			--t[spellname] = (spellname).." "..( params.size or 1 ).." "..( params.show and "SHOW" or "HIDE" ).." "..(params.spellID or "000000").." "..(params.checkID and "CheckID" or "NoCheckID")
		end
		
		return t
	end,
	set = function(self, value)			
		selectedspell = value
	end,
	get = function(self)
		return selectedspell
	end,	
}


E.GUI.args.NamePlates.args.spellList.args.settings.args.show = {
	name = "Show",
	type = "dropdown",
	order = 4,
	values = {		
		"Always",
		"Never",
		"Only mine",
		"Only on enemy",
		"Only on friendly",
	},
	set = function(self, value)
		if selectedspell then
			E.db.nameplates.spelllist[selectedspell].show = value
		end
	end,
	get = function(self)
		if selectedspell then
			return E.db.nameplates.spelllist[selectedspell].show or 1
		else
			return 1
		end
	end,	
}

E.GUI.args.NamePlates.args.spellList.args.settings.args.spellID = {
	name = "SpellID",
	type = "editbox",
	order = 4,
	set = function(self, value)
		local num = tonumber(value)
		
		if selectedspell and num then
			E.db.nameplates.spelllist[selectedspell].spellID = num
		end
	end,
	get = function(self)
		if selectedspell then
			return E.db.nameplates.spelllist[selectedspell].spellID or ''
		else
			return ''
		end
	end,	
}

E.GUI.args.NamePlates.args.spellList.args.settings.args.checkID = {
	name = "CheckID",
	type = "toggle",
	order = 4,
	set = function(self, value)
		if selectedspell then
			E.db.nameplates.spelllist[selectedspell].checkID = not E.db.nameplates.spelllist[selectedspell].checkID
		end
	end,
	get = function(self)
		if selectedspell then
			return E.db.nameplates.spelllist[selectedspell].checkID or false
		else
			return false
		end
	end,	
}

E.GUI.args.NamePlates.args.spellList.args.settings.args.size = {
	name = "Size",
	type = "slider", min = 1, max = 2, step = 0.1,
	order = 4,
	set = function(self, value)
		if selectedspell then
			E.db.nameplates.spelllist[selectedspell].size = value
		end
	end,
	get = function(self)
		if selectedspell then
			return E.db.nameplates.spelllist[selectedspell].size or 1
		else
			return 1
		end
	end,	
}


E.GUI.args.NamePlates.args.nameList = {
	name = "Name List",
	type = "group",
	order = 1,
	args = {},	
}

E.GUI.args.NamePlates.args.nameList.args.create = {
	name = "",
	type = "group",
	order = 1,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.nameList.args.settings = {
	name = "",
	type = "group",
	order = 2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.nameList.args.create.args.spellid = {
	name = "Name",
	type = "editbox",
	order = 1,
	set = function(self, value)
		if value and #value > 0 then
			if not E.db.nameplates.namelist[value] then
				E.db.nameplates.namelist[value] = {
					show = true,
					subname = value
				}
			end			
			selectedname = value
			
			NP:UpdateCustomName()
		end
	end,
	get = function(self)
		return ''
	end,

}


E.GUI.args.NamePlates.args.nameList.args.create.args.namelist = {
	name = "Select name",
	type = "dropdown",
	order = 2,
	values = function()
		local t = {}
		
		for name,params in pairs(E.db.nameplates.namelist) do				
			t[name] = name
		end
		
		NP:UpdateCustomName()
		
		return t
	end,
	set = function(self, value)			
		selectedname = value
	end,
	get = function(self)
		return selectedname
	end,	
}

E.GUI.args.NamePlates.args.nameList.args.create.args.namefromtarget = {
	name = "Name from target",
	type = "execute",
	order = 3,
	set = function(self, value)
		if UnitExists("target") and not UnitIsPlayer("target") then
			local name = UnitName("target")
			E.db.nameplates.namelist[name] = {
				show = true,
				subname = name
			}			
			
			NP:UpdateCustomName()
			
			selectedname = name
		end
	end,
	get = function(self)
		return ''
	end,

}

E.GUI.args.NamePlates.args.nameList.args.settings.args.checkID = {
	name = "Enable",
	type = "toggle",
	order = 4,
	set = function(self, value)
		if selectedname then
			E.db.nameplates.namelist[selectedname].show = not E.db.nameplates.namelist[selectedname].show
			NP:UpdateCustomName()
		end
	end,
	get = function(self)
		if selectedname then
			return E.db.nameplates.namelist[selectedname].show
		else
			return false
		end
	end,	
}

E.GUI.args.NamePlates.args.nameList.args.settings.args.subname = {
	name = "Name",
	type = "editbox",
	width = 'full',
	order = 1,
	set = function(self, value)
		if selectedname then
			if value and #value > 0 then
				E.db.nameplates.namelist[selectedname].subname = value		
				NP:UpdateCustomName()
			end
		end
	end,
	get = function(self)
		if selectedname then
			return E.db.nameplates.namelist[selectedname].subname or ''
		else
			return ''
		end
	end,

}

E.GUI.args.NamePlates.args.blackList = {
	name = "BlackList",
	type = "group",
	order = 1,
	args = {},	
}

E.GUI.args.NamePlates.args.blackList.args.create = {
	name = "",
	type = "group",
	order = 1,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.blackList.args.settings = {
	name = "",
	type = "group",
	order = 2,
	embend = true,
	args = {},	
}

E.GUI.args.NamePlates.args.blackList.args.create.args.spellid = {
	name = "Name",
	type = "editbox",
	order = 1,
	set = function(self, value)
		if value and #value > 0 then
			if not E.db.nameplates.namelist[value] then
				E.db.nameplates.namelist[value] = true
			end			
			selectedname = value
		end
	end,
	get = function(self)
		return ''
	end,

}


E.GUI.args.NamePlates.args.blackList.args.create.args.namelist = {
	name = "Select spell",
	type = "dropdown",
	order = 2,
	values = function()
		local t = {}
		
		for spellname,params in pairs(E.db.nameplates.blacklist) do		
			t[spellname] = NP:SpellString(params.spellID)
		end
		
		return t
	end,
	set = function(self, value)			
		selectedspellbl = value
	end,
	get = function(self)
		return selectedspellbl
	end,		
}


E.GUI.args.NamePlates.args.blackList.args.settings.args.enable = {
	name = "Enable",
	type = "toggle",
	order = 4,
	set = function(self, value)
		if selectedspellbl then
			E.db.nameplates.blacklist[selectedspellbl].enable = not E.db.nameplates.blacklist[selectedspellbl].enable
		end
	end,
	get = function(self)
		if selectedspellbl then
			return E.db.nameplates.blacklist[selectedspellbl].enable
		else
			return false
		end
	end,	
}

E.GUI.args.NamePlates.args.blackList.args.settings.args.spellID = {
	name = "SpellID",
	type = "editbox",
	order = 4,
	set = function(self, value)
		local num = tonumber(value)
		
		if selectedspellbl and num then
			E.db.nameplates.blacklist[selectedspellbl].spellID = num
		end
	end,
	get = function(self)
		if selectedspellbl then
			return E.db.nameplates.blacklist[selectedspellbl].spellID or ''
		else
			return ''
		end
	end,	
}

E.GUI.args.NamePlates.args.blackList.args.settings.args.checkID = {
	name = "CheckID",
	type = "toggle",
	order = 4,
	set = function(self, value)
		if selectedspellbl then
			E.db.nameplates.blacklist[selectedspellbl].checkID = not E.db.nameplates.blacklist[selectedspellbl].checkID
		end
	end,
	get = function(self)
		if selectedspellbl then
			return E.db.nameplates.blacklist[selectedspellbl].checkID or false
		else
			return false
		end
	end,	
}

do
	local hidegametooltip = CreateFrame("Frame")
	hidegametooltip:Hide()
	local gametooltip = CreateFrame("GameTooltip", "KFDjkfsjdfkljsdfshdfkjhsdfGameToolTip", hidegametooltip, 'GameToolTipTemplate')
	gametooltip:SetScript('OnTooltipAddMoney', function()end)
	gametooltip:SetScript('OnTooltipCleared', function()end)
	gametooltip:SetScript('OnHide', function()end)
	gametooltip:SetScript('OnTooltipSetDefaultAnchor',function()end)
	gametooltip:SetOwner(hidegametooltip,"ANCHOR_NONE")
	
	local match = string.match
	local lower = string.lower
	--SetUnitAura
	function E.GetCustomTooltip()	
		return gametooltip
	end
	
	function E.CheckForContainTooltip(tooltip, str)
	
		if not tooltip or not str then return end
		str = lower(str)
		
		for i = 1, tooltip:NumLines() do		
			local left = _G[tooltip:GetName().."TextLeft"..i]:GetText()
			local right = _G[tooltip:GetName().."TextRight"..i]:GetText()
			
			if left then
				local result = match(lower(left), str)
				if result then
					return result, left
				end
			end
			
			if right then
				local result = match(lower(right), str)
				if result then
					return result, right
				end
			end
		end
		
		return false
	end
	
	function E.GetAuraControllType(tooltip)
		if not tooltip then return end
		
		local text = _G[tooltip:GetName().."TextLeft2"]:GetText()
	--	local text2 = _G[tooltip:GetName().."TextLeft3"]:GetText()
		
		local result

		if text then 
			if match(lower(text), 'оглушение.$') then 
				result = 'STUN'
			elseif match(lower(text), 'дезориентация.$') then
				result = 'DEZO'
			elseif match(lower(text), 'немота.$') then
				result = 'SILENT'
			elseif match(lower(text), 'паралич.$') then
				result = 'PARA'
			end
		end
		
		return result, text
	end
end

E.spellContol = {
	[118905] = "STUN",
	[221792] = "STUN",
	[1833] = "STUN",
	[223911] = "STUN",
	[226943] = "STUN",
	[22703] = "STUN",
	[91800] = "STUN",
	[200200] = "STUN",
	[132168] = "STUN",
	[224729] = "DEZO",
	[132169] = "STUN",
	[15487] = "SILENT",
	[232055] = "STUN",
	[207171] = "STUN",
	[1330] = "SILENT",
	[211881] = "STUN",
	[8122] = "DEZO",
	[222897] = "STUN",
	[179057] = "STUN",
	[163505] = "STUN",
	[212337] = "STUN",
	[171017] = "STUN",
	[91797] = "STUN",
	[207167] = "DEZO",
	[30283] = "STUN",
	[200166] = "STUN",
	[105421] = "DEZO",
	[196958] = "STUN",
	[5211] = "STUN",
	[205290] = "STUN",
	[99] = "PARA",
	[1776] = "PARA",
	[200196] = "PARA",
	[201070] = "STUN",
	[81261] = "SILENT",
	[207165] = "STUN",
	[118699] = "DEZO",
	[207685] = "DEZO",
	[51514] = "PARA",
	[5246] = "DEZO",
	[77505] = "STUN",
	[853] = "STUN",
	[204399] = "STUN",
	[212332] = "STUN",
	[117526] = "STUN",
	[408] = "STUN",
	[224074] = "STUN",
	[221562] = "STUN",
	[31935] = "SILENT",
	[214459] = "SILENT",
	[222783] = "SILENT",
	[119381] = "STUN",
	[199804] = "STUN",
	[2094] = "DEZO",
	[205238] = "STUN",
	[204490] = "SILENT",
	[214203] = "STUN",
	[24394] = "STUN",
	[6770] = "PARA",
}
	
E.StunList = {}
E.StunDRs = {}
E.StunTimer = {}

NP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

function NP:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, hideCaster,
			srcGUID, srcName, srcFlags, srcFlags2,
			dstGUID, dstName, dstFlags, dstFlags2,
			spellID, spellName, spellSchool = ...
			
	if ( eventType == 'SPELL_AURA_APPLIED' or eventType == 'SPELL_AURA_REFRESH' ) and E.spellContol[spellID] == 'STUN' then			
		if not E.StunTimer[dstGUID] or E.StunTimer[dstGUID] < GetTime() then
			E.StunTimer[dstGUID] = GetTime()+18
			E.StunDRs[dstGUID] = 1
		end
		
		if E.StunTimer[dstGUID] then
			E.StunDRs[dstGUID] = E.StunDRs[dstGUID] * 0.5
		end
		
		
		for _, frame in pairs(C_NamePlate.GetNamePlates()) do	
			if frame.AleaNP.unit and frame.AleaNP.guid == dstGUID then
				frame.AleaNP:UpdateDRs('Stun')
			end
		end
	end
end
